unit AdminServerHealthLineGraph;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, 
  Dialogs, StdCtrls, ExtCtrls, PNGImage, AuxFuncs, TntForms, TntExtCtrls,
  TntStdCtrls, myx_admin_public_interface;

type
  THealthGraph = class;

  TAdminServerHealthLineGraphFrame = class(TTntFrame)
    QueryCacheHitrateBarPnl: TTntPanel;
    BarImg: TTntImage;
    BarLbl: TTntLabel;
    BarValueImg: TTntImage;
    QryCacheHitratePnl: TTntPanel;
    HealthGraphImg: TTntImage;
    QryCacheTotalSizeLbl: TTntLabel;
    SpacerPnl: TTntPanel;
    DetailPnl: TTntPanel;
    DetailBG1Img: TTntImage;
    DetailBG2Img: TTntImage;
    DetailBG3Img: TTntImage;
    DetailBG4Img: TTntImage;
    CurrentCaptionLbl: TTntLabel;
    CurrentLbl: TTntLabel;
    MinCaptionLbl: TTntLabel;
    MinLbl: TTntLabel;
    MaxLbl: TTntLabel;
    MaxCaptionLbl: TTntLabel;
    AvgCaptionLbl: TTntLabel;
    AvgLbl: TTntLabel;
    PeakLeftShape: TTntShape;
    PeakRightShape: TTntShape;

    constructor Create(Owner: TComponent;
      graph: TMYX_HEALTH_GRAPH;
      BarCaption: WideString = 'Hitrate';
      MinValue: Double = 0; MaxValue: Double = 100;
      ValueUnit: MYX_HEALTH_GRAPH_VALUE_UNIT = MYX_HGVU_COUNT); reintroduce;
    destructor Destroy; override;

    procedure SetValue(value: Double);
    procedure SetMaxValue(value: Double);
    procedure FrameResize(Sender: TObject);
    procedure SetBarCaption(value: double);
  private
    { Private declarations }
    BarCaption: WideString;
    DetailBGPNGImg: TPNGObject;
    graph: TMYX_HEALTH_GRAPH;
    ValueUnit: MYX_HEALTH_GRAPH_VALUE_UNIT;
    BarPeak, BarPeakResetCounter: integer;
  public
    { Public declarations }
    ProgressGraph: THealthGraph;
  end;

  THealthGraph = class(TBitmap)
    constructor Create(width, height: integer;
      MinValue, MaxValue: Double); reintroduce; overload;

    function MoveToNextValue(value: Double): integer;
    procedure DrawProgressGraphTo(theCanvas: TCanvas; x, y, w, h: integer);
    procedure SetNewMaxMin(MinValue, MaxValue: Double);
    procedure ClearGraph;
  private
    CurrentPos: integer;
    Values: Array[0..1280] of Double;
    MinValue, MaxValue: Double;
    CurrentValue: integer;
    ValuesWraped: Boolean;

    TotalValSum: double;
    NumberOfVal: integer;
    CurrentMinVal: double;
    CurrentMaxVal: double;
  public
    RefreshCounter: integer;
  end;

implementation

uses ApplicationDataModule;

{$R *.dfm}

constructor THealthGraph.Create(width, height: integer;
  MinValue, MaxValue: Double);
begin
  inherited Create;

  self.Width:=width+2;
  self.Height:=height;

  self.MinValue:=MinValue;
  self.MaxValue:=MaxValue;
  if(MaxValue=0)then
    self.MaxValue:=0.1;

  refreshCounter:=0;
  currentPos:=0;

  CurrentValue:=-1;
  ValuesWraped:=False;

  TotalValSum:=0;
  NumberOfVal:=0;
  CurrentMinVal:=0;
  CurrentMaxVal:=0;

  ClearGraph;
end;

function THealthGraph.MoveToNextValue(value: Double): integer;
var ypos: integer;
begin
  inc(CurrentValue);
  if(CurrentValue>=1280)then
  begin
    CurrentValue:=0;
    ValuesWraped:=True;
  end;

  //dummy value if MinValue-1 was submitted
  if(value=MinValue-1)then
    value:=Round(currentPos+Random*40-20);
  if(value<MinValue)then
    value:=MinValue
  else if(value>MaxValue)then
    value:=MaxValue;

  Values[CurrentValue]:=value;

  Canvas.CopyRect(Rect(0, 0, Width-3, Height-1), Canvas,
    Rect(2, 0, Width-1, Height-1));

  inc(RefreshCounter);
  if(RefreshCounter mod 6=0)then
  begin
    Canvas.Pen.Color:=$00801000;
    Canvas.MoveTo(Width-4, 0);
    Canvas.LineTo(Width-4, Height-1);
  end;
  //don't let RefreshCounter get too big
  if(RefreshCounter>1280+20)then
    RefreshCounter:=RefreshCounter-12;

  ypos:=Round(((value-MinValue)/MaxValue)*(Height-3));

  Canvas.Pen.Color:=$00FFA000;
  Canvas.MoveTo(Width-7, Height-2-currentPos);
  Canvas.LineTo(Width-5, Height-2-ypos);

  currentPos:=ypos;

  TotalValSum:=TotalValSum+value;
  inc(NumberOfVal);
  if(value<CurrentMinVal)then
    CurrentMinVal:=value;
  if(value>CurrentMaxVal)then
    CurrentMaxVal:=value;

  Result:=Round(((value-MinValue)/MaxValue)*100);
end;

procedure THealthGraph.DrawProgressGraphTo(theCanvas: TCanvas; x, y, w, h: integer);
begin
  theCanvas.CopyRect(Rect(0, 0, w-1, h-1),
    Canvas,
    Rect(Width-w-2, Height-h, Width-2-2, Height-1));
end;

procedure THealthGraph.ClearGraph;
var i: integer;
begin
  Canvas.Brush.Color:=clBlack;
  Canvas.FillRect(Rect(0, 0, self.Width-1, self.Height-1));

  Canvas.Pen.Color:=$00801000;
  for i:=0 to width div 12 do
  begin
    Canvas.MoveTo(width-4-i*12-(RefreshCounter*2) mod 12, 0);
    Canvas.LineTo(width-4-i*12-(RefreshCounter*2) mod 12, self.Height);
  end;
  for i:=1 to height div 12 do
  begin
    Canvas.MoveTo(0, i*12);
    Canvas.LineTo(self.Width, i*12);
  end;
end;

procedure THealthGraph.SetNewMaxMin(MinValue, MaxValue: Double);
var i, xpos, ypos: integer;
begin
  self.MinValue:=MinValue;
  self.MaxValue:=MaxValue;

  {if(CurrentValue=-1)then
    exit;}

  ClearGraph;

  //Redraw scaled values
  currentPos:=Round(((values[CurrentValue]-MinValue)/MaxValue)*(Height-3));

  Canvas.Pen.Color:=$00FFA000;
  Canvas.MoveTo(Width-5, Height-2-currentPos);

  xpos:=Width-7;
  for i:=CurrentValue-1 downto 0 do
  begin
    ypos:=Round(((values[i]-MinValue)/MaxValue)*(Height-3));

    Canvas.LineTo(xpos, Height-2-ypos);

    dec(xpos, 2);
    if(xpos<=0)then
      break;
  end;

  if(ValuesWraped)and(xpos>0)then
    for i:=1280-1 downto CurrentValue+1 do
    begin
      ypos:=Round(((values[i]-MinValue)/MaxValue)*(Height-3));

      Canvas.LineTo(xpos, Height-2-ypos);

      dec(xpos, 2);
      if(xpos<=0)then
        break;
    end;
end;

//------------------------------------------------------------------------------

constructor TAdminServerHealthLineGraphFrame.Create(Owner: TComponent;
  graph: TMYX_HEALTH_GRAPH;
  BarCaption: WideString;
  MinValue: Double; MaxValue: Double;
  ValueUnit: MYX_HEALTH_GRAPH_VALUE_UNIT);
begin
  inherited Create(Owner);

  QryCacheHitratePnl.FullRepaint:=False;

  DetailBGPNGImg:=LoadPNGImageFromResource('health_detailbg', DetailBG1Img);
  DetailBG2Img.Picture.Assign(DetailBGPNGImg);
  DetailBG3Img.Picture.Assign(DetailBGPNGImg);
  DetailBG4Img.Picture.Assign(DetailBGPNGImg);

  ProgressGraph:=THealthGraph.Create(1028, 57,
    MinValue, MaxValue);

  FrameResize(self);

  self.BarCaption:=BarCaption;
  self.graph:=graph;
  self.ValueUnit:=ValueUnit;

  BarPeak:=0;
  //if the PeakLevel shall never be reseted, start with 1
  BarPeakResetCounter:=Ord(Not(ApplicationDM.Options.ResetPeakLevel));

  if(Not(ApplicationDM.Options.UsePeakLevel))then
  begin
    PeakLeftShape.Visible:=False;
    PeakRightShape.Visible:=False;
  end;

  SetBarCaption(0);

  if(Owner.InheritsFrom(TWinControl))then
    Parent:=TWinControl(Owner);
end;

destructor TAdminServerHealthLineGraphFrame.Destroy;
begin
  if(ProgressGraph<>nil)then
    ProgressGraph.Free;

  inherited Destroy;
end;

procedure TAdminServerHealthLineGraphFrame.SetMaxValue(value: Double);
begin
  if(graph.autoextend_max=0)and(ProgressGraph.MaxValue<>value)then
  begin
    ProgressGraph.SetNewMaxMin(ProgressGraph.MinValue,
      value);
  end;
end;

procedure TAdminServerHealthLineGraphFrame.SetValue(value: Double);
var BarValue: integer;
begin
  if(graph.autoextend_max=1)and(ProgressGraph.MaxValue<value)then
  begin
    ProgressGraph.SetNewMaxMin(ProgressGraph.MinValue,
      value);
  end;

  BarValue:=ProgressGraph.MoveToNextValue(value);

  if(ApplicationDM.Options.UsePeakLevel)then
  begin
    if(Not(PeakLeftShape.Visible))then
    begin
      PeakLeftShape.Visible:=True;
      PeakRightShape.Visible:=True;
    end;

    //Peak level
    if(BarValue>BarPeak)or(BarPeakResetCounter=0)then
    begin
      BarPeak:=BarValue;

      PeakLeftShape.Top:=Round(37-BarPeak/100*29);
      PeakRightShape.Top:=Round(37-BarPeak/100*29);
    end;

    if(ApplicationDM.Options.ResetPeakLevel)then
    begin
      inc(BarPeakResetCounter);
      if(BarPeakResetCounter>=ApplicationDM.Options.PeakLevelResetTicks)then
        BarPeakResetCounter:=0;
    end;
  end
  else
  begin
    if(PeakLeftShape.Visible)then
    begin
      PeakLeftShape.Visible:=False;
      PeakRightShape.Visible:=False;
    end;
  end;

  //Set Bar Graph
  BarValueImg.Height:=Round(29-BarValue/100*29);

  if(ValueUnit=MYX_HGVU_PERCENTAGE)then
    SetBarCaption(BarValue)
  else
    SetBarCaption(value);

  CurrentLbl.Caption:=FormatFloat('###,###,##0', value);
  MinLbl.Caption:=FormatFloat('###,###,##0', ProgressGraph.CurrentMinVal);
  MaxLbl.Caption:=FormatFloat('###,###,##0', ProgressGraph.CurrentMaxVal);
  AvgLbl.Caption:=FormatFloat('###,###,##0', ProgressGraph.TotalValSum/ProgressGraph.NumberOfVal);

  ProgressGraph.DrawProgressGraphTo(
    HealthGraphImg.Picture.Bitmap.Canvas,
    0, 0, HealthGraphImg.Width, HealthGraphImg.Height);
  HealthGraphImg.Invalidate;
end;

procedure TAdminServerHealthLineGraphFrame.SetBarCaption(value: double);
var s: WideString;
begin
  case ValueUnit of
    MYX_HGVU_PERCENTAGE:
      if(Trunc(value)=value)then
        s:=BarCaption+' '+FormatFloat('##0', value)+' %'
      else
        s:=BarCaption+' '+FormatFloat('##0.0', value)+' %';
    MYX_HGVU_COUNT:
      if(Trunc(value)=value)then
        s:=BarCaption+' '+FormatFloat('###,###,##0', value)
      else
        s:=BarCaption+' '+FormatFloat('###,###,##0.00', value);
    MYX_HGVU_BYTE:
      s:=FormatFileSize(value);
    MYX_HGVU_SECONDS:
      if(Trunc(value)=value)then
        s:=BarCaption+' '+FormatFloat('###,###,##0', value)+' s'
      else
        s:=BarCaption+' '+FormatFloat('###,###,##0.00', value)+' s';
  end;

  BarLbl.Caption:=s;
end;


procedure TAdminServerHealthLineGraphFrame.FrameResize(Sender: TObject);
begin
  HealthGraphImg.Picture.Bitmap.Width:=HealthGraphImg.Width;
  HealthGraphImg.Picture.Bitmap.Height:=HealthGraphImg.Height;

  ProgressGraph.DrawProgressGraphTo(
    HealthGraphImg.Picture.Bitmap.Canvas,
    0, 0, HealthGraphImg.Width, HealthGraphImg.Height);
end;

end.
