unit MySQLResultSet;

interface

uses
  Windows, Messages, SysUtils, Classes, Contnrs, Controls,
  TntExtCtrls, TntComCtrls, TntClasses, TntStdCtrls, SyncObjs,
  myx_public_interface, gnugettext, auxfuncs, MyxError,
  MySQLConnection, VirtualTrees, Graphics, StrUtils;

type
  TExecuteQueryThread = class;

  RSMessageType = (RSMT_ERROR, RSMT_WARNING);

  IMySQLRSControlInterface = interface;

  TConfirmEvent = function(Sender: TObject; Msg: WideString): Boolean of object;

  TMySQLRS = class(TObject)
    constructor Create;
    destructor Destroy; override;

    procedure ConnectControl(Control: IMySQLRSControlInterface);
    procedure DisconnectControl(Control: IMySQLRSControlInterface);

    procedure ExecuteQuery(sql: WideString = '');

    procedure FreeResultSet;

    procedure DoCurrentRowChanged(Sender: IMySQLRSControlInterface; CurrentRow: Cardinal);
    procedure DoDynamicParamsChanged;
    procedure DoEditStateChanged;
    procedure DoStatusCaptionChanged;

    procedure DoSearch;
    procedure GotoFirstRow;
    procedure GotoLastRow;
    procedure DoEdit;
    procedure DoApplyChanges;
    procedure DoDiscardChanges;

    procedure DoMessagesChanged;
    procedure AddRSMessage(Msg: WideString; MsgType: RSMessageType;
      MsgNr: Integer);
    procedure ClearRSMessages;

    procedure ConnectRS(RS: TMySQLRS);
    procedure DisconnectRS(RS: TMySQLRS);
  protected
    QueryStateMultiReadSync: TMultiReadExclusiveWriteSynchronizer;

    function GetMySQLConn: TMySQLConn;
    procedure SetMySQLConn(MySQLConn: TMySQLConn);

    function GetActive: Boolean;
    procedure SetActive(Active: Boolean);

    procedure Open;
    procedure Close;

    function GetEdited: Boolean;
    procedure SetEdited(Edited: Boolean);

    function GetStatusCaption: WideString;
    procedure SetStatusCaption(StatusCaption: WideString);

    function GetParentRS: TMySQLRS;
    procedure SetParentRS(ParentRS: TMySQLRS);

    function GetRowCount: integer;

    procedure CheckAttributes;

    function GetStopQuery: Boolean;
    procedure SetStopQuery(StopQuery: Boolean);

    function GetQueryExecuting: Boolean;
    procedure SetQueryExecuting(QueryExecuting: Boolean);
  private
    ConnectedControls: TInterfaceList;
    ConnectedRS: TList;

    ExecuteQueryThread: TExecuteQueryThread;

    FMySQLConn: TMySQLConn;
    FSQL: Widestring;

    FStatusCaption: WideString;

    FEdited,
    FActive: Boolean;

    FParentRS: TMySQLRS;

    FCurrentRow: Integer;

    FOnParamChange: TNotifyEvent;

    FOnQueryExecute: TNotifyEvent;
    FOnQueryExecuted: TNotifyEvent;
    FOnQueryStopped: TNotifyEvent;
    FOnQueryStoppedByError: TNotifyEvent;

    FOnConfirmDeletion: TConfirmEvent;

    FStopQuery,
    FQueryExecuting: Boolean;
  public
    WorkWithResultset: TCriticalSection;

    ResultSet: PMYX_RESULTSET;

    GlobalParams,
    LocalParams,
    DynamicParams: TTntStringList;

    ErrorMessages: TObjectList;

    Editable,
    EditingAllowed,
    AtLeastOneRecord,
    FirstRowSelected,
    LastRowSelected: Boolean;
    ErrorOccured: Boolean;
    CurrentQuery: WideString;
    QueryHistroy: TTntStringList;
    CurrentHistoryEntry: integer;

    InsertedRows,
    DeletedRows: Cardinal;

    property MySQLConn: TMySQLConn read GetMySQLConn write SetMySQLConn default nil;
    property Active: Boolean read GetActive write SetActive default False;
    property SQL: Widestring read FSQL write FSQL;
    property Edited: Boolean read GetEdited write SetEdited default False;
    property StatusCaption: WideString read GetStatusCaption write SetStatusCaption;
    property ParentRS: TMySQLRS read GetParentRS write SetParentRS;
    property RowCount: Integer read GetRowCount;

    property OnParamChange: TNotifyEvent read FOnParamChange write FOnParamChange;
    property OnQueryExecute: TNotifyEvent read FOnQueryExecute write FOnQueryExecute;
    property OnQueryExecuted: TNotifyEvent read FOnQueryExecuted write FOnQueryExecuted;
    property OnQueryStopped: TNotifyEvent read FOnQueryStopped write FOnQueryStopped;
    property OnQueryStoppedByError: TNotifyEvent read FOnQueryStoppedByError write FOnQueryStoppedByError;

    property OnConfirmDeletion: TConfirmEvent read FOnConfirmDeletion write FOnConfirmDeletion;

    property StopQuery: Boolean read GetStopQuery write SetStopQuery default False;
    property QueryExecuting: Boolean read GetQueryExecuting write SetQueryExecuting default False;
  end;

  TExecuteQueryThread = class(TThread)
    constructor Create(MySQLRS: TMySQLRS);
    destructor Destroy; override;

    procedure CleanUp;
    procedure BuildGrid;

    procedure ExecutionFinished;
    procedure ExecutionStopped;
    procedure ShowError;
  protected
    procedure Execute; override;
  private
    ThreadPMySQL: Pointer;
    ExceptionMsg: WideString;
  public
    MySQLRS: TMySQLRS;

    Query: WideString;

    current_row_count: Longint;
    previous_row_count: Longint;
    result_set: PMYX_RESULTSET;
  end;

  TRSError = class(TObject)
    constructor Create(Msg: WideString; MsgType: RSMessageType;
      MsgNr: Integer);
  public
    Msg: WideString;
    MsgType: RSMessageType;
    MsgNr: Integer;
  end;

  IMySQLRSControlInterface = interface(IUnknown)
    ['{B46142F1-743F-4BB2-9A00-20C7D485D7AA}']
    procedure ClearValues;

    function GetMySQLRS: TMySQLRS;
    procedure SetMySQLRS(MySQLRS: TMySQLRS);
    function GetControl: TObject;

    procedure DoCurrentRowChanged;
    procedure DoEditStateChanged;
    procedure DoStatusCaptionChanged;
    procedure DoMessagesChanged;

    property MySQLRS: TMySQLRS read GetMySQLRS write SetMySQLRS;
    property Control: TObject read GetControl;
  end;

  function progress_row_fetch(current_row_count: Longint; previous_row_count: Longint; result_set: PMYX_RESULTSET; user_data: Pointer):Integer; cdecl;
  procedure resultset_realloc_before(user_data: Pointer); cdecl;
  procedure resultset_realloc_after(user_data: Pointer); cdecl;

  const IID_IMySQLRSControlInterface: TGUID = '{B46142F1-743F-4BB2-9A00-20C7D485D7AA}';
implementation

uses MySQLResultSetControls, ApplicationDataModule;

constructor TMySQLRS.Create;
begin
  inherited;

  ResultSet:=nil;
  FMySQLConn:=nil;

  FParentRS:=nil;

  ConnectedControls:=TInterfaceList.Create;
  ConnectedRS:=TList.Create;

  QueryHistroy:=TTntStringList.Create;
  CurrentHistoryEntry:=0;
  CurrentQuery:='';

  FCurrentRow:=-1;

  InsertedRows:=0;
  DeletedRows:=0;

  GlobalParams:=nil;
  LocalParams:=TTntStringList.Create;
  DynamicParams:=TTntStringList.Create;

  ErrorMessages:=TObjectList.Create;

  ErrorOccured:=False;

  Editable:=False;
  EditingAllowed:=False;
  FEdited:=False;

  QueryStateMultiReadSync:=TMultiReadExclusiveWriteSynchronizer.Create;
  FActive:=False;
  StopQuery:= False;

  WorkWithResultset:=TCriticalSection.Create;
end;

destructor TMySQLRS.Destroy;
var i: integer;
begin
  //Disconnect from ParentRS
  ParentRS:=nil;

  FreeResultSet;

  //Disconnect Controls
  for i:=0 to ConnectedControls.Count-1 do
    IMySQLRSControlInterface(ConnectedControls[0]).MySQLRS:=nil;

  ErrorMessages.Free;
  QueryHistroy.Free;

  ConnectedControls.Free;
  ConnectedRS.Free;

  LocalParams.Free;
  DynamicParams.Free;

  QueryStateMultiReadSync.Free;

  WorkWithResultset.Free;

  inherited;
end;

procedure TMySQLRS.ConnectControl(Control: IMySQLRSControlInterface);
begin
  if(Not(Supports(Control, IID_IMySQLRSControlInterface)))then
    raise EInOutError.Create(_('Only controls that implement the '+
      'IMySQLRSControlInterface can be connected to the resultset.'));

  if(Not(ConnectedControls.IndexOf(Control)>0))then
    ConnectedControls.Add(Control);
end;

procedure TMySQLRS.DisconnectControl(Control: IMySQLRSControlInterface);
begin
  if(ConnectedControls.IndexOf(Control)>=0)then
    ConnectedControls.Delete(ConnectedControls.IndexOf(Control));
end;

procedure TMySQLRS.FreeResultSet;
var i: integer;
begin
  for i:=0 to ConnectedControls.Count-1 do
    IMySQLRSControlInterface(ConnectedControls[i]).ClearValues;

  if(ResultSet<>nil)then
    myx_query_free_resultset(ResultSet);

  InsertedRows:=0;
  DeletedRows:=0;

  ResultSet:=nil;
end;

function TMySQLRS.GetMySQLConn: TMySQLConn;
begin
  Result:=FMySQLConn;
end;

procedure TMySQLRS.SetMySQLConn(MySQLConn: TMySQLConn);
begin
  FMySQLConn:=MySQLConn;
end;

procedure TMySQLRS.ExecuteQuery(sql: WideString);
var i: integer;
begin
  if(sql<>'')then
    FSQL:=sql;

  if(FSQL='')then
  begin
    ShowModalDialog(_('No SQL Command'),
      _('You tried to execute and empty string. Please type a SQL command into '+
        'the SQL edit field and execute again.'),
        myx_mtError, _('Ok'));
    exit;
  end;

  if(MySQLConn.PMySQL=nil)then
    raise EInOutError.Create(_('MySQL connection not set.'));

  FreeResultSet;

  ClearRSMessages;

  StopQuery:=False;
  QueryExecuting:=True;

  //Clear current values
  for i:=0 to ConnectedControls.Count-1 do
    IMySQLRSControlInterface(ConnectedControls[i]).ClearValues;

  if(Assigned(FOnQueryExecute))then
    FOnQueryExecute(self);

  ExecuteQueryThread:=TExecuteQueryThread.Create(self);
  try
    //Thread will be freed after execution
    ExecuteQueryThread.FreeOnTerminate:=True;

    ExecuteQueryThread.Resume;
  except
    ExecuteQueryThread.Free;
  end;
end;

procedure TMySQLRS.SetActive(Active: Boolean);
begin
  if(Active)then
    Open
  else
    Close;
end;

function TMySQLRS.GetActive: Boolean;
begin
  Result:=FActive;
end;

procedure TMySQLRS.Open;
begin
  FActive:=True;

  ExecuteQuery;
end;

procedure TMySQLRS.Close;
begin
  FreeResultSet;

  FActive:=False;
end;

function TMySQLRS.GetEdited: Boolean;
begin
  Result:=FEdited;
end;

procedure TMySQLRS.SetEdited(Edited: Boolean);
begin
  if(FEdited<>Edited)then
  begin
    FEdited:=Edited;

    DoEditStateChanged;
  end;
end;

procedure TMySQLRS.ConnectRS(RS: TMySQLRS);
begin
  if(ConnectedRS.IndexOf(RS)=-1)then
    ConnectedRS.Add(RS);
end;

procedure TMySQLRS.DisconnectRS(RS: TMySQLRS);
begin
  if(ConnectedRS.IndexOf(RS)<>-1)then
    ConnectedRS.Delete(ConnectedRS.IndexOf(RS));
end;

function TMySQLRS.GetParentRS: TMySQLRS;
begin
  Result:=FParentRS;
end;

procedure TMySQLRS.SetParentRS(ParentRS: TMySQLRS);
begin
  if(FParentRS<>nil)and(FParentRS<>ParentRS)then
    FParentRS.DisconnectRS(self);

  FParentRS:=ParentRS;

  if(FParentRS<>nil)then
    FParentRS.ConnectRS(self);
end;

function TMySQLRS.GetRowCount: integer;
begin
  if(ResultSet<>nil)then
    Result:=ResultSet.rows_num+InsertedRows-DeletedRows
  else
    Result:=0;
end;


procedure TMySQLRS.DoCurrentRowChanged(Sender: IMySQLRSControlInterface;
  CurrentRow: Cardinal);
var i: integer;
  row: PMYX_RS_ROW;
  col: PMYX_RS_COLUMN;
  field: MYX_RS_FIELD;
begin
  CheckAttributes;

  FirstRowSelected:=False;
  if(ResultSet<>nil)then
    if(CurrentRow=0)or((ResultSet.rows_num+InsertedRows=0)and(CurrentRow=1))then
      FirstRowSelected:=True;

  LastRowSelected:=False;
  if(ResultSet<>nil)then
    if((CurrentRow>=ResultSet.rows_num-1+InsertedRows)or
      (ResultSet.rows_num+InsertedRows=0))then
      LastRowSelected:=True;


  if(FCurrentRow=int(CurrentRow))then
    Exit;

  FCurrentRow:=CurrentRow;

  //Set Dynamic Parameters
  DynamicParams.Clear;
  if(FCurrentRow>-1)and(FCurrentRow<Integer(ResultSet.rows_num))then
  begin
    row:=PMYX_RS_ROW(integer(ResultSet.rows)+sizeof(MYX_RS_ROW)*FCurrentRow);

    for i:=0 to ResultSet.columns_num-1 do
    begin
      col:=PMYX_RS_COLUMN(integer(ResultSet.columns)+(sizeof(MYX_RS_COLUMN)*i));
      if(row^.fields<>nil)then
      begin
        field:=(PMYX_RS_FIELD(integer(row^.fields)+(sizeof(MYX_RS_FIELD)*i)))^;
        //only take first 128 chars and replace returns
        DynamicParams.Add(col^.name+'='+
          AnsiReplaceText(
            AnsiReplaceText(
              AnsiReplaceText(
                Copy(field.value, 1, 128),
                #13#10, ''),
              #10, ''),
            #13, ''));
      end
      else
        DynamicParams.Add(col^.name+'=');
    end;
  end;

  for i:=0 to ConnectedControls.Count-1 do
    if(ConnectedControls[i]<>Sender)then
      IMySQLRSControlInterface(ConnectedControls[i]).DoCurrentRowChanged;

  for i:=0 to ConnectedRS.Count-1 do
    TMySQLRS(ConnectedRS[i]).DoDynamicParamsChanged;

  if(Assigned(FOnParamChange))then
    FOnParamChange(self);
end;

procedure TMySQLRS.DoDynamicParamsChanged;
begin
  if(ResultSet<>nil)then
  begin
    //If dynamic params have been changed
    //reexecute query
    if(ResultSet.query.params_num>0)then
    begin
      ExecuteQuery(FSQL);

      if(Assigned(FOnParamChange))then
        FOnParamChange(self);
    end;
  end;
end;

procedure TMySQLRS.DoEditStateChanged;
var i: integer;
begin
  for i:=0 to ConnectedControls.Count-1 do
    IMySQLRSControlInterface(ConnectedControls[i]).DoEditStateChanged;
end;

procedure TMySQLRS.DoStatusCaptionChanged;
var i: integer;
begin
  for i:=0 to ConnectedControls.Count-1 do
    IMySQLRSControlInterface(ConnectedControls[i]).DoStatusCaptionChanged;
end;


function TMySQLRS.GetStatusCaption: WideString;
begin
  Result:=FStatusCaption;
end;

procedure TMySQLRS.SetStatusCaption(StatusCaption: WideString);
begin
  FStatusCaption:=StatusCaption;

  DoStatusCaptionChanged;
end;


procedure TMySQLRS.CheckAttributes;
begin
  if(ResultSet<>nil)then
    Editable:=(ResultSet.editable=1);

  AtLeastOneRecord:=False;
  if(ResultSet<>nil)then
    if(ResultSet.rows_num+InsertedRows>0)then
      AtLeastOneRecord:=True;
end;


procedure TMySQLRS.DoSearch;
var searchtxt: WideString;
begin
  if(ShowModalEditDialog(_('Search Resultset'),
    _('Enter the string you are searching for.'),
    myx_mtEdit, _('Search'+#13#10+'Cancel'),
    True, _('Search for:'), searchtxt)=1)then
  begin
  end;
end;

procedure TMySQLRS.GotoFirstRow;
var i: integer;
  RSGrid: TMySQLRSGrid;
begin
  for i:=0 to ConnectedControls.Count-1 do
  begin
    if((TObject(IMySQLRSControlInterface(ConnectedControls[i]).GetControl) is TMySQLRSGrid))then
    begin
      RSGrid:=TMySQLRSGrid(IMySQLRSControlInterface(ConnectedControls[i]).GetControl);

      if(RSGrid.CanFocus)then
        RSGrid.SetFocus;

      RSGrid.FocusedNode:=RSGrid.GetFirstNoInit;
      RSGrid.ClearSelection;
      RSGrid.Selected[RSGrid.FocusedNode]:=True;
    end;
  end;
end;

procedure TMySQLRS.GotoLastRow;
var i: integer;
  RSGrid: TMySQLRSGrid;
  LastNode: PVirtualNode;
begin
  for i:=0 to ConnectedControls.Count-1 do
  begin
    if((TObject(IMySQLRSControlInterface(ConnectedControls[i]).GetControl) is TMySQLRSGrid))then
    begin
      RSGrid:=TMySQLRSGrid(IMySQLRSControlInterface(ConnectedControls[i]).GetControl);

      if(RSGrid.CanFocus)then
        RSGrid.SetFocus;

      LastNode:=RSGrid.GetLastNoInit;
      if(LastNode<>nil)then
        if(LastNode.PrevSibling<>nil)then
          RSGrid.FocusedNode:=LastNode.PrevSibling
        else
          RSGrid.FocusedNode:=LastNode;
          
      RSGrid.ClearSelection;
      RSGrid.Selected[RSGrid.FocusedNode]:=True;
    end;
  end;
end;

procedure TMySQLRS.DoEdit;
begin
  if(Editable)then
  begin
    if(Not(EditingAllowed))then
    begin
      EditingAllowed:=True;
      DoEditStateChanged;
      //DoDiscardChanges;
    end
    else
    begin
      EditingAllowed:=False;
      DoDiscardChanges;
    end;
  end;
end;

procedure TMySQLRS.DoApplyChanges;
var errors: PMYX_RS_ACTION_ERRORS;
  error: PMYX_RS_ACTION_ERROR;
  i: integer;
  RSGrid: TMySQLRSGrid;
begin
  errors:=myx_query_apply_actions(ResultSet);
  if(errors<>nil)then
  begin
    SetStatusCaption(_('Error while applying changes.'));

    for i:=0 to errors.errors_num-1 do
    begin
      error:=PMYX_RS_ACTION_ERROR(Integer(errors.errors)+sizeof(MYX_RS_ACTION_ERROR)*i);

      AddRSMessage(error.error_text, RSMT_ERROR, error.error);
    end;
  end;

  //Delete Rows from Grids
  for i:=0 to ConnectedControls.Count-1 do
    if((TObject(IMySQLRSControlInterface(ConnectedControls[i]).GetControl) is TMySQLRSGrid))then
    begin
      RSGrid:=TMySQLRSGrid(IMySQLRSControlInterface(ConnectedControls[i]).GetControl);

      RSGrid.ApplyDeleteActions;
    end;

  Edited:=False;
  //EditingAllowed:=False;

  DoEditStateChanged;
end;

procedure TMySQLRS.DoDiscardChanges;
var DoIt: Boolean;
begin
  DoIt:=True;

  if(Assigned(FOnConfirmDeletion))and(Edited)then
    DoIt:=FOnConfirmDeletion(self, _('Do you really want to discard all changes?'));

  if(DoIt)then
  begin
    myx_query_discard_actions(ResultSet);

    Edited:=False;
    //EditingAllowed:=False;

    DoEditStateChanged;
  end;
end;

procedure TMySQLRS.DoMessagesChanged;
var i: integer;
begin
  for i:=0 to ConnectedControls.Count-1 do
    IMySQLRSControlInterface(ConnectedControls[i]).DoMessagesChanged;
end;

procedure TMySQLRS.AddRSMessage(Msg: WideString; MsgType: RSMessageType;
  MsgNr: Integer);
begin
  if(MsgType=RSMT_ERROR)then
    ErrorOccured:=True;

  ErrorMessages.Add(TRSError.Create(Msg, MsgType, MsgNr));

  DoMessagesChanged;
end;

procedure TMySQLRS.ClearRSMessages;
begin
  ErrorMessages.Clear;

  DoMessagesChanged;
end;

function TMySQLRS.GetStopQuery: Boolean;
begin
  QueryStateMultiReadSync.BeginRead;
  try
    Result:=FStopQuery;
  finally
    QueryStateMultiReadSync.EndRead;
  end;
end;

procedure TMySQLRS.SetStopQuery(StopQuery: Boolean);
begin
  QueryStateMultiReadSync.BeginWrite;
  try
    FStopQuery:=StopQuery;
  finally
    QueryStateMultiReadSync.EndWrite;
  end;
end;

function TMySQLRS.GetQueryExecuting: Boolean;
begin
  QueryStateMultiReadSync.BeginRead;
  try
    Result:=FQueryExecuting;
  finally
    QueryStateMultiReadSync.EndRead;
  end;
end;

procedure TMySQLRS.SetQueryExecuting(QueryExecuting: Boolean);
begin
  QueryStateMultiReadSync.BeginWrite;
  try
    FQueryExecuting:=QueryExecuting;
  finally
    QueryStateMultiReadSync.EndWrite;
  end;
end;



// -----------------------------------------------------------------------------

constructor TExecuteQueryThread.Create(MySQLRS: TMySQLRS);
begin
  inherited Create(True);

  self.MySQLRS:=MySQLRS;

  current_row_count:=0;
  previous_row_count:=0;
  result_set:=nil;
  ExceptionMsg:='';

  //When the user is in the middle of a transaction, don't create a new connection
  if(Not(MySQLRS.MySQLConn.InTransaction))then
  begin
    ThreadPMySQL:=myx_mysql_init();
    if(ThreadPMySQL=nil)then
      raise EMyxError.Create(_('Error while allocating memory for MySQL Struct in Query Execute Thread.'));
          

    if(myx_connect_to_instance(
      MySQLRS.MySQLConn.User_Connection.get_record_pointer,
      ThreadPMySQL)<>0)then
      raise EMyxError.Create(_('Query Execute Thread cannot connect to MySQL'));

    if(MySQLRS.MySQLConn.DefaultSchema<>'')and
      (MySQLRS.MySQLConn.DefaultSchema<>MySQLRS.MySQLConn.user_connection.schema)then
        myx_use_schema(ThreadPMySQL, MySQLRS.MySQLConn.DefaultSchema);
  end
  else
    ThreadPMySQL:=MySQLRS.MySQLConn.PMySQL;
end;

destructor TExecuteQueryThread.Destroy;
begin
  CleanUp;

  inherited Destroy;
end;

procedure TExecuteQueryThread.CleanUp;
begin
  if(Not(MySQLRS.MySQLConn.InTransaction))then
    myx_mysql_close(ThreadPMySQL);
end;

procedure TExecuteQueryThread.Execute;
var ErrorCode: MYX_LIB_ERROR;
  Parameters: TMYX_STRINGLIST;
begin
  try
    //If the user is in a transaction, wait till connection is free
    if(MySQLRS.MySQLConn.InTransaction)then
      MySQLRS.MySQLConn.FetchDataThreadCriticalSection.Acquire;

    Query:=MySQLRS.SQL;

    Parameters:=TMYX_STRINGLIST.create;
    try
      Parameters.strings.Text:=MySQLRS.LocalParams.Text;

      //Add dynamic parameters from parent
      if(MySQLRS.FParentRS<>nil)then
        Parameters.strings.Text:=Parameters.strings.Text+#13#10+
        MySQLRS.FParentRS.DynamicParams.Text;

      myx_query_execute(ThreadPMySQL,
        Query, Ord(ApplicationDM.Options.EnforceEditableQueries),
        Parameters.get_record_pointer,
        @ErrorCode, Addr(self), progress_row_fetch,
        resultset_realloc_before,
        resultset_realloc_after);
    finally
      Parameters.Free;
    end;
  finally
    if(MySQLRS.MySQLConn.InTransaction)then
      MySQLRS.MySQLConn.FetchDataThreadCriticalSection.Release;
  end;

  //If this was executed using a cloned MySQL Connection,
  //reassign the original one
  if(Not(MySQLRS.MySQLConn.InTransaction))and(MySQLRS.ResultSet<>nil)then
    MySQLRS.ResultSet.mysql:=MySQLRS.MySQLConn.PMySQL;

  if(ErrorCode=MYX_STOP_EXECUTION)then
    Synchronize(ExecutionStopped)
  else if(ErrorCode=MYX_NO_ERROR)then
    Synchronize(ExecutionFinished)
  else
    Synchronize(ShowError);
end;

function progress_row_fetch(current_row_count: Longint; previous_row_count: Longint; result_set: PMYX_RESULTSET; user_data: Pointer):Integer;
var PSender: ^TExecuteQueryThread;
begin
  PSender:=user_data;
  try
    PSender.current_row_count:=current_row_count;
    PSender.previous_row_count:=previous_row_count;
    PSender.result_set:=result_set;

    PSender.Synchronize(PSender.BuildGrid);

    Result:=Ord(PSender.MySQLRS.StopQuery);
  except
    on x: Exception do
    begin
      PSender.ExceptionMsg:=x.Message;
      PSender.Synchronize(PSender.ShowError);
      Result:=Ord(True);
    end;
  end;
end;

procedure resultset_realloc_before(user_data: Pointer);
var PSender: ^TExecuteQueryThread;
begin
  PSender:=user_data;
  try
    PSender.MySQLRS.WorkWithResultset.Acquire;
  except
    on x: Exception do
    begin
      PSender.MySQLRS.WorkWithResultset.Release;

      PSender.ExceptionMsg:=x.Message;
      PSender.Synchronize(PSender.ShowError);
    end;
  end;
end;

procedure resultset_realloc_after(user_data: Pointer);
var PSender: ^TExecuteQueryThread;
begin
  PSender:=user_data;
  try
    PSender.MySQLRS.WorkWithResultset.Release;
  except
    on x: Exception do
    begin
      PSender.ExceptionMsg:=x.Message;
      PSender.Synchronize(PSender.ShowError);
    end;
  end;
end;

procedure TExecuteQueryThread.BuildGrid;
var j: integer;
  RSGrid: TMySQLRSGrid;
begin
  if(previous_row_count=0)then
    MySQLRS.ResultSet:=result_set;

  for j:=0 to MySQLRS.ConnectedControls.Count-1 do
  begin
    if(TObject(IMySQLRSControlInterface(MySQLRS.ConnectedControls[j]).GetControl).InheritsFrom(TMySQLRSGrid))then
    begin
      RSGrid:=TMySQLRSGrid(IMySQLRSControlInterface(MySQLRS.ConnectedControls[j]).GetControl);

      //If this is the first block of rows, add columns
      if(previous_row_count=0)then
      begin
        RSGrid.MySQLRS.ResultSet:=result_set;

        RSGrid.BuildColumns;
        RSGrid.FocusedColumn:=1;
      end;

      //Add rows
      RSGrid.BuildRows(previous_row_count, current_row_count-1,
        (previous_row_count=0));

      MySQLRS.CheckAttributes;

      MySQLRS.StatusCaption:=
        Format(_('%d rows fetched so far.'), [current_row_count]);
    end;
  end;
end;

procedure TExecuteQueryThread.ExecutionFinished;
var AddToHistroy: Boolean;
  j: integer;
  RSGrid: TMySQLRSGrid;
begin
  if(MySQLRS.ResultSet<>nil)then
    for j:=0 to MySQLRS.ConnectedControls.Count-1 do
    begin
      if(TObject(IMySQLRSControlInterface(MySQLRS.ConnectedControls[j]).GetControl).InheritsFrom(TMySQLRSGrid))then
      begin
        RSGrid:=TMySQLRSGrid(IMySQLRSControlInterface(MySQLRS.ConnectedControls[j]).GetControl);

        RSGrid.BuildRowForAppending;
      end;
    end;


  AddToHistroy:=False;
  if(MySQLRS.QueryHistroy.Count>0)then
  begin
    if(MySQLRS.QueryHistroy[MySQLRS.QueryHistroy.Count-1]<>Query)then
      AddToHistroy:=True;
  end
  else
    AddToHistroy:=True;

  if(AddToHistroy)then
  begin
    MySQLRS.QueryHistroy.Add(Query);
    MySQLRS.CurrentHistoryEntry:=0;
  end;

  MySQLRS.CheckAttributes;

  MySQLRS.SetStatusCaption(
    Format(_('%d rows in set.'), [current_row_count]));

  MySQLRS.StopQuery:=False;
  MySQLRS.QueryExecuting:=False;

  if(Assigned(MySQLRS.OnQueryExecuted))then
    MySQLRS.OnQueryExecuted(MySQLRS);
end;

procedure TExecuteQueryThread.ExecutionStopped;
begin
  MySQLRS.CheckAttributes;

  MySQLRS.SetStatusCaption(
    Format(_('Query aborted. %d rows fetched so far.'), [current_row_count]));

  MySQLRS.StopQuery:=False;
  MySQLRS.QueryExecuting:=False;

  if(Assigned(MySQLRS.OnQueryStopped))then
    MySQLRS.OnQueryStopped(MySQLRS);
end;

procedure TExecuteQueryThread.ShowError;
begin
  MySQLRS.FreeResultSet;

  //Check if ExceptionMsg is set
  if(ExceptionMsg<>'')then
    MySQLRS.SetStatusCaption(ExceptionMsg)
  else
    MySQLRS.SetStatusCaption(
      _('The query could not be executed.'));

  //Check if there actually was a MySQL Error
  if(myx_mysql_errno(ThreadPMySQL)>0)then
    MySQLRS.AddRSMessage(myx_mysql_error(ThreadPMySQL), RSMT_ERROR,
      myx_mysql_errno(ThreadPMySQL));

  MySQLRS.StopQuery:=False;
  MySQLRS.QueryExecuting:=False;

  if(Assigned(MySQLRS.OnQueryStoppedByError))then
    MySQLRS.OnQueryStoppedByError(MySQLRS);
end;

// ---------------------------------------------------------------

constructor TRSError.Create(Msg: WideString; MsgType: RSMessageType;
  MsgNr: Integer);
begin
  self.Msg:=Msg;
  self.MsgType:=MsgType;
  self.MsgNr:=MsgNr;
end;

end.
