unit MySQLResultSetControls;

interface

uses
  Windows, Messages, SysUtils, Classes, Contnrs, Graphics, Controls,
  VirtualTrees, TntExtCtrls, TntComCtrls, TntClasses, TntStdCtrls, TntForms,
  myx_public_interface, gnugettext, auxfuncs, PNGImage, Forms,
  MySQLConnection, MySQLResultSet, AuxLists, TntMenus, Clipbrd,
  Options, StrUtils;

const
  WM_EditCurrentCell = WM_USER+300;

type
  TMySQLRSPanel = class;
  TMySQLRSGrid = class;
  TMySQLRSNav = class;
  TMySQLRSErrorGrid = class;

  TRSTabSheet = class(TTntPanel)
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  protected
    function GetActiveRSPanel: TMySQLRSPanel;
    procedure SetActiveRSPanel(ActiveRSPanel: TMySQLRSPanel);

    function GetParallelGridScroll: Boolean;
    procedure SetParallelGridScroll(ParallelGridScroll: Boolean);
  private
    FActiveRSPanel: TMySQLRSPanel;

    FOnBeforeActiveRSPanelChanged,
    FOnActivateRSPanel: TNotifyEvent;

    FParallelGridScroll: Boolean;

    SplitDirection: Integer;

    Splitters: TList;
  public
    RSPanels: TList;
    GridsAreScrolling: Boolean;

    function AddRSPanel(MySQLConn: TMySQLConn;
      SplitVertically: Boolean = True): TMySQLRSPanel;

    procedure DeleteRSPanel(RSPanel: TMySQLRSPanel);

    property ActiveRSPanel: TMySQLRSPanel read GetActiveRSPanel write SetActiveRSPanel;
    property OnBeforeActiveRSPanelChanged: TNotifyEvent read FOnBeforeActiveRSPanelChanged write FOnBeforeActiveRSPanelChanged;
    property OnActivateRSPanel: TNotifyEvent read FOnActivateRSPanel write FOnActivateRSPanel;
    property ParallelGridScroll: Boolean read GetParallelGridScroll write SetParallelGridScroll default False;
  end;

  TMySQLRSPanel = class(TTntPanel)
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure CreateMySQLRS(MySQLConn: TMySQLConn);
  protected
    function GetMySQLRS: TMySQLRS;
    procedure SetMySQLRS(MySQLRS: TMySQLRS);

    function GetActive: Boolean;
    procedure SetActive(Active: Boolean);

    procedure DoChildControlEnter(Sender: TObject);
  private
    FMySQLRS: TMySQLRS;

    FActive: Boolean;
  public
    RSGrid: TMySQLRSGrid;
    RSNav: TMySQLRSNav;
    RSErrorGrid: TMySQLRSErrorGrid;
  published
    property MySQLRS: TMySQLRS read GetMySQLRS write SetMySQLRS default nil;
    property Active: Boolean read GetActive write SetActive;
  end;

  TMySQLRSGrid = class(TVirtualStringTree, IMySQLRSControlInterface)
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure ClearValues;

    procedure BuildColumns;
    procedure BuildRows(StartRow: Integer; EndRow: Integer;
      SetFocusOnFirstNode: Boolean = False);
    procedure BuildRowForAppending;
  private
    FMySQLRS: TMySQLRS;

    FActive: Boolean;

    RowColor: TColor;

    NodesWithActionList,
    NodesWithDeleteActionList: TList;
  protected
    //Interface functions
    function GetMySQLRS: TMySQLRS;
    procedure SetMySQLRS(MySQLRS: TMySQLRS);
    function GetControl: TObject;

    function GetActive: Boolean;
    procedure SetActive(Active: Boolean);

    procedure DoCurrentRowChanged;
    procedure DoEditStateChanged;
    procedure DoStatusCaptionChanged;
    procedure DoMessagesChanged;
    //End Interface functions

    function GetNodeRSAction(Node: PVirtualNode; Column: integer;
      IgnoreDeleteActions: Boolean = False): PMYX_RS_ACTION;
    function GetNodeRSRow(Node: PVirtualNode): PMYX_RS_ROW;
    function GetNodeRSField(Node: PVirtualNode; Column: integer): PMYX_RS_FIELD;
    function GetNodeRSText(Node: PVirtualNode; Column: integer): WideString;

    procedure DoGetText(Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var Text: WideString); override;
    procedure DoPaintText(Node: PVirtualNode; const Canvas: TCanvas; Column: TColumnIndex;
      TextType: TVSTTextType); override;
    function DoFocusChanging(OldNode, NewNode: PVirtualNode; OldColumn,
      NewColumn: TColumnIndex): Boolean; override;
    procedure DoAfterCellPaint(Canvas: TCanvas; Node: PVirtualNode; Column:
      TColumnIndex; CellRect: TRect); override;
    procedure DoCanEdit(Node: PVirtualNode; Column: TColumnIndex; var Allowed: Boolean); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure DoFocusChange(Node: PVirtualNode; Column: TColumnIndex); override;

    procedure AddAction(action_type: MYX_RS_ACTION_TYPE; row: Cardinal;
      column: Integer; Node: PVirtualNode;
      new_value: WideString; new_value_length: Integer = -1);
    procedure DoNewText(Node: PVirtualNode; Column: TColumnIndex; Text: WideString); override;
    procedure DoBeforeItemErase(Canvas: TCanvas; Node: PVirtualNode; ItemRect: TRect; var Color: TColor;
      var EraseAction: TItemEraseAction); override;
    procedure DoAfterItemErase(Canvas: TCanvas; Node: PVirtualNode; ItemRect: TRect);  override;

    procedure DoGridScroll(Sender: TBaseVirtualTree; DeltaX, DeltaY: Integer);

    function DoCreateEditor(Node: PVirtualNode; Column: TColumnIndex): IVTEditLink; override;

    procedure DblClick; override;
    procedure WndProc(var Message: TMessage); override;
  published
    //function DoEndEdit: Boolean;

    procedure DeleteSelectedRows;
    procedure ApplyDeleteActions;

    property MySQLRS: TMySQLRS read GetMySQLRS write SetMySQLRS default nil;
    property Active: Boolean read GetActive write SetActive;
  end;

  PRSRowData = ^RSRowData;
  RSRowData = record
    row_index: Cardinal;
    actions: TIntegerList;
  end;

  TMySQLRSNav = class(TTntPanel, IMySQLRSControlInterface)
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure ClearValues;
  private
    FMySQLRS: TMySQLRS;

    FActive: Boolean;

    RSEditPNGImg,
    RSEditDisabledPNGImg,
    RSEditActivePNGImg,
    RSApplyPNGImg,
    RSApplyDisabledPNGImg,
    RSDiscardPNGImg,
    RSDiscardDisabledPNGImg,
    RSFirstPNGImg,
    RSFirstDisabledPNGImg,
    RSLastPNGImg,
    RSLastDisabledPNGImg,
    RSSearchPNGImg,
    RSSearchDisabledPNGImg: TPNGObject;

    RSEditXPos,
    RSApplyXPos,
    RSDiscardXPos,
    RSFirstXPos,
    RSLastXPos,
    RSSearchXPos: integer;
  protected
    //Interface functions
    function GetMySQLRS: TMySQLRS;
    procedure SetMySQLRS(MySQLRS: TMySQLRS);
    function GetControl: TObject;

    procedure DoCurrentRowChanged;
    procedure DoEditStateChanged;
    procedure DoStatusCaptionChanged;
    procedure DoMessagesChanged;
    //End Interface functions

    function GetActive: Boolean;
    procedure SetActive(Active: Boolean);

    procedure Paint; override;
    function PaintEditButton(Caption: WideString; var xpos: integer;
      PNGImg: TPNGObject; DisabledPNGImg: TPNGObject; Enabled: Boolean;
      ActivePNGImg: TPNGObject = nil; Active: Boolean = False): integer;

    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  published
    property MySQLRS: TMySQLRS read GetMySQLRS write SetMySQLRS default nil;
    property Active: Boolean read GetActive write SetActive;
  end;

  TMySQLRSErrorGrid = class(TVirtualStringTree, IMySQLRSControlInterface)
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure ClearValues;
  private
    FMySQLRS: TMySQLRS;

    RSErrorPNGImg,
    RSWarningPNGImg: TPNGObject;

    MsgPopupMenu: TTntPopupMenu;
    CopyMsgMenuItem: TTntMenuItem;
  protected
    //Interface functions
    function GetMySQLRS: TMySQLRS;
    procedure SetMySQLRS(MySQLRS: TMySQLRS);
    function GetControl: TObject;

    procedure DoCurrentRowChanged;
    procedure DoEditStateChanged;
    procedure DoActionsChanged;
    procedure DoStatusCaptionChanged;
    procedure DoMessagesChanged;
    //End Interface functions

    procedure DoGetText(Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var Text: WideString); override;
    procedure DoAfterCellPaint(Canvas: TCanvas; Node: PVirtualNode; Column:
      TColumnIndex; CellRect: TRect); override;

    procedure CopyMsgText(Sender: TObject);
  published
    property MySQLRS: TMySQLRS read GetMySQLRS write SetMySQLRS default nil;
  end;

  PRSMsg = ^TRSMsg;
  TRSMsg = record
    MsgIndex: Integer;
  end;

  TRSFieldEditLink = class;

  TRSStringEdit = class(TTntEdit)
  private
    FRefLink: IVTEditLink;
    FLink: TRSFieldEditLink;
    procedure CMAutoAdjust(var Message: TMessage); message CM_AUTOADJUST;
    procedure CMExit(var Message: TMessage); message CM_EXIT;
    procedure CMRelease(var Message: TMessage); message CM_RELEASE;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
    procedure WMChar(var Message: TWMChar); message WM_CHAR;
    procedure WMDestroy(var Message: TWMDestroy); message WM_DESTROY;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
    procedure WMKeyDown(var Message: TWMKeyDown); message WM_KEYDOWN;
  protected
    procedure AutoAdjustSize;
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(Link: TRSFieldEditLink); reintroduce;

    procedure Release; virtual;

    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property CharCase;
    property HideSelection;
    property MaxLength;
    property OEMConvert;
    property PasswordChar;
  end;

  TRSFieldEditLink = class(TInterfacedObject, IVTEditLink)
  private
    FEdit: TRSStringEdit;
    FTree: TMySQLRSGrid; // A back reference to the tree calling.
    FNode: PVirtualNode;             // The node to be edited.
    FColumn: TColumnIndex;           // The column of the node.
    FAlignment: TAlignment;
    FTextBounds: TRect;              // Smallest rectangle around the text.
    FStopping: Boolean;              // Set to True when the edit link requests stopping the edit action.
    procedure SetEdit(const Value: TRSStringEdit);
  public
    constructor Create;
    destructor Destroy; override;

    function BeginEdit: Boolean; virtual; stdcall;
    function CancelEdit: Boolean; virtual; stdcall;
    property Edit: TRSStringEdit read FEdit write SetEdit;
    function EndEdit: Boolean; virtual; stdcall;
    function GetBounds: TRect; virtual; stdcall;
    function PrepareEdit(Tree: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex): Boolean; virtual; stdcall;
    procedure ProcessMessage(var Message: TMessage); virtual; stdcall;
    procedure SetBounds(R: TRect); virtual; stdcall;
  end;

implementation

constructor TRSTabSheet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Parent:=TWinControl(AOwner);
  BevelOuter:=bvNone;
  Caption:='';
  Width:=650;
  Height:=500;
  SplitDirection:=-1;

  RSPanels:=TList.Create;
  Splitters:=TList.Create;
end;

destructor TRSTabSheet.Destroy;
begin
  RSPanels.Free;
  Splitters.Free;

  inherited;
end;

function TRSTabSheet.AddRSPanel(MySQLConn: TMySQLConn;
  SplitVertically: Boolean): TMySQLRSPanel;
var RSPanel, MasterRSPanel: TMySQLRSPanel;
  Splitter: TTntSplitter;
  ParentForm: TCustomForm;
begin
  if(RSPanels.Count>=1)then
    if(SplitDirection>-1)and(Ord(SplitVertically)<>SplitDirection)then
    begin
      ShowModalDialog(_('Error'),
        _('You can split the panel only in one direction.'),
        myx_mtError, 'OK');

      Result:=nil;

      Exit;
    end;

  RSPanel:=TMySQLRSPanel.Create(self);
  RSPanel.Parent:=self;
  RSPanel.CreateMySQLRS(MySQLConn);

  RSPanels.Add(RSPanel);

  if(RSPanels.Count>1)then
  begin
    SplitDirection:=Ord(SplitVertically);

    //connect this new ResultSet to the prior one
    MasterRSPanel:=TMySQLRSPanel(RSPanels[RSPanels.Count-2]);
    RSPanel.MySQLRS.ParentRS:=MasterRSPanel.MySQLRS;

    //Add events from ParentRS
    RSPanel.MySQLRS.OnParamChange:=MasterRSPanel.MySQLRS.OnParamChange;
    RSPanel.MySQLRS.OnQueryExecute:=MasterRSPanel.MySQLRS.OnQueryExecute;
    RSPanel.MySQLRS.OnQueryExecuted:=MasterRSPanel.MySQLRS.OnQueryExecuted;
    RSPanel.MySQLRS.OnQueryStopped:=MasterRSPanel.MySQLRS.OnQueryStopped;
    RSPanel.MySQLRS.OnQueryStoppedByError:=MasterRSPanel.MySQLRS.OnQueryStoppedByError;
    RSPanel.MySQLRS.OnConfirmDeletion:=MasterRSPanel.MySQLRS.OnConfirmDeletion;
    RSPanel.RSGrid.PopupMenu:=MasterRSPanel.RSGrid.PopupMenu;

    Splitter:=TTntSplitter.Create(self);
    Splitter.Parent:=self;

    Splitters.Add(Splitter);

    if(SplitVertically)then
    begin
      RSPanel.Height:=MasterRSPanel.Height div 2;
      MasterRSPanel.Height:=MasterRSPanel.Height div 2;

      RSPanel.Align:=alBottom;
      Splitter.Align:=alBottom;

      Splitter.Height:=9;
      Splitter.Top:=MasterRSPanel.Top+MasterRSPanel.Height+10;
      RSPanel.Top:=MasterRSPanel.Top+MasterRSPanel.Height+10;
    end
    else
    begin
      RSPanel.Width:=MasterRSPanel.Width div 2;
      MasterRSPanel.Width:=MasterRSPanel.Width div 2;

      RSPanel.Align:=alRight;
      Splitter.Align:=alRight;

      Splitter.Width:=11;
      Splitter.Left:=MasterRSPanel.Left+MasterRSPanel.Width+12;
      RSPanel.Left:=MasterRSPanel.Left+MasterRSPanel.Width+12;
    end;
  end
  else
    RSPanel.Align:=alClient;

  ActiveRSPanel:=RSPanel;

  //Set Focus to new RSPanel
  ParentForm:=GetParentForm(RSPanel);
  if(ParentForm<>nil)then
    if(ParentForm.Visible and ParentForm.Enabled)then
    RSPanel.SetFocus;
    
  RSPanel.Active:=True;

  Result:=RSPanel;
end;

procedure TRSTabSheet.DeleteRSPanel(RSPanel: TMySQLRSPanel);
var Index: integer;
  Splitter: TTntSplitter;
begin
  if(RSPanels.Count>1)then
  begin
    Index:=RSPanels.IndexOf(RSPanel);

    if(Index>0)then
    begin
      Splitter:=Splitters[Index-1];
      Splitters.Delete(Index-1);
      Splitter.Free;

      //Assign RSParent from one before
      if(Index<RSPanels.Count-1)then
        TMySQLRSPanel(RSPanels[Index+1]).MySQLRS.ParentRS:=
          TMySQLRSPanel(RSPanels[Index-1]).MySQLRS;

      RSPanels.Delete(Index);
      RSPanel.Free;

      FActiveRSPanel:=nil;
      ActiveRSPanel:=RSPanels[Index-1];
      ActiveRSPanel.Active:=True;
    end
    else
    begin
      Splitter:=Splitters[0];
      Splitters.Delete(0);
      Splitter.Free;

      TMySQLRSPanel(RSPanels[1]).MySQLRS.ParentRS:=nil;

      RSPanels.Delete(0);
      RSPanel.Free;

      FActiveRSPanel:=nil;
      ActiveRSPanel:=RSPanels[0];
      ActiveRSPanel.Align:=alClient;
      ActiveRSPanel.Active:=True;
    end;
  end;

  if(RSPanels.Count<=1)then
    SplitDirection:=-1;
end;

function TRSTabSheet.GetActiveRSPanel: TMySQLRSPanel;
begin
  Result:=FActiveRSPanel;
end;

procedure TRSTabSheet.SetActiveRSPanel(ActiveRSPanel: TMySQLRSPanel);
var i: integer;
begin
  if(FActiveRSPanel<>ActiveRSPanel)then
  begin
    if(Assigned(FOnBeforeActiveRSPanelChanged))then
      FOnBeforeActiveRSPanelChanged(FActiveRSPanel);

    FActiveRSPanel:=ActiveRSPanel;

    for i:=0 to RSPanels.Count-1 do
      if(RSPanels[i]<>ActiveRSPanel)then
        TMySQLRSPanel(RSPanels[i]).Active:=False;

    {if(Assigned(FActiveRSPanel.MySQLRS.OnParamChange))then
      FActiveRSPanel.MySQLRS.OnParamChange(ActiveRSPanel.MySQLRS);}

    if(Assigned(FOnActivateRSPanel))then
      FOnActivateRSPanel(FActiveRSPanel);
  end;
end;

function TRSTabSheet.GetParallelGridScroll: Boolean;
begin
  Result:=FParallelGridScroll;
end;

procedure TRSTabSheet.SetParallelGridScroll(ParallelGridScroll: Boolean);
var i: integer;
begin
  if(FParallelGridScroll)and(Not(ParallelGridScroll))then
  begin
    for i:=0 to RSPanels.Count-1 do
      TMySQLRSPanel(RSPanels[i]).RSGrid.OnScroll:=nil;
  end;

  FParallelGridScroll:=ParallelGridScroll;

  if(FParallelGridScroll)then
  begin
    for i:=0 to RSPanels.Count-1 do
      TMySQLRSPanel(RSPanels[i]).RSGrid.OnScroll:=
        TMySQLRSPanel(RSPanels[i]).RSGrid.DoGridScroll;
  end;
end;


// -----------------------------------------------------------------

constructor TMySQLRSPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  BevelOuter:=bvNone;
  Caption:='';

  FActive:=True;

  //Create RS Grid
  RSGrid:=TMySQLRSGrid.Create(self);
  RSGrid.Parent:=self;
  RSGrid.Font.Name:=MYXCommonOptions.DataFontName;//'Tahoma';
  RSGrid.Header.Font.Name:='Tahoma';
  RSGrid.Align:=alClient;
  RSGrid.OnEnter:=DoChildControlEnter;

  //Create RS Navigator
  RSNav:=TMySQLRSNav.Create(self);
  RSNav.Parent:=self;
  RSNav.Align:=alBottom;
  RSNav.OnEnter:=DoChildControlEnter;

  //Create RS Error Grid
  RSErrorGrid:=TMySQLRSErrorGrid.Create(self);
  RSErrorGrid.Parent:=self;
  RSErrorGrid.Font.Name:='Tahoma';
  RSErrorGrid.Header.Font.Name:='Tahoma';
  RSErrorGrid.OnEnter:=DoChildControlEnter;
end;

destructor TMySQLRSPanel.Destroy;
begin
  if(FMySQLRS<>nil)then
    FMySQLRS.Free;

  inherited;
end;

function TMySQLRSPanel.GetMySQLRS: TMySQLRS;
begin
  Result:=FMySQLRS;
end;

procedure TMySQLRSPanel.SetMySQLRS(MySQLRS: TMySQLRS);
begin
  FMySQLRS:=MySQLRS;
  RSGrid.MySQLRS:=MySQLRS;
  RSNav.MySQLRS:=MySQLRS;
  RSErrorGrid.MySQLRS:=MySQLRS;
end;

procedure TMySQLRSPanel.CreateMySQLRS(MySQLConn: TMySQLConn);
begin
  FMySQLRS:=TMySQLRS.Create;
  FMySQLRS.MySQLConn:=MySQLConn;

  RSGrid.MySQLRS:=MySQLRS;
  RSNav.MySQLRS:=MySQLRS;
  RSErrorGrid.MySQLRS:=MySQLRS;
end;

function TMySQLRSPanel.GetActive: Boolean;
begin
  Result:=FActive;
end;

procedure TMySQLRSPanel.SetActive(Active: Boolean);
begin
  FActive:=Active;

  RSGrid.Active:=FActive;
  RSNav.Active:=FActive;

  //Deactivate other RSPanels if this one got active
  if(FActive)and(Parent.InheritsFrom(TRSTabSheet))then
    TRSTabSheet(Parent).ActiveRSPanel:=self;


  //Trigger OnParamChange
  {if(FMySQLRS<>nil)then
    if(Assigned(FMySQLRS.OnParamChange))then
      FMySQLRS.OnParamChange(self);}
end;

procedure TMySQLRSPanel.DoChildControlEnter(Sender: TObject);
begin
  Active:=True;
end;

// -----------------------------------------------------------------------------

constructor TMySQLRSGrid.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  NodeDataSize:=SizeOf(RSRowData);

  Header.Options:=Header.Options+[hoDblClickResize, hoVisible];
  LineStyle:=lsSolid;
  ParentBackground:=False;
  TreeOptions.MiscOptions:=TreeOptions.MiscOptions+
    [toEditable, toGridExtensions]-[toToggleOnDblClick];
  TreeOptions.PaintOptions:=TreeOptions.PaintOptions+
    [toShowHorzGridLines, toShowVertGridLines]-
    [toShowRoot, toShowTreeLines, toShowDropmark];
  TreeOptions.SelectionOptions:=TreeOptions.SelectionOptions+
    [toExtendedFocus, toRightClickSelect, toMultiSelect];
  TreeOptions.StringOptions:=TreeOptions.StringOptions-
    [toSaveCaptions];
  WantTabs:=True;

  FActive:=True;

  NodesWithActionList:=TList.Create;
  NodesWithDeleteActionList:=TList.Create;
end;

destructor TMySQLRSGrid.Destroy;
var i: integer;
  NodeData: PRSRowData;
begin
  MySQLRS:=nil;

  for i:=0 to NodesWithActionList.Count-1 do
  begin
    NodeData:=GetNodeData(PVirtualNode(NodesWithActionList[i]));
    NodeData.actions.Free;
  end;

  NodesWithActionList.Free;
  NodesWithDeleteActionList.Free;

  inherited;
end;

procedure TMySQLRSGrid.ClearValues;
begin
  Clear;
  Header.Columns.Clear;

  NodesWithActionList.Clear;
  NodesWithDeleteActionList.Clear;
end;

function TMySQLRSGrid.GetMySQLRS: TMySQLRS;
begin
  Result:=FMySQLRS;
end;

procedure TMySQLRSGrid.SetMySQLRS(MySQLRS: TMySQLRS);
begin
  if(FMySQLRS<>nil)and(MySQLRS=nil)then
    FMySQLRS.DisconnectControl(self);

  FMySQLRS:=MySQLRS;

  if(MySQLRS<>nil)then
    MySQLRS.ConnectControl(self);
end;

function TMySQLRSGrid.GetControl: TObject;
begin
  Result:=self;
end;

function TMySQLRSGrid.GetActive: Boolean;
begin
  Result:=FActive;
end;

procedure TMySQLRSGrid.SetActive(Active: Boolean);
begin
  FActive:=Active;

  if(Header.Columns.Count>0)then
  begin
    if(FActive)then
      Header.Columns[0].Color:=clBtnFace
    else
      Header.Columns[0].Color:=$00E0E0E0;
  end;
end;

procedure TMySQLRSGrid.DoCurrentRowChanged;
begin
  //
end;

procedure TMySQLRSGrid.DoEditStateChanged;
var i: integer;
  NodeData: PRSRowData;
  action: PMYX_RS_ACTION;
begin
  //If actions have been removed, remove them from nodes, too
  if(FMySQLRS.ResultSet.actions=nil)then
  begin
    for i:=0 to NodesWithActionList.Count-1 do
    begin
      NodeData:=GetNodeData(PVirtualNode(NodesWithActionList[i]));
      NodeData.actions.Free;
      NodeData.actions:=nil;
    end;

    NodesWithActionList.Clear;
  end
  else
  begin
    i:=0;
    while(i<NodesWithActionList.Count)do
    begin
      NodeData:=GetNodeData(PVirtualNode(NodesWithActionList[i]));

      //Check if last action is MYX_RSA_DISCARDED
      action:=nil;
      if(NodeData.actions<>nil)then
        if(NodeData.actions.Count>0)then
          action:=PMYX_RS_ACTION(integer(MySQLRS.ResultSet.actions.actions)+
            sizeof(MYX_RS_ACTION)*NodeData.actions[NodeData.actions.Count-1]);

      //If it is, remove action from NodeData
      if(action<>nil)then
      begin
        if(action.action=MYX_RSA_DISCARDED)then
        begin
          NodeData.actions.Free;
          NodeData.actions:=nil;

          NodesWithActionList.Delete(i);
        end
        else
          inc(i);
      end
      else
        inc(i);
    end;
  end;

  //changes may be discarded, so redraw
  Invalidate;
end;

procedure TMySQLRSGrid.DoStatusCaptionChanged;
begin
  //
end;

procedure TMySQLRSGrid.DoMessagesChanged;
begin
  //
end;



procedure TMySQLRSGrid.BuildColumns;
var TVColumn: TVirtualTreeColumn;
  col: MYX_RS_COLUMN;
  i: Integer;
begin
  Header.Columns.Clear;

  //Add Indicator
  TVColumn:=Header.Columns.Add;
  TVColumn.Width:=15;
  TVColumn.Color:=clBtnFace;
  TVColumn.Options:=[coVisible];

  FocusedNode:=nil;

  //Add RS headers
  for i:=0 to FMySQLRS.ResultSet.columns_num_to_display-1 do
  begin
    col:=PMYX_RS_COLUMN(integer(FMySQLRS.ResultSet.columns)+sizeof(MYX_RS_COLUMN)*i)^;
    TVColumn:=Header.Columns.Add;
    TVColumn.Text:=col.name;
    if(col.column_type=MYX_RSCT_INTEGER)then
      TVColumn.Width:=76
    else if(col.column_type=MYX_RSCT_STRING)then
      TVColumn.Width:=140
    else if(col.column_type=MYX_RSCT_DATE)then
      TVColumn.Width:=70
    else if(col.column_type=MYX_RSCT_DATETIME)then
      TVColumn.Width:=120
    else
      TVColumn.Width:=60;
  end;
end;

procedure TMySQLRSGrid.BuildRows(StartRow: Integer; EndRow: Integer; SetFocusOnFirstNode: Boolean);
var i: Integer;
  Node: PVirtualNode;
  Nodedata: PRSRowData;
begin
  Node:=nil;
  BeginUpdate;
  try
    for i:=StartRow to EndRow do
    begin
      Node:=AddChild(nil);

      if(Node<>nil)then
      begin
        NodeData:=GetNodeData(Node);
        if(NodeData<>nil)then
        begin
          NodeData.row_index:=i;
          NodeData.actions:=nil;
        end
        else
          raise EInOutError.Create('NodeData=nil');
      end
      else
        raise EInOutError.Create('Node=nil');
    end;
  finally
    EndUpdate;
  end;

  if(SetFocusOnFirstNode)and(Node<>nil)then
  begin
    FocusedNode:=GetFirstNoInit;
    ClearSelection;
    Selected[FocusedNode]:=True;
  end;

  Refresh;
end;

procedure TMySQLRSGrid.BuildRowForAppending;
var Node: PVirtualNode;
  Nodedata: PRSRowData;
begin
  //Only append row if there is a resultset and
  //the resultset is editable
  if(FMySQLRS.ResultSet=nil)or(Not(FMySQLRS.Editable))then
    Exit;

  BeginUpdate;
  try
    Node:=AddChild(nil);
    if(Node<>nil)then
    begin
      NodeData:=GetNodeData(Node);
      if(NodeData<>nil)then
      begin
        NodeData.row_index:=FMySQLRS.ResultSet.rows_num;
        NodeData.actions:=nil;
      end
      else
        raise EInOutError.Create('NodeData=nil');
    end
    else
      raise EInOutError.Create('Node=nil');
  finally
    EndUpdate;
  end;
end;


function TMySQLRSGrid.GetNodeRSAction(Node: PVirtualNode; Column: integer;
  IgnoreDeleteActions: Boolean): PMYX_RS_ACTION;
var NodeData: PRSRowData;
  action: PMYX_RS_ACTION;
  action_column: integer;
  i: integer;
begin
  action:=nil;

  NodeData:=GetNodeData(Node);

  if(NodeData.actions<>nil)then
  begin
    for i:=NodeData.actions.Count-1 downto 0 do
    begin
      action:=PMYX_RS_ACTION(integer(MySQLRS.ResultSet.actions.actions)+sizeof(MYX_RS_ACTION)*NodeData.actions[i]);

      action_column:=(integer(action.column)-integer(MySQLRS.ResultSet.columns)) div sizeof(MYX_RS_COLUMN);

      //Check column of the action
      if(Column-1=action_column)and
        ((Not(IgnoreDeleteActions))or(action.action<>MYX_RSA_DELETE))then
        break
      else if(action_column=0)and(action.action=MYX_RSA_DELETE)and
        (not(IgnoreDeleteActions))then
        break
      else
        action:=nil;
    end;
  end;

  Result:=action;
end;

function TMySQLRSGrid.GetNodeRSRow(Node: PVirtualNode): PMYX_RS_ROW;
var NodeData: PRSRowData;
  row: PMYX_RS_ROW;
begin
  row:=nil;

  NodeData:=GetNodeData(Node);
  if(NodeData<>nil)then
  begin
    //Consider added rows
    if(NodeData.row_index<FMySQLRS.ResultSet.rows_num)then
      row:=PMYX_RS_ROW(integer(MySQLRS.ResultSet.rows)+sizeof(MYX_RS_ROW)*integer(NodeData.row_index));
  end;

  Result:=row;
end;

function TMySQLRSGrid.GetNodeRSField(Node: PVirtualNode; Column: integer): PMYX_RS_FIELD;
var NodeData: PRSRowData;
  row: PMYX_RS_ROW;
  field: PMYX_RS_FIELD;
begin
  field:=nil;

  NodeData:=GetNodeData(Node);
  if(NodeData<>nil)then
  begin
    //Consider added rows
    if(NodeData.row_index<FMySQLRS.ResultSet.rows_num)then
    begin
      row:=PMYX_RS_ROW(integer(MySQLRS.ResultSet.rows)+sizeof(MYX_RS_ROW)*integer(NodeData.row_index));

      //Consider diff rows
      if(row.fields<>nil)then
        field:=PMYX_RS_FIELD(integer(row^.fields)+(sizeof(MYX_RS_FIELD)*(Column-1)));
    end;
  end;

  Result:=field;
end;

function TMySQLRSGrid.GetNodeRSText(Node: PVirtualNode; Column: integer): WideString;
var action: PMYX_RS_ACTION;
  field: PMYX_RS_FIELD;
  s: WideString;
  ValueFromAction: Boolean;
begin
  s:='';

  MySQLRS.WorkWithResultset.Acquire;
  try
    action:=GetNodeRSAction(Node, Column);
    ValueFromAction:=False;

    if(action<>nil)then
      if(action.action<>MYX_RSA_DELETE)then
      begin
        s:=UTF8Decode(action^.new_value);
        ValueFromAction:=True;
      end
      else
      begin
        action:=GetNodeRSAction(Node, Column, True);
        if(action<>nil)then
        begin
          s:=UTF8Decode(action^.new_value);
          ValueFromAction:=True;
        end;
      end;

    if(Not(ValueFromAction))then
    begin
      field:=GetNodeRSField(Node, Column);
      if(field<>nil)then
        if(field.value<>nil)then
          s:=UTF8Decode(field.value);
    end;

    Result:=s;
  finally
    MySQLRS.WorkWithResultset.Release;
  end;
end;

procedure TMySQLRSGrid.DoGetText(Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var Text: WideString);
begin
  if(Assigned(OnGetText))then
    OnGetText(Self, Node, Column, TextType, Text)
  else
  begin
    if(Column=0)then
    begin
      Text:='';
    end
    else if(MySQLRS.ResultSet<>nil)then
    begin
      if(Column-1<integer(MySQLRS.ResultSet.columns_num))then
        Text:=AnsiReplaceStr(
          AnsiReplaceStr(
            GetNodeRSText(Node, Column),
            #13#10, ''),
          #10, '');
    end
    else
      Text:='';
  end;
end;

procedure TMySQLRSGrid.DoPaintText(Node: PVirtualNode; const Canvas: TCanvas; Column: TColumnIndex;
  TextType: TVSTTextType);
var action: PMYX_RS_ACTION;
begin
  if(Assigned(OnPaintText))then
    OnPaintText(Self, Canvas, Node, Column, TextType)
  else
  begin
    MySQLRS.WorkWithResultset.Acquire;
    try
      action:=GetNodeRSAction(Node, Column);

      if(action<>nil)then
        case action.action of
          MYX_RSA_UPDATE:
          begin
            Canvas.Font.Color:=clBlue;
          end;
          MYX_RSA_ADD:
          begin
            Canvas.Font.Color:=clGreen;
          end;
        else
          Canvas.Font.Color:=$00000000;
        end;
    finally
      MySQLRS.WorkWithResultset.Release;
    end;
  end;
end;

function TMySQLRSGrid.DoFocusChanging(OldNode, NewNode: PVirtualNode; OldColumn,
  NewColumn: TColumnIndex): Boolean;
begin
  Result:=True;

  if(NewColumn=0)then
    Result:=False
  else if Assigned(OnFocusChanging) then
    OnFocusChanging(Self, OldNode, NewNode, OldColumn, NewColumn, Result);
end;

procedure TMySQLRSGrid.DoAfterCellPaint(Canvas: TCanvas; Node: PVirtualNode; Column:
  TColumnIndex; CellRect: TRect);
var mx, my: integer;
begin
  if(Column=0)and(Node=FocusedNode)then
  begin
    mx:=(CellRect.Right-CellRect.Left) div 2;
    my:=(CellRect.Bottom-CellRect.Top) div 2;
    with Canvas do
    begin
      if(Active)then
      begin
        Pen.Color:=clBlack;
        Brush.Color:=clBlack;
      end
      else
      begin
        Pen.Color:=$00C0C0C0;
        Brush.Color:=$00C0C0C0;
      end;

      Polygon([Point(mx-3, my-5), Point(mx+2, my), Point(mx-3, my+5)]);
    end;
  end;

  if(Assigned(OnAfterCellPaint))then
    OnAfterCellPaint(Self, Canvas, Node, Column, CellRect);
end;

procedure TMySQLRSGrid.DoCanEdit(Node: PVirtualNode; Column: TColumnIndex; var Allowed: Boolean);
begin
  Allowed:=False;

  if(FMySQLRS.EditingAllowed)then
    Allowed:=True;

  if Assigned(OnEditing) then
    OnEditing(Self, Node, Column, Allowed);
end;

procedure TMySQLRSGrid.KeyDown(var Key: Word; Shift: TShiftState);
{var Node: PVirtualNode;
  NodeData: PRSRowData;}
begin
  if(Key=VK_Return)and(FocusedNode<>nil)then
    EditNode(FocusedNode, FocusedColumn)
  {else if(Key=VK_Down)and(FMySQLRS.LastRowSelected)and
    (FMySQLRS.Editable)and(FMySQLRS.EditingAllowed)then
  begin
    //Don't let the user insert a new row if there is another inserted row
    //and no actions have been defined for that row
    NodeData:=GetNodeData(FocusedNode);
    if(Not((NodeData.row_index>=FMySQLRS.ResultSet.rows_num)and
      (NodeData.actions=nil)))then
    begin
      //Insert new row
      Node:=AddChild(nil);

      NodeData:=GetNodeData(Node);
      NodeData.row_index:=FMySQLRS.ResultSet.rows_num+FMySQLRS.InsertedRows;
      NodeData.actions:=nil;

      FocusedNode:=Node;
      ClearSelection;
      Selected[FocusedNode]:=True;

      FMySQLRS.InsertedRows:=FMySQLRS.InsertedRows+1;
    end;
  end}
  else if(Key=VK_F2)and(FMySQLRS.EditingAllowed=False)then
    FMySQLRS.DoEdit
  else
    if(Assigned(OnKeyDown))then
      OnKeyDown(self, Key, Shift);
end;

procedure TMySQLRSGrid.DoFocusChange(Node: PVirtualNode; Column: TColumnIndex);
var NodeData: PRSRowData;
  RowIndex: integer;
begin
  NodeData:=GetNodeData(Node);
  if(NodeData<>nil)then
    RowIndex:=NodeData.row_index
  else
    RowIndex:=-1;

  //Notify RS that current row has changed
  FMySQLRS.DoCurrentRowChanged(self, RowIndex);

  if(Column=0)then
    FocusedColumn:=Header.Columns.Count-1;

  if(Assigned(OnFocusChanged))then
    OnFocusChanged(self, Node, Column);
end;

procedure TMySQLRSGrid.AddAction(action_type: MYX_RS_ACTION_TYPE; row: Cardinal;
  column: Integer; Node: PVirtualNode;
  new_value: WideString; new_value_length: Integer);
var NewNode: PVirtualNode;
  NodeData: PRSRowData;
  action_index: integer;
  new_action: PMYX_RS_ACTION;
  col: PMYX_RS_COLUMN;
  len: integer;
  s: UTF8String;
  BinString: string;
  FirstNodeAction: Boolean;
begin
  NodeData:=nil;
  if(Node<>nil)then
    NodeData:=GetNodeData(Node);

  if(NodeData<>nil)and(Column>0)and
    (Column-1<integer(FMySQLRS.ResultSet.columns_num))then
  begin
    col:=(PMYX_RS_COLUMN(integer(FMySQLRS.ResultSet.columns)+(sizeof(MYX_RS_COLUMN)*(Column-1))));


    //if length=-1 then get length from WideString
    //Otherwise we have binary data
    if(new_value_length=-1)then
    begin
      s:=UTF8Encode(new_value);
      len:=Length(s);
      new_action:=myx_query_create_action(
        action_type, row, col, len, PChar(s));
    end
    else
    begin
      //!!!This is dirty and need a different solution
      len:=new_value_length;

      BinString:=WideCharLenToString(PWideChar(new_value), len);
      new_action:=myx_query_create_action(
        action_type, row, col, len, PChar(BinString));
    end;

    try
      //Add action to node (maybe overwrite older actions)
      action_index:=myx_query_add_action(FMySQLRS.ResultSet, new_action);
    finally
      myx_query_free_action(new_action);
    end;

    //Add action
    if(NodeData.actions=nil)then
    begin
      NodeData.actions:=TIntegerList.Create;
      FirstNodeAction:=True;
    end
    else
      FirstNodeAction:=False;

    NodeData.actions.Add(action_index);

    //Add this Node to list of NodesWithAction
    NodesWithActionList.Add(Node);
    if(action_type=MYX_RSA_DELETE)then
      NodesWithDeleteActionList.Add(Node);

    //if this was the first action in the additional row for inserts,
    //add another row
    if(NodeData.row_index=FMySQLRS.ResultSet.rows_num+FMySQLRS.InsertedRows)and
      (FirstNodeAction)then
    begin
      FMySQLRS.InsertedRows:=FMySQLRS.InsertedRows+1;

      //Insert new row
      NewNode:=AddChild(nil);

      NodeData:=GetNodeData(NewNode);
      NodeData.row_index:=FMySQLRS.ResultSet.rows_num+FMySQLRS.InsertedRows;
      NodeData.actions:=nil;
    end;

    FMySQLRS.Edited:=True;
  end;
end;

procedure TMySQLRSGrid.DoNewText(Node: PVirtualNode; Column: TColumnIndex; Text: WideString);
var NodeData: PRSRowData;
  {col: PMYX_RS_COLUMN;
  new_action: PMYX_RS_ACTION;
  action_index: integer;}
  action_type: MYX_RS_ACTION_TYPE;
begin
  if(Assigned(OnNewText))then
    OnNewText(Self, Node, Column, Text)
  else
  begin
    MySQLRS.WorkWithResultset.Acquire;
    try
      NodeData:=nil;
      if(Node<>nil)then
        NodeData:=GetNodeData(Node);

      if(NodeData<>nil)and(Column>0)and
        (Column-1<integer(FMySQLRS.ResultSet.columns_num))then
      begin
        //col:=(PMYX_RS_COLUMN(integer(FMySQLRS.ResultSet.columns)+(sizeof(MYX_RS_COLUMN)*(Column-1))));

        if(NodeData.row_index<FMySQLRS.ResultSet.rows_num)then
          action_type:=MYX_RSA_UPDATE
        else
          action_type:=MYX_RSA_ADD;

        AddAction(action_type, NodeData.row_index, Column, Node, Text);
      end;
    finally
      MySQLRS.WorkWithResultset.Release;
    end;
  end;
end;

procedure TMySQLRSGrid.DoBeforeItemErase(Canvas: TCanvas; Node: PVirtualNode; ItemRect: TRect; var Color: TColor;
  var EraseAction: TItemEraseAction);
begin
  if(Assigned(OnBeforeItemErase))then
    OnBeforeItemErase(Self, Canvas, Node, ItemRect, Color, EraseAction)
  else
  begin
    EraseAction:=eaNone;

    if(FActive)then
    begin
      if(((Cardinal(ItemRect.Top - OffsetY) div DefaultNodeHeight) mod 2)<>0)then
        RowColor:=$00FAFAFA
      else
        RowColor:=$00FFFFFF;
    end
    else
     RowColor:=$00F0F0F0;
  end;
end;

procedure TMySQLRSGrid.DoAfterItemErase(Canvas: TCanvas; Node: PVirtualNode; ItemRect: TRect);
var action: PMYX_RS_ACTION;
  row: PMYX_RS_ROW;
  i, w: integer;
  ColDiffBit: Cardinal;
begin
  if(Header.Columns.Count>0)then
  begin
    //Make BG of first column gray
    Canvas.Brush.Color:=clBtnFace;
    Canvas.FillRect(Rect(Header.Columns[0].Left, ItemRect.Top,
      Header.Columns[0].Left+Header.Columns[0].Width, ItemRect.Bottom));

    MySQLRS.WorkWithResultset.Acquire;
    try
      row:=GetNodeRSRow(Node);
      if(row<>nil)then
      begin
        //Highlight diffs after compare
        if(row^.diff=Ord(MYX_RD_THIS_ONLY))then
          RowColor:=$00E0FFE0
        else if(row^.diff=Ord(MYX_RD_OTHER_ONLY))then
          RowColor:=$00E0E0FF;

        ColDiffBit:=1;
        for i:=1 to Header.Columns.Count-1 do
        begin
          Canvas.Brush.Color:=RowColor;

          //Highlight Column differences after compare
          if(row^.diff>0)then
            if((ColDiffBit and (row^.diff shr 4))=ColDiffBit)then
            begin
              Canvas.Brush.Color:=$00FFFFDD;
              Canvas.Brush.Style:=bsSolid;
            end;

          action:=GetNodeRSAction(Node, i);

          if(action<>nil)then
          begin
            case action.action of
              MYX_RSA_UPDATE:
                Canvas.Brush.Color:=$00FFBBBB;
              MYX_RSA_ADD:
                Canvas.Brush.Color:=$00BBFFBB;
              MYX_RSA_DELETE:
                Canvas.Brush.Color:=$00BBBBFF;
            else
              Canvas.Brush.Color:=$00FFFFFF;
            end;

            Canvas.Brush.Style:=bsSolid;
          end;

          Canvas.FillRect(Rect(Header.Columns[i].Left-OffsetX, ItemRect.Top,
            Header.Columns[i].Left+Header.Columns[i].Width-OffsetX, ItemRect.Bottom));

          if(ColDiffBit=1)then
            ColDiffBit:=2
          else
            ColDiffBit:=ColDiffBit*2;
        end;
      end
      else
      begin
        action:=GetNodeRSAction(Node, 1);

        if(action<>nil)then
        begin
          case action.action of
            MYX_RSA_ADD:
              Canvas.Brush.Color:=$00BBFFBB;
            MYX_RSA_DELETE:
              Canvas.Brush.Color:=$00BBBBFF;
          else
            Canvas.Brush.Color:=RowColor;
          end;
        end
        else
          Canvas.Brush.Color:=RowColor;

        //Get Column Width
        w:=0;
        for i:=1 to Header.Columns.Count-1 do
          w:=w+Header.Columns[i].Width;

        Canvas.FillRect(Rect(Header.Columns[1].Left-OffsetX, ItemRect.Top,
          Header.Columns[1].Left-OffsetX+w, ItemRect.Bottom));

        Canvas.Brush.Color:=$00FFFFFF;
        Canvas.FillRect(Rect(Header.Columns[1].Left-OffsetX+w+1, ItemRect.Top,
          ItemRect.Right, ItemRect.Bottom));
      end;

    finally
      MySQLRS.WorkWithResultset.Release;
    end;
  end
  else
  begin
    Canvas.Brush.Color:=RowColor;
    Canvas.FillRect(ItemRect);
  end;

  RowColor:=clWhite;
end;


procedure TMySQLRSGrid.DoGridScroll(Sender: TBaseVirtualTree; DeltaX, DeltaY: Integer);
var i: integer;
begin
  if(Parent is TMySQLRSPanel)then
    if(Parent.Parent is TRSTabSheet)then
      if(not(TRSTabSheet(Parent.Parent).GridsAreScrolling))then
      begin
        TRSTabSheet(Parent.Parent).GridsAreScrolling:=True;
        try
          Update;
          for i:=0 to TRSTabSheet(Parent.Parent).RSPanels.Count-1 do
          begin
            TMySQLRSPanel(TRSTabSheet(Parent.Parent).RSPanels[i]).RSGrid.OffsetXY:=
              OffsetXY;

            TMySQLRSPanel(TRSTabSheet(Parent.Parent).RSPanels[i]).RSGrid.Update;
          end;
        finally
          TRSTabSheet(Parent.Parent).GridsAreScrolling:=False;
        end;
      end;
end;

procedure TMySQLRSGrid.DeleteSelectedRows;
var i: integer;
  NodeData: PRSRowData;
  SelectedNodes: TNodeArray;
begin
  SelectedNodes:=GetSortedSelection(True);

  if(FMySQLRS.EditingAllowed)and(Length(SelectedNodes)>0)then
  begin
    MySQLRS.WorkWithResultset.Acquire;
    try
      for i:=0 to Length(SelectedNodes)-1 do
      begin
        NodeData:=GetNodeData(SelectedNodes[i]);

        AddAction(MYX_RSA_DELETE, NodeData.row_index, 1, SelectedNodes[i], '');
      end;
    finally
      MySQLRS.WorkWithResultset.Release;
    end;
  end;

  Invalidate;
end;

procedure TMySQLRSGrid.ApplyDeleteActions;
var i, j: integer;
  NodeData: PRSRowData;
  DeleteActionApplyed: Boolean;
  action: PMYX_RS_ACTION;
begin
  i:=0;
  while(i<NodesWithDeleteActionList.Count)do
  begin
    MySQLRS.WorkWithResultset.Acquire;
    try
      NodeData:=GetNodeData(NodesWithDeleteActionList[i]);

      DeleteActionApplyed:=True;
      for j:=NodeData.actions.Count-1 downto 0 do
      begin
        action:=PMYX_RS_ACTION(integer(MySQLRS.ResultSet.actions.actions)+sizeof(MYX_RS_ACTION)*NodeData.actions[j]);

        if(action.action=MYX_RSA_DELETE)then
        begin
          DeleteActionApplyed:=False;
          break;
        end;
      end;

      if(DeleteActionApplyed)then
      begin
        DeleteNode(NodesWithDeleteActionList[i]);
        NodesWithDeleteActionList.Delete(i);
      end
      else
        inc(i);

    finally
      MySQLRS.WorkWithResultset.Release;
    end;
  end;

  Invalidate;
end;

function TMySQLRSGrid.DoCreateEditor(Node: PVirtualNode; Column: TColumnIndex): IVTEditLink;
begin
  Result := TRSFieldEditLink.Create;
end;

procedure TMySQLRSGrid.DblClick;
begin
  if(Assigned(OnDblClick))then
    OnDblClick(Self)
  else
  begin
    if(MySQLRS.EditingAllowed)and(MySQLRS.Editable)then
      PostMessage(Handle, WM_EditCurrentCell, 0, 0);
  end;
end;

procedure TMySQLRSGrid.WndProc(var Message: TMessage);
begin
  inherited WndProc(Message);

  if(Message.Msg=WM_EditCurrentCell)then
  begin
    EditColumn:=FocusedColumn;
    DoEdit;
  end;
end;

// -----------------------------------------------------------------------------

constructor TMySQLRSNav.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  Height:=20;
  BevelOuter:=bvNone;
  Caption:='';

  RSEditPNGImg:=LoadPNGImageFromResource('rs_edit');
  RSEditDisabledPNGImg:=LoadPNGImageFromResource('rs_edit_disabled');
  RSEditActivePNGImg:=LoadPNGImageFromResource('rs_edit_active');
  RSApplyPNGImg:=LoadPNGImageFromResource('rs_apply');
  RSApplyDisabledPNGImg:=LoadPNGImageFromResource('rs_apply_disabled');
  RSDiscardPNGImg:=LoadPNGImageFromResource('rs_discard');
  RSDiscardDisabledPNGImg:=LoadPNGImageFromResource('rs_discard_disabled');

  RSFirstPNGImg:=LoadPNGImageFromResource('rs_first');
  RSFirstDisabledPNGImg:=LoadPNGImageFromResource('rs_first_disabled');

  RSLastPNGImg:=LoadPNGImageFromResource('rs_last');
  RSLastDisabledPNGImg:=LoadPNGImageFromResource('rs_last_disabled');

  RSSearchPNGImg:=LoadPNGImageFromResource('rs_search');
  RSSearchDisabledPNGImg:=LoadPNGImageFromResource('rs_search_disabled');
end;

destructor TMySQLRSNav.Destroy;
begin
  MySQLRS:=nil;

  RSEditPNGImg.Free;
  RSEditDisabledPNGImg.Free;
  RSEditActivePNGImg.Free;
  RSApplyPNGImg.Free;
  RSApplyDisabledPNGImg.Free;
  RSDiscardPNGImg.Free;
  RSDiscardDisabledPNGImg.Free;
  RSFirstPNGImg.Free;
  RSFirstDisabledPNGImg.Free;
  RSLastPNGImg.Free;
  RSLastDisabledPNGImg.Free;
  RSSearchPNGImg.Free;
  RSSearchDisabledPNGImg.Free;

  inherited;
end;

procedure TMySQLRSNav.ClearValues;
begin
  //
end;

function TMySQLRSNav.GetMySQLRS: TMySQLRS;
begin
  Result:=FMySQLRS;
end;

procedure TMySQLRSNav.SetMySQLRS(MySQLRS: TMySQLRS);
begin
  if(FMySQLRS<>nil)and(MySQLRS=nil)then
    FMySQLRS.DisconnectControl(self);

  FMySQLRS:=MySQLRS;

  if(MySQLRS<>nil)then
    MySQLRS.ConnectControl(self);
end;


function TMySQLRSNav.GetControl: TObject;
begin
  Result:=self;
end;

function TMySQLRSNav.GetActive: Boolean;
begin
  Result:=FActive;
end;

procedure TMySQLRSNav.SetActive(Active: Boolean);
begin
  FActive:=Active;

  Paint;
end;

procedure TMySQLRSNav.DoCurrentRowChanged;
begin
  Paint;
end;

procedure TMySQLRSNav.DoEditStateChanged;
begin
  Paint;
end;

procedure TMySQLRSNav.DoStatusCaptionChanged;
begin
  Paint;
end;

procedure TMySQLRSNav.DoMessagesChanged;
begin
  Paint;
end;


procedure TMySQLRSNav.Paint;
var xpos: integer;
begin
  with Canvas do
  begin
    Font.Name:='Tahoma';
    Font.Size:=7;

    Pen.Color:=$009C9B91;
    if(FActive)then
      Brush.Color:=clWhite
    else
      Brush.Color:=$00EEEEEE;
    Rectangle(Rect(0, -1, Width, Height));

    xpos:=Width;

    RSSearchXPos:=PaintEditButton(_('Search'), xpos, RSSearchPNGImg,
      RSSearchDisabledPNGImg, MySQLRS.AtLeastOneRecord);

    RSLastXPos:=PaintEditButton(_('Last'), xpos, RSLastPNGImg,
      RSLastDisabledPNGImg, Not(MySQLRS.LastRowSelected)and
        MySQLRS.AtLeastOneRecord);

    RSFirstXPos:=PaintEditButton(_('First'), xpos, RSFirstPNGImg,
      RSFirstDisabledPNGImg, Not(MySQLRS.FirstRowSelected)and
        MySQLRS.AtLeastOneRecord);

    xpos:=xpos-3;
    MoveTo(xpos, 0);
    LineTo(xpos, 20);

    RSDiscardXPos:=PaintEditButton(_('Discard Changes'), xpos, RSDiscardPNGImg,
      RSDiscardDisabledPNGImg, (MySQLRS.Edited));

    RSApplyXPos:=PaintEditButton(_('Apply Changes'), xpos, RSApplyPNGImg,
      RSApplyDisabledPNGImg, (MySQLRS.Edited));

    RSEditXPos:=PaintEditButton(_('Edit'), xpos,
      RSEditPNGImg,
      RSEditDisabledPNGImg,
      (MySQLRS.Editable)and(Not(MySQLRS.Edited))and
      (Not(MySQLRS.EditingAllowed)),
      RSEditActivePNGImg,
      (MySQLRS.EditingAllowed));

    xpos:=xpos-3;
    MoveTo(xpos, 0);
    LineTo(xpos, 20);

    Font.Color:=clBlack;

    DrawWideStringText(Canvas.Handle, PWideChar(MySQLRS.StatusCaption),
      Length(MySQLRS.StatusCaption), Rect(4, 4, xpos-3, 4+10));
  end;
end;

function TMySQLRSNav.PaintEditButton(Caption: WideString; var xpos: integer;
  PNGImg: TPNGObject; DisabledPNGImg: TPNGObject; Enabled: Boolean;
  ActivePNGImg: TPNGObject; Active: Boolean): integer;
var l: integer;
  s: WideString;
begin
  with Canvas do
  begin
    xpos:=xpos-7;

    if(Width>550)then
    begin
      s:=_(Caption);

      l:=GetWideStringTextWidth(Canvas, s);
      xpos:=xpos-l;

      if(Active)then
      begin
        Font.Color:=$0023AC23;
      end
      else if(Enabled)then
        Font.Color:=clBlack
      else
        Font.Color:=$00CCCCCC;

      DrawWideStringText(Canvas.Handle, PWideChar(s),
        Length(s), Rect(xpos, 4, xpos+l, 4+18));

      xpos:=xpos-21;
    end
    else
      xpos:=xpos-16;

    if(Active)then
      ActivePNGImg.Draw(Canvas,
        Rect(xpos+1, 2, xpos+1+RSSearchPNGImg.Width, 2+RSSearchPNGImg.Height))
    else if(Enabled)then
      PNGImg.Draw(Canvas,
        Rect(xpos+1, 2, xpos+1+RSSearchPNGImg.Width, 2+RSSearchPNGImg.Height))
    else
      DisabledPNGImg.Draw(Canvas,
        Rect(xpos+1, 2, xpos+1+RSSearchPNGImg.Width, 2+RSSearchPNGImg.Height));

    xpos:=xpos-5;
    MoveTo(xpos, 0);
    LineTo(xpos, 20);
  end;

  Result:=xpos;
end;

procedure TMySQLRSNav.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  if(Assigned(OnEnter))then
    OnEnter(self);

  if(MySQLRS.Editable)and
    (X>RSEditXPos)and(X<RSApplyXPos)then
    MySQLRS.DoEdit;

  if(MySQLRS.Editable)and(MySQLRS.EditingAllowed)and(MySQLRS.Edited)and
    (X>RSApplyXPos)and(X<RSDiscardXPos)then
    MySQLRS.DoApplyChanges;

  if(MySQLRS.Editable)and(MySQLRS.EditingAllowed)and(MySQLRS.Edited)and
    (X>RSDiscardXPos)and(X<RSFirstXPos)then
    MySQLRS.DoDiscardChanges;

  if(Not(MySQLRS.FirstRowSelected)and(MySQLRS.AtLeastOneRecord))and
    (X>RSFirstXPos)and(X<RSLastXPos)then
    MySQLRS.GotoFirstRow;

  if(Not(MySQLRS.LastRowSelected)and(MySQLRS.AtLeastOneRecord))and
    (X>RSLastXPos)and(X<RSSearchXPos)then
    MySQLRS.GotoLastRow;

  if(MySQLRS.AtLeastOneRecord)and
    (X>RSSearchXPos)then
    MySQLRS.DoSearch;
end;


// -----------------------------------------------------------------------------


constructor TMySQLRSErrorGrid.Create(AOwner: TComponent);
var TVColumn: TVirtualTreeColumn;
begin
  inherited Create(AOwner);

  NodeDataSize:=SizeOf(TRSMsg);

  Header.Options:=Header.Options+[hoDblClickResize, hoVisible];
  Header.Columns.Clear;
  TVColumn:=Header.Columns.Add;
  TVColumn.Text:='!';
  TVColumn.Width:=16;
  TVColumn:=Header.Columns.Add;
  TVColumn.Text:=_('Description');
  TVColumn:=Header.Columns.Add;
  TVColumn.Text:=_('ErrorNr.');
  TVColumn.Width:=70;
  Header.AutoSizeIndex:=1;
  Header.Options:=Header.Options+[hoAutoResize]-[hoColumnResize];
  ParentBackground:=False;
  TreeOptions.MiscOptions:=TreeOptions.MiscOptions+
    [toGridExtensions];
  TreeOptions.PaintOptions:=TreeOptions.PaintOptions-
    [toShowRoot, toShowTreeLines];
  TreeOptions.SelectionOptions:=TreeOptions.SelectionOptions+
    [toFullRowSelect];
  TreeOptions.StringOptions:=TreeOptions.StringOptions-
    [toSaveCaptions];
  HintMode:=hmTooltip;
  ShowHint:=True;
  DefaultNodeHeight:=16;
  Align:=alBottom;
  Height:=0;

  //Load PNG Images
  RSErrorPNGImg:=LoadPNGImageFromResource('rs_error');
  RSWarningPNGImg:=LoadPNGImageFromResource('rs_warning');

  //Create PopupMenu
  MsgPopupMenu:=TTntPopupMenu.Create(nil);
  CopyMsgMenuItem:=TTntMenuItem.Create(self);
  CopyMsgMenuItem.Caption:=_('Copy Message Text');
  CopyMsgMenuItem.OnClick:=CopyMsgText;
  MsgPopupMenu.Items.Add(CopyMsgMenuItem);

  Self.PopupMenu:=MsgPopupMenu;
end;

destructor TMySQLRSErrorGrid.Destroy;
begin
  MySQLRS:=nil;

  RSErrorPNGImg.Free;
  RSWarningPNGImg.Free;

  MsgPopupMenu.Free;

  inherited;
end;

procedure TMySQLRSErrorGrid.ClearValues;
begin
  Clear;
  Height:=0;
end;

function TMySQLRSErrorGrid.GetMySQLRS: TMySQLRS;
begin
  Result:=FMySQLRS;
end;

function TMySQLRSErrorGrid.GetControl: TObject;
begin
  Result:=self;
end;

procedure TMySQLRSErrorGrid.DoCurrentRowChanged;
begin
  //
end;

procedure TMySQLRSErrorGrid.DoEditStateChanged;
begin
  //
end;

procedure TMySQLRSErrorGrid.DoStatusCaptionChanged;
begin
  //
end;

procedure TMySQLRSErrorGrid.DoMessagesChanged;
var Node: PVirtualNode;
  Nodedata: PRSMsg;
  i: integer;
begin
  if(FMySQLRS.ErrorMessages.Count=0)then
  begin
    Height:=0;
    Clear;
  end
  else
  begin
    for i:=0 to FMySQLRS.ErrorMessages.Count-1 do
    begin
      Node:=AddChild(nil);

      NodeData:=GetNodeData(Node);
      NodeData.MsgIndex:=i;
    end;

    if(RootNodeCount<=4)then
      Height:=20+1+RootNodeCount*16
    else
      Height:=20+1+4*16;
  end;
end;

procedure TMySQLRSErrorGrid.DoActionsChanged;
begin
  //
end;


procedure TMySQLRSErrorGrid.SetMySQLRS(MySQLRS: TMySQLRS);
begin
  if(FMySQLRS<>nil)and(MySQLRS=nil)then
    FMySQLRS.DisconnectControl(self);

  FMySQLRS:=MySQLRS;

  if(MySQLRS<>nil)then
    MySQLRS.ConnectControl(self);
end;

procedure TMySQLRSErrorGrid.DoGetText(Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType; var Text: WideString);
var NodeData: PRSMsg;
begin
  if(Assigned(OnGetText))then
    OnGetText(Self, Node, Column, TextType, Text)
  else
  begin
    if(Column=0)then
    begin
      Text:='';
    end
    else
    begin
      NodeData:=GetNodeData(Node);

      if(NodeData<>nil)then
        if(NodeData.MsgIndex<FMySQLRS.ErrorMessages.Count)then
          case Column of
            1:
              Text:=TRSError(FMySQLRS.ErrorMessages[NodeData.MsgIndex]).Msg;
            2:
              if(TRSError(FMySQLRS.ErrorMessages[NodeData.MsgIndex]).MsgNr<>0)then
                Text:=IntToStr(TRSError(FMySQLRS.ErrorMessages[NodeData.MsgIndex]).MsgNr)
              else
                Text:='';
          end;
    end;
  end;
end;

procedure TMySQLRSErrorGrid.DoAfterCellPaint(Canvas: TCanvas; Node: PVirtualNode; Column:
  TColumnIndex; CellRect: TRect);
var NodeData: PRSMsg;
begin
  if(Column=0)then
  begin
    NodeData:=nil;
    if(Node<>nil)then
      NodeData:=GetNodeData(Node);

    if(NodeData<>nil)then
      if(NodeData.MsgIndex<FMySQLRS.ErrorMessages.Count)then
      begin
        case TRSError(FMySQLRS.ErrorMessages[NodeData.MsgIndex]).MsgType of
          RSMT_ERROR:
            RSErrorPNGImg.Draw(Canvas, Rect(4, 2,
              RSErrorPNGImg.Width+1, RSErrorPNGImg.Height+1));
          RSMT_WARNING:
            RSWarningPNGImg.Draw(Canvas, Rect(4, 2,
              RSWarningPNGImg.Width+1, RSWarningPNGImg.Height+1));
        end;
      end;
  end;

  if(Assigned(OnAfterCellPaint))then
    OnAfterCellPaint(Self, Canvas, Node, Column, CellRect);
end;

procedure TMySQLRSErrorGrid.CopyMsgText(Sender: TObject);
var NodeData: PRSMsg;
begin
  if(FocusedNode<>nil)then
  begin
    NodeData:=GetNodeData(FocusedNode);
    if(NodeData<>nil)then
      if(NodeData.MsgIndex<FMySQLRS.ErrorMessages.Count)then
      begin
        Clipboard.AsText:=TRSError(FMySQLRS.ErrorMessages[NodeData.MsgIndex]).Msg;
      end;
  end;
end;

// ---------------------------------------------------------------------


//----------------- TRSStringEdit --------------------------------------------------------------------------------------------

// Implementation of a generic node caption editor.

constructor TRSStringEdit.Create(Link: TRSFieldEditLink);

begin
  inherited Create(nil);
  ShowHint := False;
  ParentShowHint := False;
  // This assignment increases the reference count for the interface.
  FRefLink := Link;
  // This reference is used to access the link.
  FLink := Link;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSStringEdit.CMAutoAdjust(var Message: TMessage);

begin
  AutoAdjustSize;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSStringEdit.CMExit(var Message: TMessage);

begin
  if Assigned(FLink) and not FLink.FStopping then
    with FLink, FTree do
    begin
      if (toAutoAcceptEditChange in TreeOptions.StringOptions) then
        DoEndEdit
      else
        DoCancelEdit;
    end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSStringEdit.CMRelease(var Message: TMessage);

begin
  Free;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSStringEdit.CNCommand(var Message: TWMCommand);

begin
  if Assigned(FLink) and Assigned(FLink.FTree) and (Message.NotifyCode = EN_UPDATE) and
    not (toGridExtensions in FLink.FTree.TreeOptions.MiscOptions) and
    not (vsMultiline in FLink.FNode.States) then
    // Instead directly calling AutoAdjustSize it is necessary on Win9x/Me to decouple this notification message
    // and eventual resizing. Hence we use a message to accomplish that.
    if false and ((Win32Platform and VER_PLATFORM_WIN32_NT) <> 0) then
      AutoAdjustSize
    else
      PostMessage(Handle, CM_AUTOADJUST, 0, 0);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSStringEdit.WMChar(var Message: TWMChar);

begin
  if not (Message.CharCode in [VK_ESCAPE, VK_TAB]) then
    inherited;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSStringEdit.WMDestroy(var Message: TWMDestroy);

begin
  // If editing stopped by other means than accept or cancel then we have to do default processing for
  // pending changes.
  if Assigned(FLink) and not FLink.FStopping then
  begin
    with FLink, FTree do
    begin
      if (toAutoAcceptEditChange in TreeOptions.StringOptions) and Modified then
        Text[FNode, FColumn] := FEdit.Text;
    end;
    FLink := nil;
    FRefLink := nil;
  end;

  inherited;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSStringEdit.WMGetDlgCode(var Message: TWMGetDlgCode);

begin
  inherited;

  Message.Result := Message.Result or DLGC_WANTALLKEYS or DLGC_WANTTAB or DLGC_WANTARROWS;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSStringEdit.WMKeyDown(var Message: TWMKeyDown);

// Handles some control keys.

var
  Shift: TShiftState;
  EndEdit: Boolean;
  Tree: TBaseVirtualTree;
  Node: PVirtualNode;
begin
  Shift := KeyDataToShiftState(Message.KeyData);
  Tree := FLink.FTree;

  // Pretend these keycodes were send to the tree.
  if(Message.CharCode=VK_ESCAPE)then
  begin
    FLink.FTree.DoCancelEdit;
    Tree.SetFocus;
  end
  else if(Message.CharCode=VK_RETURN)then
  begin
    EndEdit := not (vsMultiline in FLink.FNode.States);
    if not EndEdit then
    begin
      // If a multiline node is being edited the finish editing only if Ctrl+Enter was pressed,
      // otherwise allow to insert line breaks into the text.
      EndEdit := ssCtrl in Shift;
    end;
    if EndEdit then
    begin
      FLink.FTree.InvalidateNode(FLink.FNode);
      FLink.FTree.DoEndEdit;
      Tree.SetFocus;
    end;
  end
  else if(Message.CharCode=VK_UP)then
  begin
    if(Tree.FocusedNode = nil)then
      Node:=Tree.GetLastVisible
    else
      Node:=Tree.GetPreviousVisible(Tree.FocusedNode);

    if(Node<>nil)then
    begin
      Tree.InvalidateNode(FLink.FNode);
      FLink.FTree.DoEndEdit;
      Tree.SetFocus;

      Tree.FocusedNode:=Node;
      Tree.ClearSelection;
      Tree.Selected[Tree.FocusedNode]:=True;

      PostMessage(Tree.Handle, WM_EditCurrentCell, 0, 0);
    end;
  end
  else if(Message.CharCode=VK_DOWN)then
  begin
    if(Tree.FocusedNode = nil)then
      Node:=Tree.GetFirstVisible
    else
      Node:=Tree.GetNextVisible(Tree.FocusedNode);

    if(Node<>nil)then
    begin
      Tree.InvalidateNode(FLink.FNode);
      FLink.FTree.DoEndEdit;
      Tree.SetFocus;

      Tree.FocusedNode:=Node;
      Tree.ClearSelection;
      Tree.Selected[Tree.FocusedNode]:=True;

      PostMessage(Tree.Handle, WM_EditCurrentCell, 0, 0);
    end;
  end
  else if(((Message.CharCode=VK_LEFT)and(Shift=[ssCtrl]))or
    ((Message.CharCode=VK_TAB)and(Shift=[ssShift])))then
  begin
    if(Tree.FocusedColumn>1)then
    begin
      Tree.InvalidateNode(FLink.FNode);
      FLink.FTree.DoEndEdit;
      Tree.SetFocus;

      Tree.FocusedColumn:=Tree.FocusedColumn-1;

      PostMessage(Tree.Handle, WM_EditCurrentCell, 0, 0);
    end;
  end
  else if(((Message.CharCode=VK_RIGHT)and(Shift=[ssCtrl]))or
    ((Message.CharCode=VK_TAB)and(Shift=[])))then
  begin
    if(Tree.FocusedColumn<FLink.FTree.Header.Columns.Count-1)then
    begin
      Tree.InvalidateNode(FLink.FNode);
      FLink.FTree.DoEndEdit;
      Tree.SetFocus;

      Tree.FocusedColumn:=Tree.FocusedColumn+1;

      PostMessage(Tree.Handle, WM_EditCurrentCell, 0, 0);
    end
    else
    begin
      Node:=Tree.GetNextVisible(Tree.FocusedNode);

      if(Node<>nil)then
      begin
        Tree.InvalidateNode(FLink.FNode);
        FLink.FTree.DoEndEdit;
        Tree.SetFocus;

        Tree.FocusedNode:=Node;
        Tree.FocusedColumn:=1;
        Tree.ClearSelection;
        Tree.Selected[Tree.FocusedNode]:=True;

        PostMessage(Tree.Handle, WM_EditCurrentCell, 0, 0);
      end;
    end;
  end
  else
    inherited;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSStringEdit.AutoAdjustSize;

// Changes the size of the edit to accomodate as much as possible of its text within its container window.
// NewChar describes the next character which will be added to the edit's text.

var
  DC: HDC;
  Size: TSize;
  LastFont: THandle;

begin
  if not (vsMultiline in FLink.FNode.States) then
  begin
    // avoid flicker
    SendMessage(Handle, WM_SETREDRAW, 0, 0);

    DC := GetDC(Handle);
    LastFont := SelectObject(DC, Font.Handle);
    try
      // Read needed space for the current text.
      {.$ifdef TntSupport}
        GetTextExtentPoint32W(DC, PWideChar(Text), Length(Text), Size);
      {.$else}
        //GetTextExtentPoint32(DC, PChar(Text), Length(Text), Size);
      {.$endif TntSupport}
      Inc(Size.cx, 2 * FLink.FTree.TextMargin);

      // Repaint associated node if the edit becomes smaller.
      if Size.cx < Width then
        FLink.FTree.InvalidateNode(FLink.FNode);

      if FLink.FAlignment = taRightJustify then
        FLink.SetBounds(Rect(Left + Width - Size.cx, Top, Left + Width, Top + Height))
      else
        FLink.SetBounds(Rect(Left, Top, Left + Size.cx, Top + Height));
    finally
      SelectObject(DC, LastFont);
      ReleaseDC(Handle, DC);
      SendMessage(Handle, WM_SETREDRAW, 1, 0);
    end;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSStringEdit.CreateParams(var Params: TCreateParams);

begin
  inherited;

  // Only with multiline style we can use the text formatting rectangle.
  // This does not harm formatting as single line control, if we don't use word wrapping.
  with Params do
  begin
    Style := Style or ES_MULTILINE;
    if vsMultiline in FLink.FNode.States then
      Style := Style and not (ES_AUTOHSCROLL or WS_HSCROLL) or WS_VSCROLL or ES_AUTOVSCROLL;
    if tsUseThemes in FLink.FTree.TreeStates then
    begin
      Style := Style and not WS_BORDER;
      ExStyle := ExStyle or WS_EX_CLIENTEDGE;
    end
    else
    begin
      Style := Style or WS_BORDER;
      ExStyle := ExStyle and not WS_EX_CLIENTEDGE;
    end;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSStringEdit.Release;

begin
  if HandleAllocated then
    PostMessage(Handle, CM_RELEASE, 0, 0);
end;

//----------------- TRSFieldEditLink ------------------------------------------------------------------------------------

constructor TRSFieldEditLink.Create;

begin
  inherited;
  FEdit := TRSStringEdit.Create(Self);
  with FEdit do
  begin
    Visible := False;
    BorderStyle := bsSingle;
    AutoSize := False;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

destructor TRSFieldEditLink.Destroy;

begin
  FEdit.Release;
  inherited;
end;

//----------------------------------------------------------------------------------------------------------------------

function TRSFieldEditLink.BeginEdit: Boolean;

// Notifies the edit link that editing can start now. Descentants may cancel node edit
// by returning False.

begin
  Result := not FStopping;
  if Result then
  begin
    FEdit.Show;
    FEdit.SelectAll;
    FEdit.SetFocus;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSFieldEditLink.SetEdit(const Value: TRSStringEdit);

begin
  if Assigned(FEdit) then
    FEdit.Free;
  FEdit := Value;
end;

//----------------------------------------------------------------------------------------------------------------------

function TRSFieldEditLink.CancelEdit: Boolean;

begin
  Result := not FStopping;
  if Result then
  begin
    FStopping := True;
    FEdit.Hide;
    FTree.CancelEditNode;
    FEdit.FLink := nil;
    FEdit.FRefLink := nil;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

function TRSFieldEditLink.EndEdit: Boolean;

begin
  Result := not FStopping;
  if Result then
  try
    FStopping := True;
    if FEdit.Modified then
      FTree.Text[FNode, FColumn] := FEdit.Text;
    FEdit.Hide;
    FEdit.FLink := nil;
    FEdit.FRefLink := nil;
  except
    FStopping := False;
    raise;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

function TRSFieldEditLink.GetBounds: TRect;

begin
  Result := FEdit.BoundsRect;
end;

//----------------------------------------------------------------------------------------------------------------------

function TRSFieldEditLink.PrepareEdit(Tree: TBaseVirtualTree; Node: PVirtualNode; Column: TColumnIndex): Boolean;

// Retrieves the true text bounds from the owner tree.

var
  Text: WideString;

begin
  Result := Tree is TMySQLRSGrid;
  if Result then
  begin
    FTree := Tree as TMySQLRSGrid;
    FNode := Node;
    FColumn := Column;
    // Initial size, font and text of the node.
    FTree.GetTextInfo(Node, Column, FEdit.Font, FTextBounds, Text);
    FEdit.Font.Color := clBlack;
    FEdit.Parent := Tree;
    FEdit.RecreateWnd;
    FEdit.HandleNeeded;
    FEdit.Text := Text;

    if Column <= NoColumn then
    begin
      FEdit.BidiMode := FTree.BidiMode;
      FAlignment := FTree.Alignment;
    end
    else
    begin
      FEdit.BidiMode := FTree.Header.Columns[Column].BidiMode;
      FAlignment := FTree.Header.Columns[Column].Alignment;
    end;

    if FEdit.BidiMode <> bdLeftToRight then
      ChangeBidiModeAlignment(FAlignment);
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSFieldEditLink.ProcessMessage(var Message: TMessage);

begin
  FEdit.WindowProc(Message);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TRSFieldEditLink.SetBounds(R: TRect);

// Sets the outer bounds of the edit control and the actual edit area in the control.

var
  Offset: Integer;

begin
  if not FStopping then
  begin
    with R do
    begin
      // Set the edit's bounds but make sure there's a minimum width and the right border does not
      // extend beyond the parent's left/right border.
      if Left < 0 then
        Left := 0;
      if Right - Left < 30 then
      begin
        if FAlignment = taRightJustify then
          Left := Right - 30
        else
          Right := Left + 30;
      end;
      if Right > FTree.ClientWidth then
        Right := FTree.ClientWidth;
      FEdit.BoundsRect := R;

      // The selected text shall exclude the text margins and be centered vertically.
      // We have to take out the two pixel border of the edit control as well as a one pixel "edit border" the
      // control leaves around the (selected) text.
      R := FEdit.ClientRect;
      Offset := 2;
      if tsUseThemes in FTree.TreeStates then
        Inc(Offset);
      InflateRect(R, -FTree.TextMargin + Offset, Offset);
      if not (vsMultiline in FNode.States) then
        OffsetRect(R, 0, FTextBounds.Top - FEdit.Top);

      SendMessage(FEdit.Handle, EM_SETRECTNP, 0, Integer(@R));
    end;
  end;
end;



end.
