unit MyxError;

interface

uses
  gnugettext, Classes, SysUtils, Forms, AuxFuncs, myx_public_interface;

type
  EMyxError = class(Exception)
  end;

  EMyxSystemError = class(Exception)
    constructor Create(Message: WideString; ErrorNr: integer); reintroduce;
  public
    ErrorNr: WideString;
    ErrorTxt: WideString;
  end;

  EMyxCriticalError = class(Exception)
  end;

  EMyxSQLError = class(Exception)
    constructor Create(Message: WideString; ErrorNr: integer; ErrorTxt: WideString); reintroduce;
  public
    ErrorNr: WideString;
    ErrorTxt: WideString;
  end;

  EMyxLibraryError = class(Exception)
    constructor Create(ErrorTxt: WideString; ErrorNr: integer; ErrorNrParam: WideString);
    function GetErrorNrText: WideString; virtual; abstract;
  public
    ErrorTxt: WideString;
    ErrorNr: integer;
    ErrorNrParam: WideString;
  end;

  EMyxCommonLibraryError = class(EMyxLibraryError)
    function GetErrorNrText: WideString; override;
  end;

procedure ShowError(E: Exception);


implementation


constructor EMyxSQLError.Create(Message: WideString; ErrorNr: integer; ErrorTxt: WideString);
begin
  self.Message:=Message;
  self.ErrorNr:=IntToStr(ErrorNr);
  self.ErrorTxt:=ErrorTxt;
end;

constructor EMyxSystemError.Create(Message: WideString; ErrorNr: integer);
begin
  self.Message:=Message;
  self.ErrorNr:=IntToStr(ErrorNr);
  self.ErrorTxt:=SysErrorMessage(ErrorNr);
end;

constructor EMyxLibraryError.Create(ErrorTxt: WideString;
  ErrorNr: integer; ErrorNrParam: WideString);
begin
  self.ErrorNr:=ErrorNr;
  self.ErrorTxt:=ErrorTxt;
  self.ErrorNrParam:=ErrorNrParam;
end;

function EMyxCommonLibraryError.GetErrorNrText: WideString;
begin
  case MYX_LIB_ERROR(ErrorNr) of
    MYX_ERROR_CANT_OPEN_FILE:
      Result:=WideFormat(_('The File %s cannot be opened.'), [ErrorNrParam]);
    MYX_ERROR_CANT_CONNECT_TO_INSTANCE:
      Result:=_('A connection to the server cannot be stablished.');
    MYX_XML_PARSE_ERROR:
      Result:=WideFormat(_('An error occured while parsing the XML file %s.'), [ErrorNrParam]);
    MYX_XML_NO_VALID_DOCUMENT:
      Result:=WideFormat(_('An error occured while validating the XML file %s.'), [ErrorNrParam]);
    MYX_XML_EMPTY_DOCUMENT:
      Result:=WideFormat(_('The XML file %s is empty.'), [ErrorNrParam]);
    MYX_SQL_ERROR:
      Result:=_('An SQL error occured.');
    MYX_STOP_EXECUTION:
      Result:=_('The execution was stopped.');
  else
    self.Message:='';
  end;
end;

procedure ShowError(E: Exception);
var res: integer;
begin
  if(not(Application.Terminated))then
  begin
    if(E is EMyxCriticalError)then
    begin
      res:=ShowModalDialog(Application.Title+' '+_('Critical Error'),
        Format(_('The following Exception occurred:'+#13#10+'%s'+#13#10#13#10+
        'Please choose whether to continue %s, terminate the application '+
        'or restart the program.'), [E.Message, Application.Title]),
        myx_mtError, _('Continue')+#13#10+_('Terminate')+#13#10+_('Restart'));

      if(res=2)then
        Application.Terminate
      else if(res=3)then
      begin
        CreateProz(Application.ExeName);
        Application.Terminate;
      end;
    end
    else if(E is EMyxSQLError)then
    begin
      ShowModalDialog(Application.Title+' '+_('SQL Error'), E.Message+#13#10#13#10+
        Format(_('MySQL Error Nr.%s'+#13#10+'%s'),
          [EMyxSQLError(E).ErrorNr, EMyxSQLError(E).ErrorTxt]),
        myx_mtError, _('OK'));
    end
    else if(E is EMyxSystemError)then
    begin
      ShowModalDialog(Application.Title+' '+_('System Error'), E.Message+#13#10#13#10+
        'System Error Nr.'+EMyxSystemError(E).ErrorNr+#13#10+
        EMyxSystemError(E).ErrorTxt, myx_mtError, _('OK'));
    end
    else if(E is EMyxLibraryError)then
    begin
      ShowModalDialog(Application.Title+' '+_('Library Error'),
        EMyxLibraryError(E).ErrorTxt+#13#10#13#10+
        WideFormat(_('Library Error Nr.%d'+#13#10+'%s'),
        [EMyxLibraryError(E).ErrorNr, EMyxLibraryError(E).GetErrorNrText]),
        myx_mtError, _('OK'));
    end
    {else if(E is EMyxLibraryError)then
    begin
      ShowModalDialog(Application.Title+' '+_('Library Error'), E.Message+#13#10#13#10+
        Format(_('Library Error Nr.%s'+#13#10+'%s'),
          EMyxLibraryError(E).ErrorNr, EMyxLibraryError(E).ErrorTxt),
        myx_mtError, _('OK'));
    end}
    else
    begin
      ShowModalDialog(Application.Title+' Error', 'The following Exception occurred:'+#13#10#13#10+
        E.Message, myx_mtError, 'OK');
    end;
  end;
end;


end.
