unit Options;

interface

uses
  Classes, TntClasses, Forms, TntForms,SysUtils, Messages, ComCtrls,
  TntComCtrls, myx_public_interface, AuxFuncs, Types, StrUtils;

const
  WM_OptionsChanged = WM_USER+100;

type
  TDoPageContentChanged = procedure (Sender: TObject) of object;

  TMyxOptions = Class(TObject)
    constructor Create; virtual;
    destructor Destroy; override;

    procedure LoadOptions; virtual; abstract;
    procedure StoreOptions; virtual; abstract;

    function CheckBoolStr(s: WideString): Boolean;
    procedure AddParam(OptionGroup: TMYX_OPTION_GROUP;
      name: WideString; value: WideString);

    procedure AddWindowPos(Win: TTntForm);
    function RestoreWindowPos(Win: TTntForm; DoSize: Boolean = True): Boolean;
  public
    WindowPosList: TTntStringList;
    IgnoreWarningsList: TTntStringList;
  end;

  TMyxWindowPos = class
    constructor Create; overload;
    constructor Create(AsWideString: WideString); overload;
    function AsWideString: WideString;
  public
    Name: WideString;
    State: TWindowState;
    Pos: TPoint;
    Size: TPoint;
  end;

  TMYXCommonOptions = Class(TMyxOptions)
    procedure LoadOptions; override;
    procedure StoreOptions; override;
  public
    ConnectionToUse: WideString;
    ConnectionUsername,
    ConnectionPassword,
    ConnectionHost,
    ConnectionPort,
    ConnectionSchema: WideString;

    AvailableLanguages: WideString;
    AvailableLanguageCodes: WideString;
    AvailablePasswordStorageTypes: WideString;

    StoreWindowsPositions,
    ShowTipOfDay: Boolean;

    Language: WideString;

    XPStyleEnabled: Boolean;

    //FileDialogPaths: TTntStringList;

    XMLDir, UserDataDir: WideString;

    PasswordStorageType: MYX_PASSWORD_STORAGE_TYPE;

    EditorTableShowSQLBeforeApplying,
    EditorTableAllColumnsNotNullPerDef,
    EditorTableIntegerUnsignedPerDef: Boolean;
    EditorTablePKDataType,
    EditorTableDefColumnDataType,
    EditorTablePKAutoNaming,
    EditorTableIndexAutoNaming,
    EditorTableFKAutoNaming: WideString;

    DefaultFontName,
    DataFontName,
    CodeFontName: WideString;
    DefaultFontSize,
    DataFontSize,
    CodeFontSize,
    CodeFontWidth,
    CodeFontCharset: Integer;
  end;

  TApplicationOptionsForm = class(TTntForm)
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetControls(PageNr: integer); virtual; abstract;
    procedure ApplyChanges(PageNr: integer); virtual; abstract;
  public
    { Public declarations }
    OptionsImgNames: TTntStringList;
    DockOptionPageControl: TTntPageControl;
    DoPageContentChanged: TDoPageContentChanged;
  end;

function GetWinPos(Win: TTntForm): TMyxWindowPos;
procedure SetWinPos(Win: TTntForm; WinPos: TMyxWindowPos; DoSize: Boolean = True);

var
  MYXCommonOptions: TMYXCommonOptions;

implementation

uses ApplicationDataModule;

constructor TMyxOptions.Create;
begin
  inherited Create;

  WindowPosList:=TTntStringList.Create;
  IgnoreWarningsList:=TTntStringList.Create;
  LoadOptions;
end;

destructor TMyxOptions.Destroy;
var i: integer;
begin
  StoreOptions;

  for i:=0 to WindowPosList.Count-1 do
    WindowPosList.Objects[i].Free;
  WindowPosList.Free;

  IgnoreWarningsList.Free;

  inherited Destroy;
end;

// Help functions

function TMyxOptions.CheckBoolStr(s: WideString): Boolean;
begin
  Result:=(StrToIntDef(s, 1)=1);
end;

procedure TMyxOptions.AddParam(OptionGroup: TMYX_OPTION_GROUP;
  name: WideString; value: WideString);
begin
  OptionGroup.name_value_pairs.Add(
    TMYX_NAME_VALUE_PAIR.create(name, value));
end;

procedure TMyxOptions.AddWindowPos(Win: TTntForm);
begin
  while(WindowPosList.IndexOf(Win.Name)>-1)do
    WindowPosList.Delete(WindowPosList.IndexOf(Win.Name));

  WindowPosList.AddObject(Win.Name, GetWinPos(Win));

  StoreOptions;
end;

function TMyxOptions.RestoreWindowPos(Win: TTntForm; DoSize: Boolean): Boolean;
var i: integer;
  WinPos: TMyxWindowPos;
begin
  WinPos:=nil;

  if(MYXCommonOptions.StoreWindowsPositions)then
  begin
    for i:=0 to WindowPosList.Count-1 do
      if(WindowPosList[i]=Win.Name)then
      begin
        WinPos:=TMyxWindowPos(WindowPosList.Objects[i]);
        break;
      end;
  end;

  if(WinPos<>nil)then
  begin
    SetWinPos(Win, WinPos, DoSize);
    Result:=True;
  end
  else
  begin
    Win.Position:=poScreenCenter;
    Result:=False;
  end;
end;

// -------------------------------------------------
// TMyxWindowPos

function GetNextNumValue(var S: WideString): integer;
var p, l: Integer;
begin
  p:=Pos('=', S);
  l:=Pos(';', S);
  result:=StrToIntDef(Copy(S, p+1, l-p-1), 0);
  S:=Copy(S, l+1, Length(S));
end;

constructor TMyxWindowPos.Create;
begin
  inherited;
end;

constructor TMyxWindowPos.Create(AsWideString: WideString);
begin
  inherited Create;

  Pos.X:=GetNextNumValue(AsWideString);
  Pos.Y:=GetNextNumValue(AsWideString);

  Size.X:=GetNextNumValue(AsWideString);
  Size.Y:=GetNextNumValue(AsWideString);

  State:=TWindowState(GetNextNumValue(AsWideString));
end;

function TMyxWindowPos.AsWideString: WideString;
var s: WideString;
begin
  s:=s+'x='+IntToStr(Pos.X)+'; y='+IntToStr(Pos.Y)+
    '; w='+IntToStr(Size.X)+'; h='+IntToStr(Size.Y)+
    '; s='+IntToStr(Ord(State))+';';

  Result:=s;
end;

function GetWinPos(Win: TTntForm): TMyxWindowPos;
var WinPos: TMyxWindowPos;
  //P: TPoint;
begin
  WinPos:=TMyxWindowPos.Create;

  WinPos.Name:=Win.Name;
  WinPos.State:=Win.WindowState;

  //P:=win.ClientToScreen(Point(0, 0));

  WinPos.Pos:=Point(Win.Left, Win.Top);
  WinPos.Size:=Point(Win.Width, Win.Height);


  Result:=WinPos;
end;

procedure SetWinPos(Win: TTntForm; WinPos: TMyxWindowPos; DoSize: Boolean);
begin
  if(WinPos.Pos.X<0)then
    Win.Left:=0
  else if(WinPos.Pos.X>=Screen.DesktopWidth-10)then
    Win.Left:=Screen.Width-10
  else
    Win.Left:=WinPos.Pos.X;

  if(WinPos.Pos.Y<0)then
    Win.Top:=0
  else if(WinPos.Pos.Y>=Screen.DesktopHeight-10)then
    Win.Left:=Screen.Height-10
  else
    Win.Top:=WinPos.Pos.Y;

  if(DoSize)then
  begin
    if(WinPos.Size.X<100)then
      Win.Width:=100
    else
      Win.Width:=WinPos.Size.X;

    if(WinPos.Size.Y<40)then
      Win.Height:=40
    else
      Win.Height:=WinPos.Size.Y;
  end;

  Win.WindowState:=WinPos.State;
end;

// -------------------------------------------------
// TMYXCommonOptions

procedure TMYXCommonOptions.LoadOptions;
var POptions: PMYX_APPLICATION_OPTIONS;
  Options: TMYX_APPLICATION_OPTIONS;
  i, j: integer;
  error: MYX_LIB_ERROR;
  OptionGroupName, OptionName, OptionValue: WideString;
  ExePath: WideString;
begin
  ExePath:=ExtractFilePath(Application.ExeName);

  //Detect if XPStyle is enabled
  XPStyleEnabled:=getXPStyleEnabled;

  AvailableLanguages:='English';//#13#10'German';
  AvailableLanguageCodes:='en';//#13#10'de';
  AvailablePasswordStorageTypes:='Plaintext'#13#10'Obscured';
  if(Not(IsWinNT))then
    AvailablePasswordStorageTypes:=
      AvailablePasswordStorageTypes+#13#10'OS Specific';

  XMLDir:=ExtractFilePath(Application.ExeName)+'XML\';
  if(UserDataDir='')then
    UserDataDir:=GetApplicationDataDir+'MySQL\';
  ForceDirectories(UserDataDir);

  //------------------------------------------------------------
  //Initialize Values
  Language:='en';

  //Initialize Default Values
  PasswordStorageType:=MYX_PASSWORD_NOT_STORED;

  StoreWindowsPositions:=True;
  ShowTipOfDay:=False;

  EditorTableShowSQLBeforeApplying:=True;
  EditorTableAllColumnsNotNullPerDef:=True;
  EditorTableIntegerUnsignedPerDef:=True;
  EditorTablePKDataType:='INTEGER';
  EditorTableDefColumnDataType:='VARCHAR(45)';
  EditorTablePKAutoNaming:='id%tablename%';
  EditorTableIndexAutoNaming:='Index_%nr%';
  EditorTableFKAutoNaming:='FK_%tablename%_%nr%';

  DefaultFontName:='MS Sans Serif';
  DataFontName:='MS Sans Serif';
  CodeFontName:='Bitstream Vera Sans Mono';
  CodeFontCharset:=1; //DEFAULT_CHARSET
  DefaultFontSize:=8;
  DataFontSize:=8;
  CodeFontSize:=8;
  CodeFontWidth:=8;

  //------------------------------------------------------------
  //Read Options file
  if(FileExists(UserDataDir+'mysqlx_common_options.xml'))then
  begin
    POptions:=myx_get_application_options(
      UserDataDir+'mysqlx_common_options.xml',
      @error);
    try
      if(error<>MYX_NO_ERROR)then
      begin
        ShowModalDialog('XML Error',
          'Error while loading Options file '''+
          UserDataDir+'mysqlx_common_options.xml'+''''#13#10+
          'Error Nr.:'+IntToStr(Ord(error)),
          myx_mtError);
      end
      else
      begin
        Options:=TMYX_APPLICATION_OPTIONS.Create(POptions);
        try
          for i:=0 to Options.option_groups.Count-1 do
            for j:=0 to Options.option_groups[i].name_value_pairs.Count-1 do
            begin
              OptionGroupName:=Options.option_groups[i].name;
              OptionName:=Options.option_groups[i].name_value_pairs[j].name;
              OptionValue:=Options.option_groups[i].name_value_pairs[j].value;

              if(CompareText(OptionGroupName, 'General')=0)then
              begin
                if(CompareText(OptionName, 'Language')=0)then
                  Language:=OptionValue
                else if(CompareText(OptionName, 'PasswordStorageType')=0)then
                  PasswordStorageType:=MYX_PASSWORD_STORAGE_TYPE(StrToIntDef(OptionValue, 1))
                else if(CompareText(OptionName, 'StoreWindowsPositions')=0)then
                  StoreWindowsPositions:=(OptionValue='1')
                else if(CompareText(OptionName, 'ShowTipOfDay')=0)then
                  ShowTipOfDay:=(OptionValue='1')
                else if(CompareText(OptionName, 'IgnoreWarningsList')=0)then
                  IgnoreWarningsList.Text:=
                    AnsiReplaceText(OptionValue, '#13#10', #13#10)
                else
                  ;

              end
              else
              if(CompareText(OptionGroupName, 'Editors')=0)then
              begin
                if(CompareText(OptionName, 'EditorTableShowSQLBeforeApplying')=0)then
                  EditorTableShowSQLBeforeApplying:=(OptionValue='1')
                else if(CompareText(OptionName, 'EditorTableAllColumnsNotNullPerDef')=0)then
                  EditorTableAllColumnsNotNullPerDef:=(OptionValue='1')
                else if(CompareText(OptionName, 'EditorTableIntegerUnsignedPerDef')=0)then
                  EditorTableIntegerUnsignedPerDef:=(OptionValue='1')
                else if(CompareText(OptionName, 'EditorTablePKDataType')=0)then
                  EditorTablePKDataType:=OptionValue
                else if(CompareText(OptionName, 'EditorTablePKAutoNaming')=0)then
                  EditorTablePKAutoNaming:=OptionValue
                else if(CompareText(OptionName, 'EditorTableIndexAutoNaming')=0)then
                  EditorTableIndexAutoNaming:=OptionValue
                else if(CompareText(OptionName, 'EditorTableFKAutoNaming')=0)then
                  EditorTableFKAutoNaming:=OptionValue
                else
                  ;
              end
              else
              if(CompareText(OptionGroupName, 'Fonts')=0)then
              begin
                if(CompareText(OptionName, 'DefaultFontName')=0)then
                  DefaultFontName:=OptionValue
                else if(CompareText(OptionName, 'DataFontName')=0)then
                  DataFontName:=OptionValue
                else if(CompareText(OptionName, 'CodeFontName')=0)then
                  CodeFontName:=OptionValue
                else if(CompareText(OptionName, 'DefaultFontSize')=0)then
                  DefaultFontSize:=StrToIntDef(OptionValue, 8)
                else if(CompareText(OptionName, 'DataFontSize')=0)then
                  DataFontSize:=StrToIntDef(OptionValue, 8)
                else if(CompareText(OptionName, 'CodeFontWidth')=0)then
                  CodeFontWidth:=StrToIntDef(OptionValue, 8)
                else if(CompareText(OptionName, 'CodeFontSize')=0)then
                  CodeFontSize:=StrToIntDef(OptionValue, 8)
                else if(CompareText(OptionName, 'CodeFontCharset')=0)then
                  CodeFontCharset:=StrToIntDef(OptionValue, 8)
                else
                  ;
              end
              else if(CompareText(OptionGroupName, 'WindowPos')=0)then
              begin
                WindowPosList.AddObject(OptionName, TMyxWindowPos.Create(OptionValue));
              end;
            end;
        finally
          Options.Free;
        end;
      end;
    finally
      //Free Application Options
      myx_free_application_options(POptions);
    end;
  end;


  AuxFuncs.DefaultFontName:=DefaultFontName;
  AuxFuncs.DataFontName:=DataFontName;
  AuxFuncs.DefaultFontSize:=DefaultFontSize;
  AuxFuncs.DataFontSize:=DataFontSize;
end;

procedure TMYXCommonOptions.StoreOptions;
var Options: TMYX_APPLICATION_OPTIONS;
  OptionGroup: TMYX_OPTION_GROUP;
  ExePath: WideString;
  i: integer;
begin
  ExePath:=ExtractFilePath(Application.ExeName);

  //Create Application Options
  Options:=TMYX_APPLICATION_OPTIONS.create;
  try
    //-----------------------------------------------
    //Create Option Group
    OptionGroup:=TMYX_OPTION_GROUP.create('General');
    Options.option_groups.Add(OptionGroup);

    //Add Values
    AddParam(OptionGroup, 'Language', Language);

    AddParam(OptionGroup, 'PasswordStorageType',
      IntToStr(Ord(PasswordStorageType)));

    AddParam(OptionGroup, 'StoreWindowsPositions',
      IntToStr(Ord(StoreWindowsPositions)));

    AddParam(OptionGroup, 'ShowTipOfDay',
      IntToStr(Ord(ShowTipOfDay)));

    AddParam(OptionGroup, 'Language', Language);

    AddParam(OptionGroup, 'IgnoreWarningsList',
      AnsiReplaceText(IgnoreWarningsList.Text, #13#10, '#13#10'));

    //------------------------------------------------------
    //Create Option Group
    OptionGroup:=TMYX_OPTION_GROUP.create('Editors');
    Options.option_groups.Add(OptionGroup);

    AddParam(OptionGroup, 'EditorTableShowSQLBeforeApplying',
      IntToStr(Ord(EditorTableShowSQLBeforeApplying)));

    AddParam(OptionGroup, 'EditorTableAllColumnsNotNullPerDef',
      IntToStr(Ord(EditorTableAllColumnsNotNullPerDef)));

    AddParam(OptionGroup, 'EditorTableIntegerUnsignedPerDef',
      IntToStr(Ord(EditorTableIntegerUnsignedPerDef)));

    AddParam(OptionGroup, 'EditorTablePKDataType', EditorTablePKDataType);

    AddParam(OptionGroup, 'EditorTablePKAutoNaming', EditorTablePKAutoNaming);

    AddParam(OptionGroup, 'EditorTableIndexAutoNaming', EditorTableIndexAutoNaming);

    AddParam(OptionGroup, 'EditorTableFKAutoNaming', EditorTableFKAutoNaming);

    //------------------------------------------------------
    //Create Option Group
    OptionGroup:=TMYX_OPTION_GROUP.create('Fonts');
    Options.option_groups.Add(OptionGroup);

    AddParam(OptionGroup, 'DefaultFontName', DefaultFontName);

    AddParam(OptionGroup, 'DataFontName', DataFontName);

    AddParam(OptionGroup, 'CodeFontName', CodeFontName);

    AddParam(OptionGroup, 'DefaultFontSize', IntToStr(DefaultFontSize));

    AddParam(OptionGroup, 'DataFontSize', IntToStr(DataFontSize));

    AddParam(OptionGroup, 'CodeFontWidth', IntToStr(CodeFontWidth));

    AddParam(OptionGroup, 'CodeFontSize', IntToStr(CodeFontSize));

    AddParam(OptionGroup, 'CodeFontCharset', IntToStr(CodeFontCharset));

    //-----------------------------------------------
    //Create Option Group
    OptionGroup:=TMYX_OPTION_GROUP.create('WindowPos');
    Options.option_groups.Add(OptionGroup);

    //Store all window positions
    for i:=0 to WindowPosList.Count-1 do
      AddParam(OptionGroup, WindowPosList[i],
        TMyxWindowPos(WindowPosList.Objects[i]).AsWideString);

    //-----------------------------------------------
    //Store Options to file
    myx_store_application_options(Options.get_record_pointer,
      UserDataDir+'mysqlx_common_options.xml');
  finally
    Options.Free;
  end;
end;

constructor TApplicationOptionsForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  OptionsImgNames:=TTntStringList.Create;
  DockOptionPageControl:=nil;
  DoPageContentChanged:=nil;
end;

destructor TApplicationOptionsForm.Destroy;
begin
  OptionsImgNames.Free;

  inherited Destroy;
end;

end.
