unit SchemataTreeView;

interface

uses
  gnugettext, Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs, AdvancedEdit, StdCtrls, ExtCtrls, ComCtrls, Menus,
  MySQLConnection, AuxFuncs, myx_public_interface, MyxError, ImgList,
  EditorTable, VirtualTrees, PNGImage, Contnrs, TntClasses, TntForms,
  TntMenus, TntStdCtrls, TntExtCtrls;

type
  TSchemaVirtualNode = class;

  TSchemataFrame = class(TTntFrame)
    TopPnl: TTntPanel;
    SchemataLbl: TTntLabel;
    SpacerPnl: TTntPanel;
    AdvancedEdit: TAdvancedEditFrame;
    SchemaSearchPopupMenu: TTntPopupMenu;
    SchemaTreeViewPopupMenu: TTntPopupMenu;
    SchemataAssetsImageList: TImageList;
    CatalogVST: TVirtualStringTree;
    AllcatalogsMI: TTntMenuItem;
    RefreshCatalogsSchemataListMI: TTntMenuItem;
    N3: TTntMenuItem;
    DropTableMI: TTntMenuItem;
    DropSchemaMI: TTntMenuItem;
    N2: TTntMenuItem;
    CreateNewTableMI: TTntMenuItem;
    CreateNewSchemaMI: TTntMenuItem;
    N1: TTntMenuItem;
    EditTableMI: TTntMenuItem;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure ReloadSchemaTree;
    procedure RefreshSchemaTree(searchStr: WideString = ''; ReloadSchemata: Boolean = False);
    procedure AllCatalogsMIClick(Sender: TObject);
    procedure RefreshCatalogsSchemataListMIClick(Sender: TObject);
    procedure AdvancedEditSearchEdChange(Sender: TObject);
    procedure RebuildSchemaTree(MySQLMajorVersion: integer; CatalogList: TMYX_CATALOGS;
      Catalog: TMYX_CATALOG; searchStr: WideString);

    procedure GetCatalogList(Sender: TObject);
    procedure RefreshCatalogList(Sender: TObject);

    procedure FetchSchemaTables(Sender: TObject);
    procedure SchemaTablesFetched(Sender: TObject);
    procedure RefreshTableNames(SchemaNode: TSchemaVirtualNode);

    procedure FetchSchemaIndices(Sender: TObject);
    procedure SchemaIndicesFetched(Sender: TObject);
    procedure RefreshIndicesNames(SchemaNode: TSchemaVirtualNode);

    procedure FillSpacerPnl;
    procedure DropSchemaMIClick(Sender: TObject);
    procedure CreateNewSchemaMIClick(Sender: TObject);
    procedure DropTableMIClick(Sender: TObject);
    procedure SchemaTreeViewPopupMenuPopup(Sender: TObject);
    procedure CreateNewTableMIClick(Sender: TObject);

    procedure EditTable(Catalog: WideString;
      Schema: WideString; Table: WideString);
    procedure EditTableMIClick(Sender: TObject);
    procedure CatalogVSTAfterCellPaint(Sender: TBaseVirtualTree;
      TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
      CellRect: TRect);
    procedure CatalogVSTGetText(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
      var CellText: WideString);
    procedure CatalogVSTGetImageIndex(Sender: TBaseVirtualTree;
      Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
      var Ghosted: Boolean; var ImageIndex: Integer);
    procedure CatalogVSTMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CatalogVSTExpanding(Sender: TBaseVirtualTree;
      Node: PVirtualNode; var Allowed: Boolean);
    procedure CatalogVSTChange(Sender: TBaseVirtualTree;
      Node: PVirtualNode);
    procedure ClearCatalogTree;

    procedure DoTableEditorApplyedChanges(Sender: TObject);

    procedure GetExpandedNodes(NodesList: TTntStringList;
      Tree: TVirtualStringTree; Node: PVirtualNode);
    function ReExpandedNodes(NodesList: TTntStringList; index: integer;
      Tree: TVirtualStringTree; Node: PVirtualNode): integer;
    function GetCatalogVSTFocusedObject(MatchClass: TClass = nil): TObject;
    procedure CatalogVSTPaintText(Sender: TBaseVirtualTree;
      const TargetCanvas: TCanvas; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType);
  private
    { Private declarations }
    LastCatalogName, LastSchemaName: WideString;

    FCreateTable,
    FEditTable,
    FSchemaTreeReloaded: TNotifyEvent;

    TreeSpacerImgList: TImageList;

    TreeBtnOpenPNGImg,
    TreeBtnClosedPNGImg: TPNGObject;

    CatalogPNGImg,
    SchemaPNGImg,
    TablePNGImg,
    ColumnPKPNGImg,
    ColumnPNGImg,
    IndexPNGImg,
    UserPNGImg: TPNGObject;

    ExpandedNodesList: TTntStringList;
    ThreadsDisabled: Boolean;

    FPaintDefaultSchemaBold: Boolean;

    FDefaultSchema: WideString;
  public
    { Public declarations }
    CatalogList: TMYX_CATALOGS;
    SchemasToAdd: TTntStringList;
    AddedSchemataList: TObjectList;

    CurrentCatalog: TMYX_CATALOG;
    CurrentSchema: TMYX_SCHEMA;
    CurrentSchemaNodes: TList;
    CurrentTable: TMYX_SCHEMA_TABLE;

    //User must not expand SchemaTree manually
    LockSchemaTreeExpansion: Boolean;
    LockSchemaTreeSelection: Boolean;

    MySQLConn: TMySQLConn;

    EditorTableForm: TEditorTableForm;

    ExpandSchemaOnLaunch: WideString;

    FShowSchemaAssets,
    ShowTablesOnSchemaExpansion,
    FPopupMenuEditEntries: Boolean;

    SelectTableAfterTablesNodeExpand: WideString;

    property DefaultSchema: WideString read FDefaultSchema write FDefaultSchema;
  protected
    procedure SetShowSchemaAssets(ShowSchemaAssets: Boolean);
    function GetShowSchemaAssets: Boolean;

    procedure SetPopupMenuEditEntries(PopupMenuEditEntries: Boolean);
    function GetPopupMenuEditEntries: Boolean;
  published
    property PopupMenuEditEntries: Boolean read GetPopupMenuEditEntries write SetPopupMenuEditEntries;
    property ShowSchemaAssets: Boolean read GetShowSchemaAssets write SetShowSchemaAssets;
    property OnCreateTable: TNotifyEvent read FCreateTable write FCreateTable;
    property OnEditTable: TNotifyEvent read FEditTable write FEditTable;
    property OnSchemaTreeReloaded: TNotifyEvent read FSchemaTreeReloaded write FSchemaTreeReloaded;
    property PaintDefaultSchemaBold: Boolean read FPaintDefaultSchemaBold write FPaintDefaultSchemaBold;
  end;

  TSchemaSubNodeType = (TSNTables, TSNIndices, TSNViews,
    TSNStoredProcedures, TSNEvents, TSNDevDocs, TSNUsers);

  TSchemaSubNode = class(TObject)
    constructor Create(Catalog: TMYX_CATALOG;
      Schema: TMYX_SCHEMA; SubNodeType: TSchemaSubNodeType);
  private
    FCatalog: TMYX_CATALOG;
    FSchema: TMYX_SCHEMA;
    FSubNodeType: TSchemaSubNodeType;
  published
    property Catalog: TMYX_CATALOG read FCatalog;
    property Schema: TMYX_SCHEMA read FSchema;
    property SubNodeType: TSchemaSubNodeType read FSubNodeType;
  end;

  TSchemaVirtualNode = class(TObject)
    constructor Create(Node: PVirtualNode; Schema: TMYX_SCHEMA);
  private
    FSchema: TMYX_SCHEMA;
  published
    property Schema: TMYX_SCHEMA read FSchema;
  public
    Node: PVirtualNode;
  end;


implementation

uses ApplicationDataModule;

{$R *.dfm}

constructor TSchemaSubNode.Create(Catalog: TMYX_CATALOG; Schema: TMYX_SCHEMA; SubNodeType: TSchemaSubNodeType);
begin
  FCatalog:=Catalog;
  FSchema:=Schema;
  FSubNodeType:=SubNodeType;
end;

//------------------------------------------------------------------------------

constructor TSchemaVirtualNode.Create(Node: PVirtualNode; Schema: TMYX_SCHEMA);
begin
  self.Node:=Node;
  FSchema:=Schema;
end;

//------------------------------------------------------------------------------

constructor TSchemataFrame.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  MySQLConn:=nil;
  CatalogList:=nil;
  SchemasToAdd:=TTntStringList.Create;
  AddedSchemataList:=TObjectList.Create;

  ThreadsDisabled:=False;

  //Create Lists
  CurrentSchemaNodes:=TList.Create;

  //Init variables
  CurrentCatalog:=nil; //Catalog;
  CurrentSchema:=nil;
  CurrentTable:=nil;
  LockSchemaTreeExpansion:=True;
  LockSchemaTreeSelection:=True;

  FDefaultSchema:='';

  FPaintDefaultSchemaBold:=False;

  ShowTablesOnSchemaExpansion:=False;
  FShowSchemaAssets:=True;
  FPopupMenuEditEntries:=True;

  ExpandSchemaOnLaunch:='';
  SelectTableAfterTablesNodeExpand:='';

  ExpandedNodesList:=TTntStringList.Create;

  EditorTableForm:=nil;
  FCreateTable:=nil;
  FEditTable:=nil;
  FSchemaTreeReloaded:=nil;

  TreeBtnOpenPNGImg:=LoadPNGImageFromResource('tree_button_open');
  TreeBtnClosedPNGImg:=LoadPNGImageFromResource('tree_button_closed');

  CatalogPNGImg:=LoadPNGImageFromResource('catalog_16x16');
  SchemaPNGImg:=LoadPNGImageFromResource('schema_16x16');
  TablePNGImg:=LoadPNGImageFromResource('asset_table_16x16');
  ColumnPKPNGImg:=LoadPNGImageFromResource('column_pk');
  ColumnPNGImg:=LoadPNGImageFromResource('column');
  IndexPNGImg:=LoadPNGImageFromResource('asset_index_16x16');
  UserPNGImg:=LoadPNGImageFromResource('users_16x16');

  TreeSpacerImgList:=GetTransparentImgList(10);
  CatalogVST.Images:=TreeSpacerImgList;

  CatalogVST.NodeDataSize:=Sizeof(Pointer);
end;

destructor TSchemataFrame.Destroy;
begin
  CurrentSchemaNodes.Free;

  if(EditorTableForm<>nil)then
    EditorTableForm.Free;

  if(CatalogList<>nil)then
    CatalogList.Free;

  SchemasToAdd.Free;
  AddedSchemataList.Free;

  TreeSpacerImgList.Free;

  TreeBtnOpenPNGImg.Free;
  TreeBtnClosedPNGImg.Free;

  CatalogPNGImg.Free;
  SchemaPNGImg.Free;
  IndexPNGImg.Free;
  TablePNGImg.Free;
  ColumnPKPNGImg.Free;
  ColumnPNGImg.Free;
  UserPNGImg.Free;

  ExpandedNodesList.Free;

  inherited Destroy;
end;

procedure TSchemataFrame.SetShowSchemaAssets(ShowSchemaAssets: Boolean);
begin
  FShowSchemaAssets:=ShowSchemaAssets;

  if(Not(FShowSchemaAssets)and(Not(ShowTablesOnSchemaExpansion)))then
    CatalogVST.Images:=nil
  else
    CatalogVST.Images:=TreeSpacerImgList;
end;

function TSchemataFrame.GetShowSchemaAssets: Boolean;
begin
  Result:=FShowSchemaAssets;
end;

procedure TSchemataFrame.SetPopupMenuEditEntries(PopupMenuEditEntries: Boolean);
begin
  FPopupMenuEditEntries:=PopupMenuEditEntries;

  if(Not(FPopupMenuEditEntries))then
  begin
    EditTableMI.Visible:=False;
    N1.Visible:=False;
    CreateNewSchemaMI.Visible:=False;
    CreateNewTableMI.Visible:=False;
    N2.Visible:=False;
    DropSchemaMI.Visible:=False;
    DropTableMI.Visible:=False;
    N3.Visible:=False;
  end
  else
  begin
    EditTableMI.Visible:=True;
    N1.Visible:=True;
    CreateNewSchemaMI.Visible:=True;
    CreateNewTableMI.Visible:=True;
    N2.Visible:=True;
    DropSchemaMI.Visible:=True;
    DropTableMI.Visible:=True;
    N3.Visible:=True;
  end;
end;

function TSchemataFrame.GetPopupMenuEditEntries: Boolean;
begin
  Result:=FPopupMenuEditEntries;
end;

procedure TSchemataFrame.FillSpacerPnl;
var Shape: TTntShape;
begin
  Shape:=TTntShape.Create(self);
  Shape.Parent:=SpacerPnl;
  Shape.Align:=alClient;
  Shape.Pen.Color:=clWhite;
end;

procedure TSchemataFrame.ReloadSchemaTree;
begin
  RefreshSchemaTree('', True);

  if(Assigned(FSchemaTreeReloaded))then
    FSchemaTreeReloaded(self);
end;

procedure TSchemataFrame.GetExpandedNodes(NodesList: TTntStringList;
  Tree: TVirtualStringTree; Node: PVirtualNode);
var caption, state: WideString;
  NodeData: ^TObject;
begin
  if(Node=nil)then
    Exit;

  if(Tree.Expanded[Node])or
    (Tree.Selected[Node])or
    (Tree.FocusedNode=Node)then
  begin
    caption:='';
    if(Assigned(Tree.OnGetText))then
      Tree.OnGetText(TBaseVirtualTree(Tree), Node, 0, ttNormal, caption)
    else
      Exit;

    state:='';
    if(Tree.Selected[Node])then
      state:='S'+state;
    if(Tree.Expanded[Node])then
      state:='E'+state;
    if(Tree.FocusedNode=Node)then
      state:='F'+state;

    NodeData:=Tree.GetNodeData(Node);
    if(NodeData<>nil)then
      if(NodeData^<>nil)then
        if(NodeData^ is TMYX_SCHEMA)then
          state:='1'+state
        else if(NodeData^ is TMYX_SCHEMA_TABLE)then
          state:='2'+state
        else if(NodeData^ is TMYX_SCHEMA_TABLE_COLUMN)then
          state:='3'+state;

    if(caption<>'')then
      NodesList.Add(state+'='+caption);
  end;

  //Get next Child
  GetExpandedNodes(NodesList, Tree, Node.FirstChild);

  //Get next Sibling
  GetExpandedNodes(NodesList, Tree, Node.NextSibling);
end;

function TSchemataFrame.ReExpandedNodes(NodesList: TTntStringList; index: integer;
  Tree: TVirtualStringTree; Node: PVirtualNode): integer;
var caption: WideString;
  NodeData: ^TObject;
  NodeType: WideString;
begin
  if(Node=nil)or(index>=NodesList.Count)then
  begin
    Result:=index;
    Exit;
  end;

  caption:='';
  if(Assigned(Tree.OnGetText))then
    Tree.OnGetText(TBaseVirtualTree(Tree), Node, 0, ttNormal, caption)
  else
  begin
    Result:=index;
    Exit;
  end;

  if(caption=NodesList.ValueFromIndex[index])then
  begin
    //Check NodeType
    NodeData:=Tree.GetNodeData(Node);
    NodeType:='-';

    if(NodeData<>nil)then
      if(NodeData^<>nil)then
        if(NodeData^ is TMYX_SCHEMA)then
          NodeType:='1'
        else if(NodeData^ is TMYX_SCHEMA_TABLE)then
          NodeType:='2'
        else if(NodeData^ is TMYX_SCHEMA_TABLE_COLUMN)then
          NodeType:='3';

    if(Pos(NodeType, NodesList.Names[index])>0)then
    begin
      if(Pos('E', NodesList.Names[index])>0)then
      begin
        if(NodeType='1')then
          RefreshTableNames(TSchemaVirtualNode.Create(Node, TMYX_SCHEMA(NodeData^)))
        else
          Tree.Expanded[Node]:=True;
      end;

      if(Pos('S', NodesList.Names[index])>0)then
        Tree.Selected[Node]:=True;

      if(Pos('F', NodesList.Names[index])>0)then
        Tree.FocusedNode:=Node;

      inc(index);
    end;
  end;

  //Get next Child
  index:=ReExpandedNodes(NodesList, index, Tree, Node.FirstChild);

  //Get next Sibling
  index:=ReExpandedNodes(NodesList, index, Tree, Node.NextSibling);

  //If the node was not found, continue with the next
  {if(Found=0)then
  begin
    Result:=ReExpandedNodes(NodesList, StartIndex+1, Tree, Node);
  end
  else}
  Result:=index;
end;

procedure TSchemataFrame.RefreshSchemaTree(searchStr: WideString; ReloadSchemata: Boolean);
begin
  if(Not(MySQLConn.Connected))then
    Exit;

  if(CatalogList=nil)or(ReloadSchemata)then
  begin
    //Store current selected and expanded nodes
    ExpandedNodesList.Clear;
    GetExpandedNodes(ExpandedNodesList,
      CatalogVST, CatalogVST.GetFirst);

    ClearCatalogTree;

    if(CatalogList<>nil)then
    begin
      CatalogList.Free;
      CatalogList:=nil;
    end;

    //(Re-)load CatalogList
    //AllowDuplicatedKindOfData
    MySQLConn.FetchData(KindOfData_CatalogSchema,
      GetCatalogList, RefreshCatalogList,
      nil, _('Fetching Catalogs/Schemata data ...'),
      False, True);
  end
  else
  begin
    //Refresh SchemaTree
    LockSchemaTreeExpansion:=False;
    try
      RebuildSchemaTree(MySQLConn.MySQLMajorVersion,
        CatalogList, CurrentCatalog,
        searchStr);
    finally
      LockSchemaTreeExpansion:=True;
    end;
  end;
end;

procedure TSchemataFrame.GetCatalogList(Sender: TObject);
var catalogs: PMYX_CATALOGS;
begin
  catalogs:=myx_get_catalogs(TFetchDataThread(Sender).MySQLConn.PMySQL);
  if(catalogs=nil)then
    raise EMyxSQLError.Create(_('Could not fetch Catalogs/Schemata data.'),
      myx_mysql_errno(TFetchDataThread(Sender).MySQLConn.PMySQL),
      myx_mysql_error(TFetchDataThread(Sender).MySQLConn.PMySQL));

  try
    CatalogList:=TMYX_CATALOGS.Create(catalogs);
  finally
    myx_free_catalogs(catalogs);
  end;
end;


procedure TSchemataFrame.RefreshCatalogList(Sender: TObject);
var i: integer;
  MenuItem: TMenuItem;
begin
  //Refresh Popup Menu
  while(SchemaSearchPopupMenu.Items.Count>1)do
  begin
    MenuItem:=SchemaSearchPopupMenu.Items[SchemaSearchPopupMenu.Items.Count-1];
    SchemaSearchPopupMenu.Items.Delete(SchemaSearchPopupMenu.Items.Count-1);
    MenuItem.Free;
  end;

  for i:=0 to CatalogList.Catalogs.Count-1 do
  begin
    MenuItem:=TMenuItem.Create(self);
    MenuItem.Name:='Catalog'+IntToStr(i)+'MI';
    MenuItem.Caption:=CatalogList.Catalogs[i].catalog_name;
    MenuItem.OnClick:=AllcatalogsMIClick;
    SchemaSearchPopupMenu.Items.Add(MenuItem);
  end;

  RefreshSchemaTree(AdvancedEdit.SearchEd.Text);
end;

procedure TSchemataFrame.AllCatalogsMIClick(Sender: TObject);
var i: integer;
begin
  for i:=0 to TMenuItem(Sender).Parent.Count-1 do
    TMenuItem(Sender).Parent.Items[i].Checked:=False;

  TMenuItem(Sender).Checked:=True;

  if(TMenuItem(Sender).Parent.IndexOf(TMenuItem(Sender))=0)then
    CurrentCatalog:=nil
  else
    CurrentCatalog:=CatalogList.catalogs[TMenuItem(Sender).Parent.IndexOf(TMenuItem(Sender))-1];

  RefreshSchemaTree(AdvancedEdit.SearchEd.Text);

  AdvancedEdit.SearchEd.SetFocus;
end;

procedure TSchemataFrame.RefreshCatalogsSchemataListMIClick(
  Sender: TObject);
begin
  ReloadSchemaTree;
end;

procedure TSchemataFrame.AdvancedEditSearchEdChange(Sender: TObject);
begin
  AdvancedEdit.SearchEdChange(Sender);

  RefreshSchemaTree(AdvancedEdit.SearchEd.Text);

  //if there is no Node selected
  if(CatalogVST.SelectedCount=0)then
  begin
    CurrentSchema:=nil;
    CurrentTable:=nil;
  end;
end;

function SortSchemaList(Item1, Item2: Pointer): Integer;
begin
  SortSchemaList:=CompareStr(TMYX_SCHEMA(Item1).schema_name, TMYX_SCHEMA(Item2).schema_name);
end;

function StrMatchesSearchStr(Str: WideString; searchStr: WideString): Boolean;
begin
  StrMatchesSearchStr:=True;

  if(searchStr<>'')then
    StrMatchesSearchStr:=(myx_match_string(Str,searchStr, 0)<>0);
end;

function AddSchemaToCatalogTree(MySQLMajorVersion: integer; TreeView: TVirtualStringTree;
  CatalogNode: PVirtualNode; Catalog: TMYX_CATALOG; Schema: TMYX_SCHEMA;
  searchStr: WideString; ShowSchemaAssets: Boolean;
  ShowTablesOnSchemaExpansion: Boolean): PVirtualNode;
var SchemaNode, Node: PVirtualNode;
begin
  SchemaNode:=TreeView.AddChild(CatalogNode, Schema);

  if(ShowSchemaAssets)then
  begin
    //Add SchemaSubNodes

    //Add Tables node and subnode dummy, so the user can expand the node
    Node:=TreeView.AddChild(SchemaNode,
      TSchemaSubNode.Create(Catalog, Schema, TSNTables));

    TreeView.AddChild(Node);

    //Add Indices
    Node:=TreeView.AddChild(SchemaNode,
      TSchemaSubNode.Create(Catalog, Schema, TSNIndices));

    TreeView.AddChild(Node);

    //MySQL 5.0 (not now)
    if(MySQLMajorVersion>=5+100)then
    begin
      TreeView.AddChild(SchemaNode,
        TSchemaSubNode.Create(Catalog, Schema, TSNViews));
      TreeView.AddChild(SchemaNode,
        TSchemaSubNode.Create(Catalog, Schema, TSNStoredProcedures));
      TreeView.AddChild(SchemaNode,
        TSchemaSubNode.Create(Catalog, Schema, TSNEvents));
    end;

    {AddTreeViewChildNode(TreeView, SchemaNode, 'Developer Docs', 18,
      TSchemaSubNode.Create(Schema, TSNDevDocs));}

    //Add Users
    TreeView.AddChild(SchemaNode,
      TSchemaSubNode.Create(Catalog, Schema, TSNUsers));
  end
  else if(ShowTablesOnSchemaExpansion)then
    TreeView.AddChild(SchemaNode);

  Result:=SchemaNode;
end;

procedure TSchemataFrame.RebuildSchemaTree(MySQLMajorVersion: integer;
  CatalogList: TMYX_CATALOGS; Catalog: TMYX_CATALOG; searchStr: WideString);
var i, j, k: integer;
  schema_name: WideString;
  found: boolean;
  PCatalogNode, PNode: PVirtualNode;
  AddedSchema: TMYX_SCHEMA;
begin
  if(CatalogList=nil)then
    Exit;

  PCatalogNode:=nil;

  CatalogVST.BeginUpdate;
  try
    ClearCatalogTree;

    if(Catalog=nil)then
    begin
      for i:=0 to CatalogList.Catalogs.Count-1 do
      begin
        {CatalogNode:=AddTreeViewChildNode(TreeView, nil,
          CatalogList.Catalogs[i].catalog_name, 2,
          CatalogList.Catalogs[i]);}

        for j:=0 to CatalogList.Catalogs[i].schemata.Count-1 do
          if(StrMatchesSearchStr(
            CatalogList.Catalogs[i].schemata[j].schema_name, searchStr))then
          begin
            PNode:=AddSchemaToCatalogTree(MySQLMajorVersion, CatalogVST,
              PCatalogNode, CatalogList.Catalogs[i],
              CatalogList.Catalogs[i].schemata[j], searchStr,
              FShowSchemaAssets, ShowTablesOnSchemaExpansion);

            if(CatalogList.Catalogs[i].catalog_name=LastCatalogName)and
              (CatalogList.Catalogs[i].schemata[j].schema_name=LastSchemaName)then
            begin
              CatalogVST.Selected[PNode]:=True;
              CatalogVST.FocusedNode:=PNode;
            end;

            if(CompareText(ExpandSchemaOnLaunch,
              CatalogList.Catalogs[i].schemata[j].schema_name)=0)then
            begin
              CatalogVST.DeleteChildren(PNode, True);

              RefreshTableNames(TSchemaVirtualNode.Create(PNode,
                CatalogList.Catalogs[i].schemata[j]));

              ExpandSchemaOnLaunch:='';
            end;
          end;

        for j:=0 to SchemasToAdd.Count-1 do
        begin
          if(CompareText(
            Copy(SchemasToAdd[j], 1, Pos('/', SchemasToAdd[j])-1),
            CatalogList.Catalogs[i].catalog_name)=0)then
          begin
            found:=False;
            schema_name:=Copy(SchemasToAdd[j], Pos('/', SchemasToAdd[j])+1,
              Length(SchemasToAdd[j]));

            for k:=0 to CatalogList.Catalogs[i].schemata.Count-1 do
              if(CompareText(schema_name,
                CatalogList.Catalogs[i].schemata[k].schema_name)=0)then
              begin
                found:=True;
                break;
              end;

            if(Not(found))and(StrMatchesSearchStr(schema_name, searchStr))then
            begin
              AddedSchema:=TMYX_SCHEMA.create(schema_name, CatalogList.Catalogs[i].catalog_name, nil, nil);

              AddedSchemataList.Add(AddedSchema);

              PNode:=AddSchemaToCatalogTree(MySQLMajorVersion, CatalogVST,
                PCatalogNode, CatalogList.Catalogs[i],
                AddedSchema, searchStr, FShowSchemaAssets,
                ShowTablesOnSchemaExpansion);

              if(CatalogList.Catalogs[i].catalog_name=LastCatalogName)and
                (schema_name=LastSchemaName)then
              begin
                CatalogVST.FocusedNode:=PNode;
                CatalogVST.ClearSelection;
                CatalogVST.Selected[PNode]:=True;
              end;
            end;
          end;
        end;

        if(PCatalogNode<>nil)then
          if(CatalogVST.ChildCount[PCatalogNode]>0)then
            CatalogVST.Expanded[PCatalogNode]:=True
          else
            CatalogVST.DeleteNode(PCatalogNode);
      end;
    end
    else
    begin
      {CatalogNode:=AddTreeViewChildNode(TreeView, nil,
        Catalog.catalog_name, 0,
        Catalog);

      k:=0;
      for j:=0 to Catalog.schemata.Count-1 do
        if(StrMatchesSearchStr(Catalog.schemata[j].schema_name, searchStr))then
        begin
          Node:=AddSchemaToTree(MySQLMajorVersion, TreeView, CatalogNode,
            Catalog,
            Catalog.schemata[j], searchStr, FShowSchemaAssets);
          inc(k);

          if(Catalog.catalog_name=LastCatalogName)and
            (Catalog.schemata[j].schema_name=LastSchemaName)then
            TreeView.Selected:=Node;
        end;

      if(k>0)then
        CatalogNode.Expand(False)
      else
        TreeView.Items.Delete(CatalogNode);}
    end;

    //Re-expand tree nodes
    ThreadsDisabled:=True;
    try
      ReExpandedNodes(ExpandedNodesList, 0, CatalogVST, CatalogVST.GetFirst);
    finally
      ThreadsDisabled:=False;
    end;
  finally
    CatalogVST.EndUpdate;
  end;

  if(CatalogVST.FocusedNode<>nil)then
    CatalogVST.ScrollIntoView(CatalogVST.FocusedNode, True);
end;

procedure FetchSchemaTablesFromNode(MySQLConn: TMySQLConn;
  SchemaNode: TSchemaVirtualNode);
var SchemaTables: PMYX_SCHEMA_TABLES;
begin
  SchemaTables:=myx_get_schema_tables(MySQLConn.PMySQL,
    '',SchemaNode.Schema.schema_name);
  if(SchemaTables=nil)then
    raise EMyxSQLError.Create(_('Could not fetch Schema Tables.'),
      myx_mysql_errno(MySQLConn.PMySQL),
      myx_mysql_error(MySQLConn.PMySQL));

  try
    SchemaNode.Schema.schema_tables:=
      TMYX_SCHEMA_TABLES.create(SchemaTables);
  finally
    myx_free_schema_tables(SchemaTables);
  end;
end;

procedure TSchemataFrame.FetchSchemaTables(Sender: TObject);
begin
  FetchSchemaTablesFromNode(TFetchDataThread(Sender).MySQLConn,
    TSchemaVirtualNode(TFetchDataThread(Sender).Target));
end;

procedure TSchemataFrame.SchemaTablesFetched(Sender: TObject);
begin
  RefreshTableNames(TSchemaVirtualNode(TFetchDataThread(Sender).Target));
end;

procedure TSchemataFrame.RefreshTableNames(SchemaNode: TSchemaVirtualNode);
var i, j: integer;
  TableNode: PVirtualNode;
begin
  if(SchemaNode.schema.schema_tables=nil)and(Not(ThreadsDisabled))then
    MySQLConn.FetchData(KindOfData_SchemaTables,
      FetchSchemaTables, SchemaTablesFetched,
      SchemaNode, 'Fetching Tables ...',
      False, True)
  else
  begin
    //If ThreadsDisabled is turned off, fetch SchemaTables now
    if(SchemaNode.schema.schema_tables=nil)and(ThreadsDisabled)then
      FetchSchemaTablesFromNode(MySQLConn, SchemaNode);

    CatalogVST.BeginUpdate;
    try
      //Attach tables to schema node
      CatalogVST.DeleteChildren(SchemaNode.Node);
      for i:=0 to SchemaNode.schema.schema_tables.schema_tables.Count-1 do
      begin
        TableNode:=CatalogVST.AddChild(SchemaNode.Node,
          SchemaNode.schema.schema_tables.schema_tables[i]);

        for j:=0 to SchemaNode.schema.schema_tables.schema_tables[i].columns.Count-1 do
          CatalogVST.AddChild(TableNode,
            SchemaNode.schema.schema_tables.schema_tables[i].columns[j]);

        if(SelectTableAfterTablesNodeExpand<>'')then
          if(SchemaNode.schema.schema_tables.schema_tables[i].table_name=
            SelectTableAfterTablesNodeExpand)then
          begin
            CatalogVST.ClearSelection;
            CatalogVST.Selected[TableNode]:=True;
            CatalogVST.FocusedNode:=TableNode;
            CatalogVST.ScrollIntoView(TableNode, False);

            SelectTableAfterTablesNodeExpand:='';
          end;

        CatalogVST.Expanded[SchemaNode.Node]:=True;
      end;
    finally
      CatalogVST.EndUpdate;
      SchemaNode.Free;
    end;
  end;
end;

procedure TSchemataFrame.FetchSchemaIndices(Sender: TObject);
var SchemaIndices: PMYX_SCHEMA_INDICES;
begin
  SchemaIndices:=myx_get_schema_indices(TFetchDataThread(Sender).MySQLConn.PMySQL,
    '', TSchemaVirtualNode(TFetchDataThread(Sender).Target).Schema.schema_name);
  if(SchemaIndices=nil)then
    raise EMyxSQLError.Create(_('Could not fetch Schema Indices.'),
      myx_mysql_errno(TFetchDataThread(Sender).MySQLConn.PMySQL),
      myx_mysql_error(TFetchDataThread(Sender).MySQLConn.PMySQL));

  try
    TSchemaVirtualNode(TFetchDataThread(Sender).Target).Schema.schema_indices:=
      TMYX_SCHEMA_INDICES.create(SchemaIndices);
  finally
    myx_free_schema_indices(SchemaIndices);
  end;
end;

procedure TSchemataFrame.SchemaIndicesFetched(Sender: TObject);
begin
  RefreshIndicesNames(TSchemaVirtualNode(TFetchDataThread(Sender).Target));
end;

procedure TSchemataFrame.RefreshIndicesNames(SchemaNode: TSchemaVirtualNode);
var i, j: integer;
  IndexNode: PVirtualNode;
begin
  if(SchemaNode.schema.schema_indices=nil)then
    MySQLConn.FetchData(KindOfData_SchemaIndices,
      FetchSchemaIndices, SchemaIndicesFetched,
      SchemaNode, 'Fetching Indices ...',
      False, True)
  else
  begin
    CatalogVST.BeginUpdate;
    try
      //Attach tables to schema node
      CatalogVST.DeleteChildren(SchemaNode.Node);
      for i:=0 to SchemaNode.schema.schema_indices.indices.Count-1 do
      begin
        IndexNode:=CatalogVST.AddChild(SchemaNode.Node,
          SchemaNode.schema.schema_indices.indices[i]);

        for j:=0 to SchemaNode.schema.schema_indices.indices[i].index_columns.Count-1 do
          CatalogVST.AddChild(IndexNode,
            SchemaNode.schema.schema_indices.indices[i].index_columns[j]);

        CatalogVST.Expanded[SchemaNode.Node]:=True;
      end;
    finally
      CatalogVST.EndUpdate;
      SchemaNode.Free;
    end;
  end;
end;

procedure TSchemataFrame.DropSchemaMIClick(Sender: TObject);
begin
  if(CurrentSchema<>nil)then
  begin
    if(CompareText(CurrentSchema.schema_name,
      'mysql')=0)then
      ShowModalDialog(_('Cannot Drop MySQL Schema'),
        _('You cannot drop the MySQL schema'),
        myx_mtError, 'OK')
    else
    if(ShowModalDialog(_('Delete Schema'),
      Format(_('Are you sure you want to delete the selected schema %s?'#13#10#13#10+
          'Please note that this action cannot be rolled back.'),
        [CurrentSchema.schema_name]),
      myx_mtConfirmation, 'OK'#13#10'Cancel')=1)then
    begin
      MySQLConn.ExecuteDirect('DROP DATABASE `'+CurrentSchema.schema_name+'`');

      ReloadSchemaTree;

      MySQLConn.SchemaListChanged;
    end;
  end;
end;

procedure TSchemataFrame.CreateNewSchemaMIClick(Sender: TObject);
var name: WideString;
begin
  name:='';

  if(ShowModalEditDialog(_('Create new Schema'),
    _('Please enter a name for the new schema.'),
    myx_mtEdit, 'OK'#13#10'Cancel',
    True, _('Schema name:'), name)=1)then
  begin
    if(name<>'')then
    begin
      MySQLConn.ExecuteDirect('CREATE DATABASE `'+name+'`');

      ReloadSchemaTree;

      MySQLConn.SchemaListChanged;
    end;
  end;
end;

procedure TSchemataFrame.DropTableMIClick(Sender: TObject);
begin
  if(CurrentSchema<>nil)and(CurrentTable<>nil)then
  begin
    if(ShowModalDialog(_('Drop Table'),
      Format(_('Are you sure you want to drop the table %s?'),
        [CurrentSchema.schema_name+'.'+CurrentTable.table_name]),
      myx_mtConfirmation, 'OK'#13#10'Cancel')=1)then
    begin
      MySQLConn.ExecuteDirect('DROP TABLE `'+
        CurrentSchema.schema_name+'`.'+'`'+CurrentTable.table_name+'`');

      ReloadSchemaTree;
    end;
  end;
end;

procedure TSchemataFrame.SchemaTreeViewPopupMenuPopup(Sender: TObject);
begin
  DropSchemaMI.Enabled:=(CurrentSchema<>nil);
  CreateNewTableMI.Enabled:=(CurrentSchema<>nil);
  EditTableMI.Enabled:=(CurrentTable<>nil);
  DropTableMI.Enabled:=(CurrentTable<>nil);
end;

procedure TSchemataFrame.CreateNewTableMIClick(Sender: TObject);
begin
  if(Assigned(FCreateTable))then
    FCreateTable(self)
  else
  begin
    if(CurrentCatalog=nil)then
      EditTable('', CurrentSchema.schema_name, '')
    else
      EditTable(CurrentCatalog.catalog_name, CurrentSchema.schema_name, '');
  end;
end;

procedure TSchemataFrame.DoTableEditorApplyedChanges(Sender: TObject);
begin
  ReloadSchemaTree;
end;

procedure TSchemataFrame.EditTable(Catalog: WideString;
  Schema: WideString; Table: WideString);
begin
  if(EditorTableForm=nil)then
  begin
    EditorTableForm:=TEditorTableForm.Create(self);
    EditorTableForm.SetEditMode(DBMEditMode_Online, MySQLConn);
    EditorTableForm.OnApplyedChanges:=DoTableEditorApplyedChanges
  end;

  EditorTableForm.SetDatabaseVersion(
    myx_get_mysql_major_version(MySQLConn.PMySQL),
    myx_get_mysql_minor_version(MySQLConn.PMySQL));

  EditorTableForm.EditSchemaTable(Catalog, Schema, Table,
    CurrentSchema.schema_tables);

  EditorTableForm.Show;
end;

procedure TSchemataFrame.EditTableMIClick(Sender: TObject);
begin
  if(Assigned(FEditTable))then
    FEditTable(self)
  else
  begin
    if(CurrentTable<>nil)then
    begin
      if(CurrentCatalog=nil)then
        EditTable('', CurrentSchema.schema_name, CurrentTable.table_name)
      else
        EditTable(CurrentCatalog.catalog_name, CurrentSchema.schema_name, CurrentTable.table_name);
    end;
  end;
end;

procedure TSchemataFrame.CatalogVSTAfterCellPaint(Sender: TBaseVirtualTree;
  TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  CellRect: TRect);
var NodeData: ^TObject;
  TxtRect: TRect;
  x: integer;
begin
  NodeData:=Sender.GetNodeData(Node);
  if(NodeData^<>nil)then
  begin
    TxtRect:=Sender.GetDisplayRect(Node, Column, True);

    x:=TxtRect.Left-Sender.OffsetX;

    //Draw >
    if(Node.ChildCount>0)or(vsHasChildren in Node.States)then
    begin
      if(Sender.Expanded[Node])then
        TreeBtnOpenPNGImg.Draw(TargetCanvas,
          Rect(x-16-12, CellRect.Top+4, x-16-4, CellRect.Top+16+4))
      else
        TreeBtnClosedPNGImg.Draw(TargetCanvas,
          Rect(x-16-12, CellRect.Top+4, x-16-4, CellRect.Top+16+4))
    end;

    if(NodeData^ is TMYX_SCHEMA)then
      SchemaPNGImg.Draw(TargetCanvas,
        Rect(x-16, CellRect.Top+1, x, CellRect.Top+17))
    else if(NodeData^ is TSchemaSubNode)then
    begin
      if(TSchemaSubNode(NodeData^).SubNodeType=TSNTables)then
        TablePNGImg.Draw(TargetCanvas,
          Rect(x-16, CellRect.Top+1, x, CellRect.Top+17))
      else if(TSchemaSubNode(NodeData^).SubNodeType=TSNIndices)then
        IndexPNGImg.Draw(TargetCanvas,
          Rect(x-16, CellRect.Top+1, x, CellRect.Top+17))
      else if(TSchemaSubNode(NodeData^).SubNodeType=TSNUsers)then
        UserPNGImg.Draw(TargetCanvas, Rect(x-16, CellRect.Top+1, x, CellRect.Top+17));
    end
    else if(NodeData^ is TMYX_SCHEMA_TABLE)then
      TablePNGImg.Draw(TargetCanvas,
        Rect(x-16, CellRect.Top+1, x, CellRect.Top+17))
    else if(NodeData^ is TMYX_SCHEMA_TABLE_COLUMN)then
    begin
      if(TMYX_SCHEMA_TABLE_COLUMN(NodeData^).primary_key=1)then
        ColumnPKPNGImg.Draw(TargetCanvas,
          Rect(x-16, CellRect.Top+1, x, CellRect.Top+17))
      else
        ColumnPNGImg.Draw(TargetCanvas,
          Rect(x-16, CellRect.Top+1, x, CellRect.Top+17));
    end
    else if(NodeData^ is TMYX_TABLE_INDEX)then
      IndexPNGImg.Draw(TargetCanvas,
        Rect(x-16, CellRect.Top+1, x, CellRect.Top+17))
    else if(NodeData^ is TMYX_TABLE_INDEX_COLUMN)then
      ColumnPNGImg.Draw(TargetCanvas,
        Rect(x-16, CellRect.Top+1, x, CellRect.Top+17))
  end;
end;

procedure TSchemataFrame.CatalogVSTGetText(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Column: TColumnIndex; TextType: TVSTTextType;
  var CellText: WideString);
var NodeData: ^TObject;
begin
  NodeData:=Sender.GetNodeData(Node);
  if(NodeData<>nil)then
    if(NodeData^<>nil)then
    begin
      Font.Style:=[];

      if(NodeData^ is TMYX_SCHEMA)then
      begin
        CellText:=TMYX_SCHEMA(NodeData^).schema_name;
      end
      else if(NodeData^ is TSchemaSubNode)then
      begin
        if(TSchemaSubNode(NodeData^).SubNodeType=TSNTables)then
          CellText:=_('Tables')
        else if(TSchemaSubNode(NodeData^).SubNodeType=TSNIndices)then
          CellText:=_('Indices')
        else if(TSchemaSubNode(NodeData^).SubNodeType=TSNUsers)then
          CellText:=_('Users');
      end
      else if(NodeData^ is TMYX_SCHEMA_TABLE)then
        CellText:=TMYX_SCHEMA_TABLE(NodeData^).table_name
      else if(NodeData^ is TMYX_SCHEMA_TABLE_COLUMN)then
        CellText:=TMYX_SCHEMA_TABLE_COLUMN(NodeData^).column_name
      else if(NodeData^ is TMYX_TABLE_INDEX)then
        CellText:=TMYX_TABLE_INDEX(NodeData^).key_name+' ('+
          TMYX_TABLE_INDEX(NodeData^).table_name+')'
      else if(NodeData^ is TMYX_TABLE_INDEX_COLUMN)then
        CellText:=TMYX_TABLE_INDEX_COLUMN(NodeData^).column_name;
    end;
end;

procedure TSchemataFrame.CatalogVSTGetImageIndex(Sender: TBaseVirtualTree;
  Node: PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
  var Ghosted: Boolean; var ImageIndex: Integer);
begin
  ImageIndex:=0;
end;

procedure TSchemataFrame.CatalogVSTMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var HitInfo: THitInfo;
  TxtRect: TRect;
begin
  if(Sender.InheritsFrom(TBaseVirtualTree))then
  begin
    TBaseVirtualTree(Sender).GetHitTestInfoAt(X, Y, True, HitInfo);

    if(HitInfo.HitNode<>nil)then
    begin
      TxtRect:=TBaseVirtualTree(Sender).GetDisplayRect(
        HitInfo.HitNode, -1, True);

      if(X>TxtRect.Left-16-16)and(X<TxtRect.Left-16)and
        ((HitInfo.HitNode.ChildCount>0)or(vsHasChildren in HitInfo.HitNode.States))then
      begin
        TBaseVirtualTree(Sender).Expanded[HitInfo.HitNode]:=
          Not(TBaseVirtualTree(Sender).Expanded[HitInfo.HitNode]);
      end;
    end
  end;
end;

procedure TSchemataFrame.CatalogVSTExpanding(Sender: TBaseVirtualTree;
  Node: PVirtualNode; var Allowed: Boolean);
var NodeData, ChildNodeData: ^TObject;
begin
  if(ShowTablesOnSchemaExpansion)then
  begin
    NodeData:=Sender.GetNodeData(Node);

    if(NodeData^<>nil)then
      if(NodeData^ is TMYX_SCHEMA)then
      begin
        CurrentSchema:=TMYX_SCHEMA(NodeData^);
        if(TMYX_SCHEMA(NodeData^).schema_tables=nil)then
        begin
          RefreshTableNames(TSchemaVirtualNode.Create(Node, TMYX_SCHEMA(NodeData^)));
        end;
      end;
  end
  else
  begin
    NodeData:=Sender.GetNodeData(Node);

    //Expand Tables only if
    // Parent node is a TMYX_SCHEMA
    // The node itself holds a TSchemaSubNode of type TSNTables in Data
    // the first childnode's data is null
    if(NodeData^<>nil)and(Sender.GetFirstChild(Node)<>nil)then
    begin
      ChildNodeData:=Sender.GetNodeData(Sender.GetFirstChild(Node));

      if(TObject(NodeData^) is TSchemaSubNode)then
        if(TSchemaSubNode(NodeData^).SubNodeType=TSNTables)and
          (ChildNodeData^=nil)then
        begin
          Sender.DeleteChildren(Node);
          RefreshTableNames(TSchemaVirtualNode.Create(
            Node, TSchemaSubNode(NodeData^).Schema));
        end;
    end;

    //Expand Indices only if
    // The node itself holds a TSchemaSubNode of type TSNIndices in Data
    // the first childnode's data is null
    if(NodeData^<>nil)and(Sender.GetFirstChild(Node)<>nil)then
    begin
      ChildNodeData:=Sender.GetNodeData(Sender.GetFirstChild(Node));

      if(TObject(NodeData^) is TSchemaSubNode)then
        if(TSchemaSubNode(NodeData^).SubNodeType=TSNIndices)and
          (ChildNodeData^=nil)then
        begin
          Sender.DeleteChildren(Node);
          RefreshIndicesNames(TSchemaVirtualNode.Create(
            Node, TSchemaSubNode(NodeData^).Schema));
        end;
    end;
  end;
end;

procedure TSchemataFrame.CatalogVSTChange(Sender: TBaseVirtualTree;
  Node: PVirtualNode);
var NodeData, ParentNodeData: ^TObject;
  ParentNode: PVirtualNode;
begin
  CurrentSchema:=nil;
  CurrentTable:=nil;

  NodeData:=Sender.GetNodeData(Node);

  if(NodeData<>nil)then
    if(NodeData^<>nil)then
      if(TObject(NodeData^) is TMYX_SCHEMA)then
      begin
        CurrentSchema:=TMYX_SCHEMA(NodeData^);
      end
      else if(TObject(NodeData^) is TSchemaSubNode)then
      begin
        CurrentSchema:=TSchemaSubNode(NodeData^).Schema;
      end
      else if(TObject(NodeData^) is TMYX_SCHEMA_TABLE)then
      begin
        ParentNode:=Sender.NodeParent[Node];
        if(ParentNode<>nil)then
        begin
          ParentNodeData:=Sender.GetNodeData(ParentNode);

          if(ParentNodeData<>nil)then
            if(ParentNodeData^<>nil)then
            begin
              if(TObject(ParentNodeData^) is TSchemaSubNode)then
                CurrentSchema:=TSchemaSubNode(ParentNodeData^).Schema
              else if(TObject(ParentNodeData^) is TMYX_SCHEMA)then
                CurrentSchema:=TMYX_SCHEMA(ParentNodeData^);
            end;

          if(CurrentSchema<>nil)then
            CurrentTable:=TMYX_SCHEMA_TABLE(NodeData^);
        end;
      end;
end;

procedure TSchemataFrame.ClearCatalogTree;
begin
  CatalogVST.Clear;
  //Free mem for AddedSchemata
  AddedSchemataList.Clear;
end;

function TSchemataFrame.GetCatalogVSTFocusedObject(MatchClass: TClass = nil): TObject;
var NodeData: ^TObject;
begin
  Result:=nil;

  if(CatalogVST.FocusedNode<>nil)then
  begin
    NodeData:=CatalogVST.GetNodeData(CatalogVST.FocusedNode);
    if(NodeData<>nil)then
      if(NodeData^<>nil)then
        if(NodeData^ is MatchClass)then
          Result:=NodeData^;
  end;
end;



procedure TSchemataFrame.CatalogVSTPaintText(Sender: TBaseVirtualTree;
  const TargetCanvas: TCanvas; Node: PVirtualNode; Column: TColumnIndex;
  TextType: TVSTTextType);
var NodeData: ^TObject;
begin
  if(FPaintDefaultSchemaBold)then
  begin
    if(TargetCanvas.Font.Style<>[])then
      TargetCanvas.Font.Style:=[];

    NodeData:=Sender.GetNodeData(Node);
    if(NodeData<>nil)then
      if(NodeData^<>nil)then
        if(NodeData^ is TMYX_SCHEMA)then
          if(TMYX_SCHEMA(NodeData^).schema_name=FDefaultSchema)then
          begin
            TargetCanvas.Font.Style:=[fsBold];
          end;
  end;
end;

end.
