program SetVersion;

{$APPTYPE CONSOLE}

uses
  SysUtils,
  Classes,
  RegExpr in '..\LibInterfaceMapper\RegExpr.pas';

procedure WritePascalIncludeFile(version: string; build_level: string;
  pascalIncludeFile: string);
var f: TextFile;
begin
  AssignFile(f, pascalIncludeFile);
  Rewrite(f);
  WriteLn(f, 'const product_version='''+version+''';');
  WriteLn(f, 'const product_build_level='''+build_level+''';');
  CloseFile(f);
end;

function GetNewGUID: string;
var Guid: TGUID;
  s: string;
begin
  CreateGUID(Guid);

  try
    s:=GUIDToString(Guid);
    Result:=Copy(s, 2, Length(s)-2);
  except
    WriteLn('Error: Cannot get a new GUID.');
  end;
end;

procedure UpdateWIXFile(version: string; build_level: string;
  WIXFileName: string);
var WixFile: TStringList;
  RegExpr: TRegExpr;
begin
  WixFile:=TStringList.Create;
  try
    RegExpr:=TRegExpr.Create;
    try
      WixFile.LoadFromFile(WIXFileName);

      RegExpr.Expression:=' Version="([0-9\.]{0,})"';
      WixFile.Text:=RegExpr.Replace(WixFile.Text,
        ' Version="'+version+'"', True);

      RegExpr.Expression:='Maximum="([0-9\.]{0,})"';
      WixFile.Text:=RegExpr.Replace(WixFile.Text,
        'Maximum="'+version+'"', True);

      RegExpr.Expression:='"NEWERPRODUCTFOUND" Minimum="([0-9\.]{0,})"';
      WixFile.Text:=RegExpr.Replace(WixFile.Text,
        '"NEWERPRODUCTFOUND" Minimum="'+version+'"', True);

      RegExpr.Expression:='Product Id="([0-9A-F\-]{0,})"';
      WixFile.Text:=RegExpr.Replace(WixFile.Text,
        'Product Id="'+GetNewGUID+'"', True);

      RegExpr.Expression:='"Pversion">([0-9\.]{0,})<';
      WixFile.Text:=RegExpr.Replace(WixFile.Text,
        '"Pversion">'+version+'<', True);

      RegExpr.Expression:='"QualifyedProductName">([a-zA-Z\s]{0,})(\s[0-9\.]{0,})<';
      WixFile.Text:=RegExpr.Replace(WixFile.Text,
        '"QualifyedProductName">$1 '+version+'<', True);

      WixFile.SaveToFile(WIXFileName);
    finally
      RegExpr.Free;
    end;
  finally
    WixFile.Free;
  end;
end;

function GetNewGUIDString: string;
begin
  Result:='Guid="'+GetNewGUID+'"';
end;

procedure UpdateWIXMergeFile(version: string; build_level: string;
  WIXFileName: string);
var WixFile: TStringList;
  RegExpr: TRegExpr;
begin
  WixFile:=TStringList.Create;
  try
    RegExpr:=TRegExpr.Create;
    try
      WixFile.LoadFromFile(WIXFileName);

      RegExpr.Expression:=' Version="([0-9\.]{0,})"';
      WixFile.Text:=RegExpr.Replace(WixFile.Text,
        ' Version="'+version+'"', True);

      RegExpr.Expression:='Guid=''([0-9A-F\-]{0,})''';
      WixFile.Text:=RegExpr.Replace(WixFile.Text,
        'Guid='''+GetNewGUID+'''', True);

      RegExpr.Expression:='Guid="([0-9A-F\-]{0,})"';
      WixFile.Text:=RegExpr.ReplaceWithCallback(WixFile.Text,
        GetNewGUIDString, True);

      WixFile.SaveToFile(WIXFileName);
    finally
      RegExpr.Free;
    end;
  finally
    WixFile.Free;
  end;
end;

procedure DisplayUsage;
begin
  WriteLn('MySQL SetVersion version 1.0.0.2, for Win2k/XP, by Michael Zinner');
  WriteLn('This software comes with ABSOLUTELY NO WARRANTY. This is free software,');
  WriteLn('and you are welcome to modify and redistribute it under the GPL license');
  WriteLn('');
  WriteLn('Setting version constants for GUI tools ');
  WriteLn('Usage: setversion -p[Pascal Include File] -x[WIX wxs File] ');
  WriteLn('                  -xm[WIX Merge Module File]');
  WriteLn('');
  WriteLn('Optional parameters:');
  WriteLn('                  -v[Version] -b[Build level]');
  WriteLn('');
end;

procedure ParseParams;
var pascalIncludeFile: string;
  WIXFile, WIXMergeFile: string;
  version: string;
  build_level: string;
  somethingToDo: Boolean;
  i: integer;
  RegExpr: TRegExpr;
begin
  pascalIncludeFile:='';
  WIXFile:='';
  WIXMergeFile:='';

  somethingToDo:=False;
  version:='';
  build_level:='';

  //Check all parameters
  for i:=1 to ParamCount do
  begin
    //-p Write Pascal Include File
    if(Copy(ParamStr(i), 1, 2)='-p')then
    begin
      pascalIncludeFile:=Copy(ParamStr(i), 3, Length(ParamStr(i))-1);
      somethingToDo:=True;
    end
    else if(Copy(ParamStr(i), 1, 2)='-x')and(Copy(ParamStr(i), 3, 1)<>'m')then
    begin
      WIXFile:=Copy(ParamStr(i), 3, Length(ParamStr(i))-1);
      somethingToDo:=True;
    end
    else if(Copy(ParamStr(i), 1, 3)='-xm')then
    begin
      WIXMergeFile:=Copy(ParamStr(i), 4, Length(ParamStr(i))-2);
      somethingToDo:=True;
    end
    else if(Copy(ParamStr(i), 1, 2)='-v')then
    begin
      version:=Copy(ParamStr(i), 3, Length(ParamStr(i))-1);
    end
    else if(Copy(ParamStr(i), 1, 2)='-b')then
    begin
      build_level:=Copy(ParamStr(i), 3, Length(ParamStr(i))-1);
    end;
  end;

  if(version='')and(build_level='')then
  begin
    WriteLn('MySQL SetVersion 1.0'#13#10+
            '--------------------'#13#10);

    version:='';
    while(version='')do
    begin
      Write('Please enter the version number (e.g. 1.0.0): ');
      ReadLn(version);

      if(version='q')or(version='exit')then
        Exit;

      RegExpr:=TRegExpr.Create;
      try
        RegExpr.Expression:='(\d{1,})\.(\d{1,})\.(\d{1,})';
        if(Not(RegExpr.Exec(version)))then
          version:='';
      finally
        RegExpr.Free;
      end;
    end;

    //Get build level
    build_level:='...';
    while(build_level<>'alpha')and
      (build_level<>'beta')and
      (build_level<>'gamma')and
      (build_level<>'internal')and
      (build_level<>'')do
    begin
      Write('Please enter the build level (a=alpha, b=beta, g=gamma, p=production, i=internal): ');
      ReadLn(build_level);

      if(version='q')or(version='exit')then
        Exit
      else if(build_level='a')then
        build_level:='alpha'
      else if(build_level='b')then
        build_level:='beta'
      else if(build_level='g')then
        build_level:='gamma'
      else if(build_level='p')then
        build_level:=''
      else if(build_level='i')then
        build_level:='internal'
      else
        build_level:='...';
    end;
  end;

  if(Not(somethingToDo))then
    DisplayUsage
  else
  begin
    if(pascalIncludeFile<>'')then
      WritePascalIncludeFile(version, build_level, pascalIncludeFile);

    if(WIXFile<>'')then
      UpdateWIXFile(version, build_level, WIXFile);

    if(WIXMergeFile<>'')then
      UpdateWIXMergeFile(version, build_level, WIXMergeFile);
  end;
end;

begin
  //If params are specified
  if(ParamCount>0)and(ParamStr(1)<>'--help')then
    ParseParams
  else
    //else write usage
    DisplayUsage;
end.
