unit TabHeader;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, ExtCtrls, VirtualTrees, TntClasses, TntExtCtrls, TntForms,
  PNGImage, AuxFuncs, Contnrs, tntmenus;

type
  TPageChangeEvent = procedure(Sender: TObject;
    PreviousSelectedTab: integer;
    NewSelectedTab: integer; Obj: TObject) of object;

  TTab = class;

  TExtTntPaintBox = class;

  TTabHeaderFrame = class(TTntFrame)
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure ClearTabSheets;
    function AddTabSheet(AOwner: TComponent; Caption: WideString;
      TabPanel: TTntPanel; Obj: TObject = nil; SelectNewPage: Boolean = True;
      DockTabPanel: Boolean = True; TabPanelHeight: integer = -1): integer;
    procedure DeleteTab(TabIndex: Integer);

    function TabCount: integer;

    procedure Show;
    procedure Hide;
    procedure TntFrameResize(Sender: TObject);

    procedure SelectNextTabSheet;
  protected
    procedure TabPaintBoxPaint(Sender: TObject);
    procedure RefreshTabSheetsXPos;
    procedure TabPaintBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure TabPaintBoxMouseMove(Sender: TObject;
      Shift: TShiftState; X, Y: Integer);
    procedure TabPaintBoxMouseLeave(Sender: TObject);

    function GetSelectedTab: integer;
    procedure SetSelectedTab(SelectedTabIndex: integer);

    procedure TabMenuItemSelected(Sender: TObject);
    function GetTab(index: integer): TTab;
  private
    { Private declarations }
    PaintBox: TExtTntPaintBox;

    Tabs: TObjectList;
    TabListBtnShown: Boolean;

    NewTabPNGImg,
    CloseTabPNGImg,
    ListTabPNGImg: TPNGObject;

    FSelectedTab: integer;
    FHighlightedTab,
    FHighlightedButton: integer;

    FOnBeforePageChange,
    FOnPageChange: TPageChangeEvent;

    FOnRequestNewPage: TNotifyEvent;
    FOnPageDelete: TNotifyEvent;

    TabsPopupMenu: TTntPopupMenu;

    PaintBoxBufferedImage: TBitmap;
  public
    { Public declarations }
    ShowButtons,
    DrawBottom,
    TopSpacer: Boolean;

    AutoHideTabHeader: Boolean;

    property TabSheets[Index: integer]: TTab read GetTab;
    property SelectedTab: integer read GetSelectedTab write SetSelectedTab;
    property OnBeforePageChange: TPageChangeEvent read FOnBeforePageChange write FOnBeforePageChange;
    property OnPageChange: TPageChangeEvent read FOnPageChange write FOnPageChange;
    property OnRequestNewPage: TNotifyEvent read FOnRequestNewPage write FOnRequestNewPage;
    property OnPageDelete: TNotifyEvent read FOnPageDelete write FOnPageDelete;
    function GetTabIndex(Tab: TTab): integer;
  end;

  TTab = class(TObject)
    constructor Create(AOwner: TComponent; Caption: WideString;
      x1: integer; x2: integer; TabPanel: TTntPanel; Obj: TObject;
      DockTabPanel: Boolean); reintroduce;
    destructor Destroy; override;
  public
    Caption: WideString;
    x1, x2: integer;
    TabPanel: TTntPanel;
    Obj: TObject;
    DockTabPanel: Boolean;
  end;

  TExtTntPaintBox = class(TTntPaintBox)
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  private
    FOnMouseLeave: TNotifyEvent;
  published
    property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
  end;

implementation

{$R *.dfm}

procedure TExtTntPaintBox.CMMouseLeave(var Message: TMessage);
begin
  if(Assigned(FOnMouseLeave))then
    FOnMouseLeave(self);
end;

constructor TTabHeaderFrame.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  InitFrame(self);

  //Set Font size
  if(Screen.PixelsPerInch<>96)then
    Font.Size:=7
  else
    Font.Size:=8;//RescaleFontSize(8);

  FOnPageChange:=nil;
  FOnBeforePageChange:=nil;

  PaintBox:=TExtTntPaintBox.Create(self);
  PaintBox.Parent:=self;
  PaintBox.Align:=alClient;
  PaintBox.OnMouseDown:=TabPaintBoxMouseDown;
  PaintBox.OnMouseMove:=TabPaintBoxMouseMove;
  PaintBox.OnMouseLeave:=TabPaintBoxMouseLeave;
  PaintBox.OnPaint:=TabPaintBoxPaint;

  PaintBoxBufferedImage:=nil;

  ShowButtons:=False;
  DrawBottom:=False;
  TopSpacer:=False;
  FSelectedTab:=0;
  FHighlightedTab:=-1;
  FHighlightedButton:=-1;
  AutoHideTabHeader:=False;

  TabListBtnShown:=False;

  TabsPopupMenu:=TTntPopupMenu.Create(nil);

  NewTabPNGImg:=LoadPNGImageFromResource('tab_new');
  CloseTabPNGImg:=LoadPNGImageFromResource('tab_close');
  ListTabPNGImg:=LoadPNGImageFromResource('tab_list');

  Tabs:=TObjectList.Create;
  //RefreshTabSheetsXPos;
end;

destructor TTabHeaderFrame.Destroy;
begin
  ShowButtons:=False;

  if(PaintBoxBufferedImage<>nil)then
    PaintBoxBufferedImage.Free;

  Tabs.Free;

  NewTabPNGImg.Free;
  CloseTabPNGImg.Free;
  ListTabPNGImg.Free;

  TabsPopupMenu.Free;

  inherited;
end;

function TTabHeaderFrame.GetTab(index: integer): TTab;
begin
  if(index<Tabs.Count)then
    Result:=TTab(Tabs[index])
  else
    Result:=nil;
end;

procedure TTabHeaderFrame.TabPaintBoxPaint(Sender: TObject);
var PrevSelTabs, PostSelTabs, SelTab: WideString;
  i, PrevSelTabsWidth, SelTabWidth, TopSpace, TotalWidth,
  NewBtnWidth, CloseBtnWidth, ListBtnWidth: integer;
  aRect, HighlightRect: TRect;
begin
  if(PaintBoxBufferedImage=nil)then
    PaintBoxBufferedImage:=TBitmap.Create;

  PaintBoxBufferedImage.Width:=PaintBox.Width;
  PaintBoxBufferedImage.Height:=PaintBox.Height;

  with PaintBoxBufferedImage.Canvas do
  begin
    Brush.Style:=bsSolid;

    NewBtnWidth:=0;
    CloseBtnWidth:=0;
    ListBtnWidth:=0;
    if(ShowButtons)then
    begin
      NewBtnWidth:=NewTabPNGImg.Width;
      if(Tabs.Count>1)then
        CloseBtnWidth:=CloseTabPNGImg.Width+3;
    end;

    //Set Font
    Font.Name:='Tahoma';
    Font.Style:=[];

    //Build strings

    //Get Tabs-Header WideString prior select button
    PrevSelTabs:='';
    for i:=0 to SelectedTab-1 do
    begin
      if(PrevSelTabs<>'')then
        PrevSelTabs:=PrevSelTabs+'  |  ';

      if(FHighlightedTab=i)then
        HighlightRect.Left:=NewBtnWidth+10+TextWidth(PrevSelTabs);

      PrevSelTabs:=PrevSelTabs+TTab(Tabs[i]).Caption;

      if(FHighlightedTab=i)then
        HighlightRect.Right:=NewBtnWidth+10+TextWidth(PrevSelTabs);
    end;

    if(SelectedTab<Tabs.Count)and(SelectedTab>=0)then
      SelTab:=TTab(Tabs[SelectedTab]).Caption;

    //Get StringWidths
    PrevSelTabsWidth:=0;
    if(PrevSelTabs<>'')then
      PrevSelTabsWidth:=TextWidth(PrevSelTabs)+12;

    Font.Style:=[fsBold];
    SelTabWidth:=TextWidth(SelTab);
    Font.Style:=[];

    //Get Tabs-Header WideString post select button
    PostSelTabs:='';
    for i:=SelectedTab+1 to Tabs.Count-1 do
    begin
      if(PostSelTabs<>'')then
        PostSelTabs:=PostSelTabs+'  |  ';

      if(FHighlightedTab=i)then
        HighlightRect.Left:=NewBtnWidth+4+PrevSelTabsWidth+1+SelTabWidth+14+9+TextWidth(PostSelTabs);

      PostSelTabs:=PostSelTabs+TTab(Tabs[i]).Caption;

      if(FHighlightedTab=i)then
        HighlightRect.Right:=NewBtnWidth+4+PrevSelTabsWidth+1+SelTabWidth+14+9+TextWidth(PostSelTabs);
    end;


    TotalWidth:=NewBtnWidth+4+PrevSelTabsWidth+1+SelTabWidth+14+9+
      TextWidth(PostSelTabs);


    //Top gray line
    Brush.Color:=clBtnFace;
    if(TopSpacer)then
    begin
      TopSpace:=4;
      FillRect(Rect(0, 0, Width, 4));
    end
    else
    begin
      TopSpace:=0;
      FillRect(Rect(0, 0, NewBtnWidth+4+PrevSelTabsWidth+1, 4));
    end;

    //Bottom white line
    Brush.Color:=clWhite;
    FillRect(Rect(1, 23-4+TopSpace, Width-1, Height));


    //Draw dark line
    Pen.Color:=$009C9B91;
    MoveTo(0, 30-4+TopSpace);
    LineTo(0, 22-4+TopSpace);
    //Don't draw selection if it is to far on the right
    if(NewBtnWidth+4+PrevSelTabsWidth+1+SelTabWidth+14+1<
      Width-CloseBtnWidth-ListBtnWidth-3)then
    begin
      LineTo(NewBtnWidth+4+PrevSelTabsWidth+1, 22-4+TopSpace);
      LineTo(NewBtnWidth+4+PrevSelTabsWidth+1, 4-4+TopSpace);
      LineTo(NewBtnWidth+4+PrevSelTabsWidth+1+SelTabWidth+14+1, 4-4+TopSpace);
      LineTo(NewBtnWidth+4+PrevSelTabsWidth+1+SelTabWidth+14+1, 22-4+TopSpace);
    end;
    LineTo(Width-1, 22-4+TopSpace);
    LineTo(Width-1, 30-4+TopSpace);

    if(DrawBottom)then
    begin
      LineTo(Width-1, Height-1);
      LineTo(0, Height-1);
      LineTo(0, 30-4+TopSpace);
    end;

    //Fill areas
    Brush.Color:=clBtnFace;
    FillRect(Rect(0, 4, NewBtnWidth+4+PrevSelTabsWidth+1, 22-4+TopSpace));
    FillRect(Rect(NewBtnWidth+4+PrevSelTabsWidth+1+SelTabWidth+14+1+1,
      4-4+TopSpace, Width, 22-4+TopSpace));

    if(NewBtnWidth+4+PrevSelTabsWidth+1+SelTabWidth+14+1<
      Width-CloseBtnWidth-ListBtnWidth-3)then
    begin
      Brush.Color:=clWhite;
      FillRect(Rect(NewBtnWidth+4+PrevSelTabsWidth+1+1, 5-4+TopSpace,
        NewBtnWidth+4+PrevSelTabsWidth+1+SelTabWidth+14+1, 23-4+TopSpace));
    end
    else //Don't draw selection if it is to far on the right
      FillRect(Rect(NewBtnWidth+4+PrevSelTabsWidth+1+1-1, 5-4+TopSpace-1,
        NewBtnWidth+4+PrevSelTabsWidth+1+SelTabWidth+14+1+1, 23-4+TopSpace-1));


    //Draw Buttons
    if(ShowButtons)then
    begin
      if(FHighlightedButton=0)then
        NewTabPNGImg.Draw(PaintBoxBufferedImage.Canvas,
          Rect(3, 0, 3+NewTabPNGImg.Width, NewTabPNGImg.Height))
      else
        NewTabPNGImg.DrawFaded(PaintBoxBufferedImage.Canvas,
          Rect(3, 0, 3+NewTabPNGImg.Width, NewTabPNGImg.Height), 60);

      if(Tabs.Count>1)then
      begin
        if(FHighlightedButton=1)then
          CloseTabPNGImg.Draw(PaintBoxBufferedImage.Canvas,
            Rect(Width-3-CloseTabPNGImg.Width, 0,
              Width-3, CloseTabPNGImg.Height))
        else
          CloseTabPNGImg.DrawFaded(PaintBoxBufferedImage.Canvas,
            Rect(Width-3-CloseTabPNGImg.Width, 0,
              Width-3, CloseTabPNGImg.Height), 60);
      end;

      if(TotalWidth>Width-3-CloseTabPNGImg.Width-3-ListTabPNGImg.Width)then
      begin
        if(FHighlightedButton=2)then
          ListTabPNGImg.Draw(PaintBoxBufferedImage.Canvas,
            Rect(Width-3-CloseTabPNGImg.Width-3-ListTabPNGImg.Width, 0,
            Width-3-CloseTabPNGImg.Width-3, ListTabPNGImg.Height))
        else
          ListTabPNGImg.DrawFaded(PaintBoxBufferedImage.Canvas,
            Rect(Width-3-CloseTabPNGImg.Width-3-ListTabPNGImg.Width, 0,
            Width-3-CloseTabPNGImg.Width-3, ListTabPNGImg.Height), 60);

        ListBtnWidth:=ListTabPNGImg.Width+3;

        TabListBtnShown:=True;
      end
      else
        TabListBtnShown:=False;
    end;

    //Paint HighlightedTab Rect
    if(FHighlightedTab>=0)and(FHighlightedTab<>FSelectedTab)then
    begin
      Brush.Style:=bsSolid;
      Brush.Color:=$00F5F5F5;

      HighlightRect.Left:=HighlightRect.Left-1;
      HighlightRect.Top:=3;
      HighlightRect.Right:=HighlightRect.Right+1;
      HighlightRect.Bottom:=16;

      FillRect(HighlightRect);

      HighlightRect.Left:=HighlightRect.Left-1;
      HighlightRect.Top:=4;
      HighlightRect.Right:=HighlightRect.Right+1;
      HighlightRect.Bottom:=15;

      FillRect(HighlightRect);
    end;

    Font.Color:=$00969696;
    Brush.Style:=bsClear;

    //PrevSelTabsWidth
    aRect:=Rect(NewBtnWidth+10, 7-4+TopSpace,
      Width-CloseBtnWidth-ListBtnWidth-3{NewBtnWidth+4+PrevSelTabsWidth-4}, 20-4+TopSpace);
    DrawTextW(Handle, PWideChar(PrevSelTabs),
      Length(PrevSelTabs), aRect, DT_LEFT, False);

    //PostSelTabs
    if(NewBtnWidth+4+PrevSelTabsWidth+1+SelTabWidth+14+9<
      Width-CloseBtnWidth-ListBtnWidth-3-5)then
    begin
      aRect:=Rect(NewBtnWidth+4+PrevSelTabsWidth+1+SelTabWidth+14+9, 7-4+TopSpace,
        Width-CloseBtnWidth-ListBtnWidth-3, 20-4+TopSpace);
      DrawTextW(Handle, PWideChar(PostSelTabs),
        Length(PostSelTabs), aRect, DT_LEFT, False);
    end;

    //SelTab
    if(NewBtnWidth+4+PrevSelTabsWidth+1+7+2<
      Width-CloseBtnWidth-ListBtnWidth-3-5)then
    begin
      Font.Color:=clBlack;
      Font.Style:=[fsBold];
      aRect:=Rect(NewBtnWidth+4+PrevSelTabsWidth+1+7+2, 7-4+TopSpace,
        Width-CloseBtnWidth-ListBtnWidth-3{NewBtnWidth+4+PrevSelTabsWidth+1+7+2+SelTabWidth}, 20-4+TopSpace);
      DrawTextW(Handle, PWideChar(SelTab),
        Length(SelTab), aRect, DT_LEFT, False);
    end;
  end;

  PaintBox.Canvas.Draw(0, 0, PaintBoxBufferedImage);
end;

function TTabHeaderFrame.GetSelectedTab: integer;
begin
  Result:=FSelectedTab;
end;

procedure TTabHeaderFrame.SetSelectedTab(SelectedTabIndex: integer);
var prevSelectedTab: integer;
begin
  if(SelectedTabIndex<Tabs.Count)then
  begin
    prevSelectedTab:=FSelectedTab;

    //Call event if assigned
    if(Assigned(FOnBeforePageChange))then
      FOnBeforePageChange(self, prevSelectedTab, SelectedTabIndex,
        TTab(Tabs[prevSelectedTab]).Obj);

    if(FSelectedTab<Tabs.Count)then
      if(Assigned(TTab(Tabs[FSelectedTab]).TabPanel))then
        TTab(Tabs[FSelectedTab]).TabPanel.Hide;

    FSelectedTab:=SelectedTabIndex;

    if(FSelectedTab<Tabs.Count)and(FSelectedTab>=0)then
      if(Assigned(TTab(Tabs[FSelectedTab]).TabPanel))then
      begin
        TTab(Tabs[FSelectedTab]).TabPanel.Show;
        TTab(Tabs[FSelectedTab]).TabPanel.BringToFront;
      end;

    //Call event if assigned
    if(Assigned(FOnPageChange))then
      FOnPageChange(self, prevSelectedTab, FSelectedTab,
        TTab(Tabs[FSelectedTab]).Obj);

    TabPaintBoxPaint(self);
  end;
end;

procedure TTabHeaderFrame.RefreshTabSheetsXPos;
var i, xpos, prevXpos: integer;
begin
  xpos:=10;
  if(ShowButtons)then
    xpos:=xpos+NewTabPNGImg.Width+2;

  with PaintBox.Canvas do
  begin
    //Build strings
    Font.Style:=[];
    for i:=0 to Tabs.Count-1 do
    begin
      if(i=SelectedTab)then
      begin
        if(i>0)then
          xpos:=xpos+14;
        xpos:=xpos+1;

        Font.Style:=[fsBold];
      end;

      prevXpos:=xpos;
      xpos:=xpos+TextWidth(TTab(Tabs[i]).Caption);

      TTab(Tabs[i]).x1:=prevXpos-2;
      TTab(Tabs[i]).x2:=xpos+2;

      if(i<SelectedTab-1)or
        ((i>SelectedTab)and(i<Tabs.Count-1))then
        xpos:=xpos+TextWidth('  |  ')
      else if(i=SelectedTab)then
        xpos:=xpos+14;

      if(i=SelectedTab)then
        Font.Style:=[];
    end;
  end;
end;

procedure TTabHeaderFrame.TabPaintBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var i: integer;
  Pnt: TPoint;
  MenuItem: TTntMenuItem;
begin
  if(Y<24)then
  begin
    //User clicked on New Tab icon
    if(X<24+3)and(ShowButtons)then
    begin
      if(Assigned(FOnRequestNewPage))then
        FOnRequestNewPage(self);
    end
    //User clicked on Tabsheet
    else if(X<Width-(24+3+Ord(TabListBtnShown)*(24+3))*Ord(ShowButtons))then
    begin
      for i:=0 to Tabs.Count-1 do
      begin
        if(X>=TTab(Tabs[i]).x1-4)and
          (X<TTab(Tabs[i]).x2+4)then
        begin
          SelectedTab:=i;
          RefreshTabSheetsXPos;

          break;
        end;
      end;

      TabPaintBoxPaint(self);
    end
    //User clicked on Tab List icon
    else if(X>=Width-24-3-24-3)and(X<Width-24-3)then
    begin
      Pnt:=TControl(Sender).ClientToScreen(Point(0, 0));

      TabsPopupMenu.Items.Clear;
      for i:=0 to Tabs.Count-1 do
      begin
        MenuItem:=TTntMenuItem.Create(TabsPopupMenu);
        MenuItem.Caption:=TTab(Tabs[i]).Caption;
        MenuItem.Tag:=i;
        MenuItem.OnClick:=TabMenuItemSelected;
        TabsPopupMenu.Items.Add(MenuItem);
      end;

      TabsPopupMenu.Popup(Pnt.X+Width-24-3-24-3+1, Pnt.Y+16);
    end
    //User clicked on Tab Close icon
    else if(X>=Width-24-3)then
    begin
      //Delete Tab
      DeleteTab(FSelectedTab);
    end;
  end;
end;

procedure TTabHeaderFrame.TabPaintBoxMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var i: integer;
begin
  FHighlightedTab:=-1;
  FHighlightedButton:=-1;

  if(Y<24)then
  begin
    //User clicked on New Tab icon
    if(X<24+3)and(ShowButtons)then
    begin
      FHighlightedButton:=0;

      TabPaintBoxPaint(self);
    end
    //User clicked on Tabsheet
    else if(X<Width-(24+3+Ord(TabListBtnShown)*(24+3))*Ord(ShowButtons))then
    begin
      for i:=0 to Tabs.Count-1 do
      begin
        if(X>=TTab(Tabs[i]).x1-4)and
          (X<TTab(Tabs[i]).x2+4)then
        begin
          FHighlightedTab:=i;

          break;
        end;
      end;

      TabPaintBoxPaint(self);
    end
    //User clicked on Tab List icon
    else if(X>=Width-24-3-24-3)and(X<Width-24-3)then
    begin
      FHighlightedButton:=2;

      TabPaintBoxPaint(self);
    end
    //User clicked on Tab Close icon
    else if(X>=Width-24-3)then
    begin
      FHighlightedButton:=1;

      TabPaintBoxPaint(self);
    end;
  end;
end;

procedure TTabHeaderFrame.TabPaintBoxMouseLeave(Sender: TObject);
begin
  FHighlightedTab:=-1;
  FHighlightedButton:=-1;

  TabPaintBoxPaint(self);
end;

procedure TTabHeaderFrame.DeleteTab(TabIndex: Integer);
var ShiftSelectedTab: Boolean;
begin
  if(Tabs.Count>1)then
  begin
    ShiftSelectedTab:=False;
    if(TabIndex=FSelectedTab)then
    begin
      //When the last tab is selected, now select one prior
      //else, select next
      if(FSelectedTab=Tabs.Count-1)then
        SelectedTab:=Tabs.Count-2
      else
      begin
        SelectedTab:=FSelectedTab+1;
        ShiftSelectedTab:=True;
      end;
    end
    else if(TabIndex>FSelectedTab)then
      ShiftSelectedTab:=True;

    if(Assigned(FOnPageDelete))then
      FOnPageDelete(Tabs[TabIndex]);

    if(TTab(Tabs[TabIndex]).Obj<>nil)then
      TTab(Tabs[TabIndex]).Obj.Free;

    if(TTab(Tabs[TabIndex]).TabPanel<>nil)then
      TTab(Tabs[TabIndex]).TabPanel.Free;

    Tabs.Delete(TabIndex);

    //if the deletion influences the selected tab
    //also shift the SelectedTab
    if(ShiftSelectedTab)then
      dec(FSelectedTab);

    RefreshTabSheetsXPos;
    TabPaintBoxPaint(self);

    if(Tabs.Count=1)and(AutoHideTabHeader)then
      Hide;
  end;
end;

procedure TTabHeaderFrame.TabMenuItemSelected(Sender: TObject);
begin
  SelectedTab:=TTntMenuItem(Sender).Tag;
end;

procedure TTabHeaderFrame.ClearTabSheets;
begin
  Tabs.Clear;
  TabPaintBoxPaint(self);
end;

function TTabHeaderFrame.AddTabSheet(AOwner: TComponent; Caption: WideString;
  TabPanel: TTntPanel; Obj: TObject; SelectNewPage: Boolean;
  DockTabPanel: Boolean; TabPanelHeight: integer): integer;
begin
  Tabs.Add(TTab.Create(AOwner, Caption, 0, 0, TabPanel, Obj, DockTabPanel));
  RefreshTabSheetsXPos;

  if(TabPanel<>nil)then
  begin
    TabPanel.Visible:=False;

    if(DockTabPanel)then
    begin
      TabPanel.Parent:=Parent;
      //Get new Handles for all docked controls
      InitHandles(TabPanel);

      TabPanel.Align:=alNone;
      TabPanel.Left:=Left+4;
      TabPanel.Top:=Top+23;
      TabPanel.Width:=Width-9;
      TabPanel.Height:=Height-28;
    end
    else if(TabPanelHeight>-1)then
    begin
      TabPanel.Left:=0;
      TabPanel.Width:=Width;

      if(Visible)then
      begin
        TabPanel.Top:=Top+24;
        TabPanel.Height:=TabPanelHeight-24;
      end
      else
      begin
        TabPanel.Top:=Top;
        TabPanel.Height:=TabPanelHeight;
      end;
    end;

    if(Align=alBottom)then
      TabPanel.Anchors:=[akLeft, akRight, akBottom]
    else
      TabPanel.Anchors:=[akLeft, akTop, akRight, akBottom];

    TabPanel.BringToFront;
    //Resize WinControl
    //TabPanel.Perform(WM_SIZE, 0, 0);
  end;

  if(SelectNewPage)then
    SelectedTab:=Tabs.Count-1
  else if(Visible)then
    TabPaintBoxPaint(self);

  Result:=Tabs.Count-1;
end;

function TTabHeaderFrame.TabCount: integer;
begin
  Result:=Tabs.Count;
end;

procedure TTabHeaderFrame.Show;
var i: integer;
begin
  if(Not(Visible))then
  begin
    for i:=0 to Tabs.Count-1 do
    begin
      TTab(Tabs[i]).TabPanel.Height:=TTab(Tabs[i]).TabPanel.Height-24;
      TTab(Tabs[i]).TabPanel.Top:=TTab(Tabs[i]).TabPanel.Top+24;
    end;
  end;

  inherited Show;
end;

procedure TTabHeaderFrame.Hide;
var i: integer;
begin
  if(Visible)then
  begin
    for i:=0 to Tabs.Count-1 do
    begin
      TTab(Tabs[i]).TabPanel.Top:=TTab(Tabs[i]).TabPanel.Top-24;
      TTab(Tabs[i]).TabPanel.Height:=TTab(Tabs[i]).TabPanel.Height+24;
    end;
  end;

  inherited Hide;
end;

procedure TTabHeaderFrame.TntFrameResize(Sender: TObject);
var i: integer;
begin
  for i:=0 to Tabs.Count-1 do
  begin
    if(TTab(Tabs[i]).DockTabPanel)and(TTab(Tabs[i]).TabPanel<>nil)then
    begin
      TTab(Tabs[i]).TabPanel.Left:=Left+4;
      TTab(Tabs[i]).TabPanel.Top:=Top+23;
      TTab(Tabs[i]).TabPanel.Width:=Width-9;
      TTab(Tabs[i]).TabPanel.Height:=Height-28;
    end;
  end;
end;

procedure TTabHeaderFrame.SelectNextTabSheet;
var NextTab: integer;
begin
  NextTab:=FSelectedTab+1;
  if(NextTab>=TabCount)then
    NextTab:=0;

  SelectedTab:=NextTab;
end;

function TTabHeaderFrame.GetTabIndex(Tab: TTab): integer;
begin
  Result:=Tabs.IndexOf(Tab);
end;


//--------------------------------------------------------------------------

constructor TTab.Create(AOwner: TComponent; Caption: WideString;
  x1: integer; x2: integer; TabPanel: TTntPanel; Obj: TObject;
  DockTabPanel: Boolean);
begin
  self.Caption:=Caption;
  self.x1:=x1;
  self.x2:=x2;
  self.TabPanel:=TabPanel;
  self.Obj:=Obj;
  self.DockTabPanel:=DockTabPanel;
end;


destructor TTab.Destroy;
begin
  inherited;
end;

end.
