unit Toolbar;

interface

uses
  Windows, Messages, Classes, Graphics, Forms, ExtCtrls, PNGImage, Controls,
  AuxFuncs, TntStdCtrls, TntExtCtrls;

type
  TToolbarItem = class;

  TPNGButton = class(TTntImage)
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure SetImages(NormalImg: WideString; DownImg: WideString;
      DisabledImg: WideString; HighlightImg: WideString);

    procedure DoMouseLeave(Sender: TObject);

    procedure ResetButton;
  private
    NormalPNGImg,
    DownPNGImg,
    DisabledPNGImg,
    HighlightPNGImg: TPNGObject;

    FEnabled: Boolean;

    FDown: Boolean;
    FClicked: Boolean;
    FMouseOver: Boolean;

    FOnBtnClick: TNotifyEvent;

    procedure CMMouseLeave(var Msg: TMessage); message CM_MOUSELEAVE;
    procedure RefreshImage;
  protected
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;

    procedure MouseLeave(Sender: TObject);

    function GetEnabled: Boolean; override;
    procedure SetEnabled(Enabled: Boolean); override;

    function GetDown: Boolean;
    procedure SetDown(Down: Boolean);
  public
    ToolbarItem: TToolbarItem;
  published
    property Down: Boolean read GetDown write SetDown;
    property OnBtnClick: TNotifyEvent read FOnBtnClick write FOnBtnClick;
  end;


  TToolbarItem = class(TObject)
    constructor Create(Toolbar: TWinControl;
      ImageResource: WideString;
      Caption: WideString; Hint: WideString;
      Left: Integer; Top: Integer;
      ShowLbl: Boolean; LblAtBottom: Boolean;
      AnchorRight: Boolean;
      OnBtnClick: TNotifyEvent;
      OnLblClick: TNotifyEvent = nil;
      GroupId: Integer = 0); reintroduce; overload;
    destructor Destroy; override;
  private
    FEnabled: Boolean;
    FDown: Boolean;

    function GetEnabled: Boolean;
    procedure SetEnabled(Enabled: Boolean);

    function GetDown: Boolean;
    procedure SetDown(Down: Boolean);
  public
    PNGBtn: TPNGButton;
    BtnLbl: TTntLabel;
    LblDropdownImg: TTntImage;
    LblDropdownPngImg,
    LblDropdownDisabledPngImg: TPNGObject;

    GroupId: Integer;
  published
    property Enabled: Boolean read GetEnabled write SetEnabled;
    property Down: Boolean read GetDown write SetDown;
  end;

implementation

constructor TPNGButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FEnabled:=True;
  FDown:=False;
  FClicked:=False;
  FMouseOver:=False;

  ControlStyle:=ControlStyle+[csOpaque];
end;

destructor TPNGButton.Destroy;
begin
  NormalPNGImg.Free;
  if(DownPNGImg<>nil)then
    DownPNGImg.Free;
  if(DisabledPNGImg<>nil)then
    DisabledPNGImg.Free;
  if(HighlightPNGImg<>nil)then
    HighlightPNGImg.Free;

  inherited;
end;

procedure TPNGButton.SetImages(NormalImg: WideString; DownImg: WideString;
  DisabledImg: WideString; HighlightImg: WideString);
begin
  NormalPNGImg:=LoadPNGImageFromResource(NormalImg, TTntImage(self));

  Width:=NormalPNGImg.Width;
  Height:=NormalPNGImg.Height;

  if(DownImg<>'')then
    DownPNGImg:=LoadPNGImageFromResource(DownImg)
  else
    DownPNGImg:=nil;

  if(DisabledImg<>'')then
    DisabledPNGImg:=LoadPNGImageFromResource(DisabledImg)
  else
    DisabledPNGImg:=nil;

  if(HighlightImg<>'')then
    HighlightPNGImg:=LoadPNGImageFromResource(HighlightImg)
  else
    HighlightPNGImg:=nil;
end;

procedure TPNGButton.CMMouseLeave(var Msg: TMessage);
begin
  inherited;

  MouseLeave(self);
end;

procedure TPNGButton.MouseLeave(Sender: TObject);
begin
  if(Enabled)then
  begin
    FMouseOver:=False;

    RefreshImage;
  end;
end;

procedure TPNGButton.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);

  if(Button=mbLeft)and(Enabled)then
  begin
    FClicked:=True;

    RefreshImage;
  end;
end;

procedure TPNGButton.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseMove(Shift, X, Y);

  if(FMouseOver=False)and(Enabled)then
  begin
    FMouseOver:=True;

    RefreshImage;
  end;
end;

procedure TPNGButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);

  if FClicked then
  begin
    FClicked:=False;

    if((X<0)or(X>=Width)and
      (Y<0)or(Y>=Height))then
      FMouseOver:=False;

    RefreshImage;
  end;

  if(Assigned(FOnBtnClick))then
    FOnBtnClick(self);
end;

procedure TPNGButton.RefreshImage;
begin
  if(Not(Enabled))then
    Picture.Assign(DisabledPNGImg)
  else if(FDown)or(FClicked)then
    Picture.Assign(DownPNGImg)
  else if(FMouseOver)then
    Picture.Assign(HighlightPNGImg)
  else
    Picture.Assign(NormalPNGImg);
end;

procedure TPNGButton.ResetButton;
begin
  FDown:=False;
  FClicked:=False;
  FMouseOver:=False;

  RefreshImage;
end;

function TPNGButton.GetEnabled: Boolean;
begin
  Result:=FEnabled;
end;

procedure TPNGButton.SetEnabled(Enabled: Boolean);
begin
  FMouseOver:=False;

  self.FEnabled:=Enabled;

  RefreshImage;
end;

procedure TPNGButton.DoMouseLeave(Sender: TObject);
begin
  MouseLeave(nil);
end;

function TPNGButton.GetDown: Boolean;
begin
  Result:=FDown;
end;

procedure TPNGButton.SetDown(Down: Boolean);
begin
  FDown:=Down;

  RefreshImage;
end;

// -----------------------------------------------------------------------------

constructor TToolbarItem.Create(Toolbar: TWinControl;
  ImageResource: WideString;
  Caption: WideString; Hint: WideString;
  Left: Integer; Top: Integer;
  ShowLbl: Boolean; LblAtBottom: Boolean;
  AnchorRight: Boolean;
  OnBtnClick: TNotifyEvent;
  OnLblClick: TNotifyEvent;
  GroupId: Integer);
begin
  inherited Create;

  self.GroupId:=GroupId;

  PNGBtn:=TPNGButton.Create(Toolbar);
  PNGBtn.SetImages(ImageResource, ImageResource+'_down',
    ImageResource+'_disabled', ImageResource+'_highlight');
  PNGBtn.Parent:=Toolbar;
  PNGBtn.ToolbarItem:=self;
  PNGBtn.Left:=Left;
  PNGBtn.Top:=Top;
  if(AnchorRight)then
    PNGBtn.Anchors:=[akTop, akRight];

  if(Hint<>'')then
  begin
    PNGBtn.Hint:=Hint;
    PNGBtn.ShowHint:=True;
  end;

  PNGBtn.OnBtnClick:=OnBtnClick;

  if(ShowLbl)then
  begin
    BtnLbl:=TTntLabel.Create(Toolbar);
    BtnLbl.Parent:=Toolbar;
    BtnLbl.Font.Name:='Tahoma';
    BtnLbl.Font.Size:=7;
    BtnLbl.Caption:=Caption;
    BtnLbl.Transparent:=True;

    if(LblAtBottom)then
    begin
      BtnLbl.Left:=PNGBtn.Left+(PNGBtn.Width-BtnLbl.Width) div 2;
      BtnLbl.Top:=PNGBtn.Top+PNGBtn.Height+1;
    end
    else
    begin
      BtnLbl.Left:=PNGBtn.Left+PNGBtn.Width+8;
      BtnLbl.Top:=PNGBtn.Top+(PNGBtn.Height-BtnLbl.Height) div 2;
    end;

    if(AnchorRight)then
      BtnLbl.Anchors:=[akTop, akRight];


    if(Hint<>'')then
    begin
      BtnLbl.Hint:=Hint;
      BtnLbl.ShowHint:=True;
    end;

    //Add dropdown
    if(Assigned(OnLblClick))then
    begin
      BtnLbl.OnClick:=OnLblClick;

      BtnLbl.AutoSize:=False;
      LblDropdownImg:=TTntImage.Create(Toolbar);
      LblDropdownImg.Parent:=Toolbar;
      LblDropdownImg.Left:=BtnLbl.Left+BtnLbl.Width+2;
      LblDropdownImg.Top:=BtnLbl.Top+BtnLbl.Height-6;
      LblDropdownPngImg:=LoadPNGImageFromResource('label_extra', LblDropdownImg);
      LblDropdownDisabledPngImg:=LoadPNGImageFromResource('label_extra_disabled');

      if(AnchorRight)then
        LblDropdownImg.Anchors:=[akTop, akRight];

      BtnLbl.Width:=BtnLbl.Width+7;
      BtnLbl.BringToFront;
    end
    else
    begin
      LblDropdownPngImg:=nil;
      LblDropdownDisabledPngImg:=nil;
      BtnLbl.OnClick:=OnBtnClick;
    end;
  end
  else
    BtnLbl:=nil;
end;

destructor TToolbarItem.Destroy;
begin
  if(LblDropdownPngImg<>nil)then
    LblDropdownPngImg.Free;
  if(LblDropdownPngImg<>nil)then
    LblDropdownDisabledPngImg.Free;
  inherited;
end;

function TToolbarItem.GetEnabled: Boolean;
begin
  Result:=FEnabled;
end;

procedure TToolbarItem.SetEnabled(Enabled: Boolean);
begin
  FEnabled:=Enabled;
  PNGBtn.Enabled:=Enabled;
  if(Assigned(BtnLbl))then
    BtnLbl.Enabled:=Enabled;

  if(LblDropdownPngImg<>nil)then
  begin
    if(FEnabled)then
      LblDropdownImg.Picture.Assign(LblDropdownPngImg)
    else
      LblDropdownImg.Picture.Assign(LblDropdownDisabledPngImg);
  end;
end;

function TToolbarItem.GetDown: Boolean;
begin
  Result:=FDown;
end;

procedure TToolbarItem.SetDown(Down: Boolean);
begin
  FDown:=Down;
end;


end.
