unit UCESQLHighlighter;

//----------------------------------------------------------------------------------------------------------------------
//
// UniCodeEditor, a Unicode Source Code Editor for Delphi.
// Copyright (c) 1999, 2000, 2004 by Dipl. Ing. Mike Lischke (public@lischke-online.de)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//----------------------------------------------------------------------------------------------------------------------
//
// Description
//   This unit contains the implementation of a syntax highlighter for the Unicode Syntax Edit control (USE).
//   The code below has been created by DCG, the Delphi Compiler Generator.
//
// Changes:
//   Version 2.0, 2003-08-17, Mike Zinner
//     Repackaged for D7
//   Version 1.0, 1999-03-10, Mike Lischke
//     Initial Version
//
//----------------------------------------------------------------------------------------------------------------------

interface
 
uses Classes, Graphics, UCEHighlighter;
 
const CR = #13;
      LF = #10;
 
// user defined start states
 
const Normal = 2;
      Comment1 = 4;
      Comment2 = 6;

 
const kwACTIVE = 257;
      kwADD = 258;
      kwAFTER = 259;
      kwALL = 260;
      kwALLOWED = 261;
      kwALTER = 262;
      kwAND = 263;
      kwANY = 264;
      kwAS = 265;
      kwASC = 266;
      kwASCENDING = 267;
      kwAT = 268;
      kwAUDIT = 269;
      kwAUTO = 270;
      kwAUTOINC = 271;
      kwAVG = 272;
      kwBASE_NAME = 273;
      kwBEFORE = 274;
      kwBEGIN = 275;
      kwBETWEEN = 276;
      kwBLOB = 277;
      kwBOOLEAN = 278;
      kwBOTH = 279;
      kwBY = 280;
      kwBYTES = 281;
      kwCACHE = 282;
      kwCAST = 283;
      kwCHAR = 284;
      kwCHARACTER = 285;
      kwCHECK = 286;
      kwCHECK_POINT_LENGTH = 287;
      kwCLOSE = 288;
      kwCOLLATE = 289;
      kwCOLUMN = 290;
      kwCOMMENT = 291;
      kwCOMMIT = 292;
      kwCOMMITTED = 293;
      kwCOMPOSITE = 294;
      kwCOMPUTED = 295;
      kwCONDITIONAL = 296;
      kwCONSTRAINT = 297;
      kwCONTAINING = 298;
      kwCOUNT = 299;
      kwCREATE = 300;
      kwCSTRING = 301;
      kwCURRENT = 302;
      kwCURSOR = 303;
      kwDATABASE = 304;
      kwDATE = 305;
      kwDAY = 306;
      kwDEBUG = 307;
      kwDEC = 308;
      kwDECIMAL = 309;
      kwDECLARE = 310;
      kwDEFAULT = 311;
      kwDELETE = 312;
      kwDESC = 313;
      kwDESCENDING = 314;
      kwDISTINCT = 315;
      kwDIVIDEBY = 316;
      kwDO = 317;
      kwDOMAIN = 318;
      kwDOUBLE = 319;
      kwDROP = 320;
      kwDUMP = 321;
      kwELSE = 322;
      kwEND = 323;
      kwENTRY_POINT = 324;
      kwESCAPE = 325;
      kwEXCEPTION = 326;
      kwEXECUTE = 327;
      kwEXISTS = 328;
      kwEXIT = 329;
      kwEXTERNAL = 330;
      kwEXTRACT = 331;
      kwFILE = 332;
      kwFILTER = 333;
      kwFLOAT = 334;
      kwFOR = 335;
      kwFOREIGN = 336;
      kwFRAGMENT = 337;
      kwFROM = 338;
      kwFULL = 339;
      kwFUNCTION = 340;
      kwGDSCODE = 341;
      kwGENERATOR = 342;
      kwGEN_ID = 343;
      kwGRANT = 344;
      kwGROUP = 345;
      kwGROUP_COMMIT_WAIT_TIME = 346;
      kwHAVING = 347;
      kwHOUR = 348;
      kwIF = 349;
      kwIN = 350;
      kwINACTIVE = 351;
      kwINDEX = 352;
      kwINFO = 353;
      kwINNER = 354;
      kwINPUT_TYPE = 355;
      kwINSERT = 356;
      kwINT = 357;
      kwINTEGER = 358;
      kwINTERSECT = 359;
      kwINTO = 360;
      kwIS = 361;
      kwISOLATION = 362;
      kwJOIN = 363;
      kwKEY = 364;
      kwLEADING = 365;
      kwLEFT = 366;
      kwLENGTH = 367;
      kwLEVEL = 368;
      kwLIKE = 369;
      kwLOCK = 370;
      kwLOGFILE = 371;
      kwLOG_BUFFER_SIZE = 372;
      kwLONG = 373;
      kwLOWER = 374;
      kwMANUAL = 375;
      kwMAX = 376;
      kwMAXIMUM_SEGMENT = 377;
      kwMERGE = 378;
      kwMESSAGE = 379;
      kwMIN = 380;
      kwMINUTE = 381;
      kwMODULE_NAME = 382;
      kwMONEY = 383;
      kwMONTH = 384;
      kwNAMES = 385;
      kwNATIONAL = 386;
      kwNATURAL = 387;
      kwNCHAR = 388;
      kwNO = 389;
      kwNOT = 390;
      kwNULL = 391;
      kwNUMERIC = 392;
      kwNUM_LOG_BUFFERS = 393;
      kwOF = 394;
      kwON = 395;
      kwONLY = 396;
      kwOPEN = 397;
      kwOPTION = 398;
      kwOR = 399;
      kwORDER = 400;
      kwOUTER = 401;
      kwOUTPUT_TYPE = 402;
      kwOVERFLOWPAGE_SIZE = 403;
      kwPAGE = 404;
      kwPAGES = 405;
      kwPARAMETER = 406;
      kwPASSWORD = 407;
      kwPLAN = 408;
      kwPOSITION = 409;
      kwPOST_EVENT = 410;
      kwPRECISION = 411;
      kwPRIMARY = 412;
      kwPRIVILEGES = 413;
      kwPROCEDURE = 414;
      kwPROGRAM = 415;
      kwPROTECTED = 416;
      kwRAW_PARTITIONS = 417;
      kwREAD = 418;
      kwREAL = 419;
      kwRECORD_VERSION = 420;
      kwREFERENCES = 421;
      kwRESERV = 422;
      kwRESERVING = 423;
      kwRETAIN = 424;
      kwRETURNING_VALUES = 425;
      kwRETURNS = 426;
      kwREVOKE = 427;
      kwRIGHT = 428;
      kwROLLBACK = 429;
      kwROLLFORWARD = 430;
      kwSCHEMA = 431;
      kwSECOND = 432;
      kwSEGMENT = 433;
      kwSELECT = 434;
      kwSET = 435;
      kwSHADOW = 436;
      kwSHARED = 437;
      kwSHOW = 438;
      kwSINGULAR = 439;
      kwSIZE = 440;
      kwSMALLINT = 441;
      kwSNAPSHOT = 442;
      kwSOME = 443;
      kwSORT = 444;
      kwSQLCODE = 445;
      kwSTABILITY = 446;
      kwSTART = 447;
      kwSTARTING = 448;
      kwSTARTS = 449;
      kwSTATISTICS = 450;
      kwSTORE = 451;
      kwSUBSTRING = 452;
      kwSUB_TYPE = 453;
      kwSUM = 454;
      kwSUSPEND = 455;
      kwSYNC = 456;
      kwSYNONYM = 457;
      kwTABLE = 458;
      kwTHEN = 459;
      kwTIME = 460;
      kwTIMESTAMP = 461;
      kwTIMEZONE_HOUR = 462;
      kwTIMEZONE_MINUTE = 463;
      kwTO = 464;
      kwTRAILING = 465;
      kwTRANSACTION = 466;
      kwTRIGGER = 467;
      kwTRIM = 468;
      kwTYPE = 469;
      kwUNCOMMITTED = 470;
      kwUNION = 471;
      kwUNIQUE = 472;
      kwUPDATE = 473;
      kwUPPER = 474;
      kwUSER = 475;
      kwVALUE = 476;
      kwVALUES = 477;
      kwVARCHAR = 478;
      kwVARIABLE = 479;
      kwVARYING = 480;
      kwVIEW = 481;
      kwWAIT = 482;
      kwWHEN = 483;
      kwWHERE = 484;
      kwWHILE = 485;
      kwWITH = 486;
      kwWORK = 487;
      kwWRITE = 488;
      kwYEAR = 489;

      INTEGERNUMBER = 490;
      FLOATNUMBER = 491;
      WHITESPACE = 492;
      IDENTIFIER = 493;
      STRINGCONSTANT = 494;
      MLCOMMENT = 495;
      SLCOMMENT = 496;
      SYMBOL = 497;

      kwUSE = 1000;
 
type
  TUCESQLHighlighter = class(TUCEHighlighter)
  private
    FCurrentToken: Integer;            // type of current token
    FInput: PChar;                     // input string to be tokenized
    FCurrentPos: Cardinal;             // current lexer position in FInput
    FTokenPos: Cardinal;               // current token position in FInput
    FLookahead: Char;                  // next char after current position
    FCurrentState,
    FStartState,
    FLineState: Cardinal;              // lexer states
    FLastChar: Char;                   // last matched char (#0 if no match)
    FPositions: array of Cardinal;     // for each rule the last marked position, zeroed when
                                       // rule has already been considered (allocated once)
    FMatchStack: array of Cardinal;    // stack containing matched rules (growing dynamically)
    FMatchCount: Integer;              // contains current number of matches
    FRejected: Boolean;                // current match rejected?
    FDoStop: Boolean;                  // token or eof found?
    FEOL: Boolean;
 
    FStringAttributes,
    FNumberAttributes,
    FKeyAttributes,
    FSymbolAttributes,
    FCommentAttributes,
    FIdentifierAttributes,
    FSpaceAttributes: THighlightAttributes;
    procedure RuleToToken(Rule: Integer);
    function IsKeyword: Boolean;
    function GetNextChar: Char;
    function MatchRule(var Rule: Integer): Boolean;
    function DoDefault: Boolean;
    function GetCurrentChar: Char;
    procedure SetAttribute(const Index: Integer; const Value: THighlightAttributes);
    procedure HighlightChange(Sender: TObject);
  protected
    procedure Reset; virtual;
    procedure SetMatch(I: Integer);
    procedure SetState(Value: Cardinal); virtual;
    procedure SetToken(Token: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
 
    function EOL: Boolean; override;
    function GetToken: String; override;
    function GetTokenInfo: TTokenData; override;
    procedure Next; override;
    procedure Reject;
    function GetAttribute(Index: Integer): THighlightAttributes; override;
    function GetAttributeCount: Integer; override;
    function GetIdentChars: TIdentChars; override;
    function GetLanguageName: String; override;
    function GetRange: Pointer; override;
    procedure ResetRange; override;
    procedure SetRange(Value: Pointer); override;
    procedure SetLine(const NewValue: String); override;
 
    property CurrentChar: Char read GetCurrentChar;
    property NextChar: Char read GetNextChar;
    property Lookahead: Char read FLookahead;
    property State: Cardinal read FCurrentState write SetState; // read to get current lexer state
                                                                // write to set start state for next run
    property TokenPosition: Cardinal read FTokenPos;
  published
    property CommentAttributes: THighlightAttributes index 1 read FCommentAttributes write SetAttribute;
    property IdentifierAttributes: THighlightAttributes index 2 read FIdentifierAttributes write SetAttribute;
    property KeyAttributes: THighlightAttributes index 3 read FKeyAttributes write SetAttribute;
    property NumberAttributes: THighlightAttributes index 4 read FNumberAttributes write SetAttribute;
    property SpaceAttributes: THighlightAttributes index 5 read FSpaceAttributes write SetAttribute;
    property StringAttributes: THighlightAttributes index 6 read FStringAttributes write SetAttribute;
    property SymbolAttributes: THighlightAttributes index 7 read FSymbolAttributes write SetAttribute;
  end;
 
//--------------------------------------------------------------------------------
 
implementation
 
uses SysUtils;

// DFA table
type TTransition = record
       CharClass : set of Char;
       NextState : Integer;
     end;

const
  MarkPositionCount = 30;
  MatchCount        = 30;
  TransitionCount   = 57;
  StateCount        = 32;

  MarkPositionTable : array[0..MarkPositionCount - 1] of Integer = (
    7, 7, 11, 11, 14, 0, 13, 2, 3, 13, 4, 13, 5, 13, 6, 13, 13, 2, 13, 13, 7, 8, 
    11, 12, 0, 1, 3, 9, 10, 1);

  MatchTable : array [0..MatchCount - 1] of Integer = (
    7, 7, 11, 11, 14, 0, 13, 2, 3, 13, 4, 13, 5, 13, 6, 13, 13, 2, 13, 13, 7, 8,
    11, 12, 0, 1, 3, 9, 10, 1);

  TransitionTable : array [0..TransitionCount - 1] of TTransition = (
    (CharClass: [#0]; NextState: 8),
    (CharClass: [#0]; NextState: 8),
    (CharClass: [#0]; NextState: 8),
    (CharClass: [#1..#9, #11..' ']; NextState: 16),
    (CharClass: [#10]; NextState: 10),
    (CharClass: ['!', '#'..'&', '('..'.', ':'..'@', '['..'^', '|'..#255]; NextState: 17),
    (CharClass: ['"', '`']; NextState: 13),
    (CharClass: ['''']; NextState: 12),
    (CharClass: ['/']; NextState: 15),
    (CharClass: ['0'..'9']; NextState: 9),
    (CharClass: ['A'..'Z', '_', 'a'..'z']; NextState: 11),
    (CharClass: ['{']; NextState: 14),
    (CharClass: [#0]; NextState: 8),
    (CharClass: [#1..#9, #11..' ']; NextState: 16),
    (CharClass: [#10]; NextState: 10),
    (CharClass: ['!', '#'..'&', '('..'.', ':'..'@', '['..'^', '|'..#255]; NextState: 17),
    (CharClass: ['"', '`']; NextState: 13),
    (CharClass: ['''']; NextState: 12),
    (CharClass: ['/']; NextState: 15),
    (CharClass: ['0'..'9']; NextState: 9),
    (CharClass: ['A'..'Z', '_', 'a'..'z']; NextState: 11),
    (CharClass: ['{']; NextState: 14),
    (CharClass: [#0]; NextState: 8),
    (CharClass: [#1..'|', '~'..#255]; NextState: 18),
    (CharClass: ['}']; NextState: 19),
    (CharClass: [#0]; NextState: 8),
    (CharClass: [#1..'|', '~'..#255]; NextState: 18),
    (CharClass: ['}']; NextState: 19),
    (CharClass: [#0]; NextState: 8),
    (CharClass: [#1..')', '+'..#255]; NextState: 20),
    (CharClass: ['*']; NextState: 21),
    (CharClass: [#0]; NextState: 8),
    (CharClass: [#1..')', '+'..#255]; NextState: 20),
    (CharClass: ['*']; NextState: 21),
    (CharClass: ['.']; NextState: 23),
    (CharClass: ['0'..'9']; NextState: 22),
    (CharClass: ['E', 'e']; NextState: 24),
    (CharClass: [#1..' ']; NextState: 10),
    (CharClass: ['0'..'9', 'A'..'Z', '_', 'a'..'z']; NextState: 25),
    (CharClass: ['*']; NextState: 27),
    (CharClass: ['/']; NextState: 26),
    (CharClass: [#1..' ']; NextState: 10),
    (CharClass: [#1..'|', '~'..#255]; NextState: 18),
    (CharClass: [#1..')', '+'..#255]; NextState: 20),
    (CharClass: ['.']; NextState: 23),
    (CharClass: ['0'..'9']; NextState: 22),
    (CharClass: ['E', 'e']; NextState: 24),
    (CharClass: ['0'..'9']; NextState: 23),
    (CharClass: ['E', 'e']; NextState: 28),
    (CharClass: ['+', '-']; NextState: 29),
    (CharClass: ['0'..'9']; NextState: 30),
    (CharClass: ['0'..'9', 'A'..'Z', '_', 'a'..'z']; NextState: 25),
    (CharClass: ['+', '-']; NextState: 31),
    (CharClass: ['0'..'9']; NextState: 30),
    (CharClass: ['0'..'9']; NextState: 30),
    (CharClass: ['0'..'9']; NextState: 30),
    (CharClass: ['0'..'9']; NextState: 30)
    );

  MarksLow : array [0..StateCount-1] of Integer = (
    0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 7, 8, 10, 12, 14, 16, 17, 19, 20, 21, 22, 23, 
    24, 25, 26, 26, 27, 28, 29, 29, 29, 30);

  MarksHigh : array [0..StateCount-1] of Integer = (
    -1, -1, -1, -1, 0, 1, 2, 3, 4, 6, 7, 9, 11, 13, 15, 16, 18, 19, 20, 21, 22, 
    23, 24, 25, 25, 26, 27, 28, 28, 28, 29, 29);

  MatchesLow : array [0..StateCount-1] of Integer = (
    0, 0, 0, 0, 0, 1, 2, 3, 4, 5, 7, 8, 10, 12, 14, 16, 17, 19, 20, 21, 22, 23, 
    24, 25, 26, 26, 27, 28, 29, 29, 29, 30);

  MatchesHigh : array [0..StateCount-1] of Integer = (
    -1, -1, -1, -1, 0, 1, 2, 3, 4, 6, 7, 9, 11, 13, 15, 16, 18, 19, 20, 21, 22,
    23, 24, 25, 25, 26, 27, 28, 28, 28, 29, 29);

  TransitionsLow : array [0..StateCount-1] of Integer = (
    0, 1, 2, 12, 22, 25, 28, 31, 34, 34, 37, 38, 39, 39, 39, 39, 41, 42, 42, 43, 
    43, 44, 44, 47, 49, 51, 52, 52, 52, 54, 55, 56);

  TransitionsHigh : array [0..StateCount-1] of Integer = (
    0, 1, 11, 21, 24, 27, 30, 33, 33, 36, 37, 38, 38, 38, 38, 40, 41, 41, 42, 42, 
    43, 43, 46, 48, 50, 51, 51, 51, 53, 54, 55, 56);

type // keyword table support
     TKeyword = record
       Keyword: String;
       Token: Integer;
     end;

const KeywordCount = 234;
      Keywords : array[0..KeywordCount - 1] of TKeyword = (
        (Keyword: 'active'; Token: kwACTIVE), 
        (Keyword: 'add'; Token: kwADD), 
        (Keyword: 'after'; Token: kwAFTER), 
        (Keyword: 'all'; Token: kwALL), 
        (Keyword: 'allowed'; Token: kwALLOWED), 
        (Keyword: 'alter'; Token: kwALTER), 
        (Keyword: 'and'; Token: kwAND), 
        (Keyword: 'any'; Token: kwANY), 
        (Keyword: 'as'; Token: kwAS), 
        (Keyword: 'asc'; Token: kwASC), 
        (Keyword: 'ascending'; Token: kwASCENDING),
        (Keyword: 'at'; Token: kwAT), 
        (Keyword: 'audit'; Token: kwAUDIT), 
        (Keyword: 'auto'; Token: kwAUTO), 
        (Keyword: 'autoinc'; Token: kwAUTOINC), 
        (Keyword: 'avg'; Token: kwAVG), 
        (Keyword: 'base_name'; Token: kwBASE_NAME),
        (Keyword: 'before'; Token: kwBEFORE), 
        (Keyword: 'begin'; Token: kwBEGIN),
        (Keyword: 'between'; Token: kwBETWEEN), 
        (Keyword: 'blob'; Token: kwBLOB), 
        (Keyword: 'boolean'; Token: kwBOOLEAN), 
        (Keyword: 'both'; Token: kwBOTH), 
        (Keyword: 'by'; Token: kwBY), 
        (Keyword: 'bytes'; Token: kwBYTES),
        (Keyword: 'cache'; Token: kwCACHE), 
        (Keyword: 'cast'; Token: kwCAST), 
        (Keyword: 'char'; Token: kwCHAR), 
        (Keyword: 'character'; Token: kwCHARACTER),
        (Keyword: 'check'; Token: kwCHECK), 
        (Keyword: 'check_point_length'; Token: kwCHECK_POINT_LENGTH),
        (Keyword: 'close'; Token: kwCLOSE), 
        (Keyword: 'collate'; Token: kwCOLLATE),
        (Keyword: 'column'; Token: kwCOLUMN), 
        (Keyword: 'comment'; Token: kwCOMMENT), 
        (Keyword: 'commit'; Token: kwCOMMIT),
        (Keyword: 'committed'; Token: kwCOMMITTED), 
        (Keyword: 'composite'; Token: kwCOMPOSITE), 
        (Keyword: 'computed'; Token: kwCOMPUTED), 
        (Keyword: 'conditional'; Token: kwCONDITIONAL), 
        (Keyword: 'constraint'; Token: kwCONSTRAINT), 
        (Keyword: 'containing'; Token: kwCONTAINING), 
        (Keyword: 'count'; Token: kwCOUNT),
        (Keyword: 'create'; Token: kwCREATE), 
        (Keyword: 'cstring'; Token: kwCSTRING), 
        (Keyword: 'current'; Token: kwCURRENT), 
        (Keyword: 'cursor'; Token: kwCURSOR), 
        (Keyword: 'database'; Token: kwDATABASE), 
        (Keyword: 'date'; Token: kwDATE), 
        (Keyword: 'day'; Token: kwDAY), 
        (Keyword: 'debug'; Token: kwDEBUG), 
        (Keyword: 'dec'; Token: kwDEC), 
        (Keyword: 'decimal'; Token: kwDECIMAL), 
        (Keyword: 'declare'; Token: kwDECLARE),
        (Keyword: 'default'; Token: kwDEFAULT), 
        (Keyword: 'delete'; Token: kwDELETE),
        (Keyword: 'desc'; Token: kwDESC), 
        (Keyword: 'descending'; Token: kwDESCENDING), 
        (Keyword: 'distinct'; Token: kwDISTINCT), 
        (Keyword: 'divideby'; Token: kwDIVIDEBY), 
        (Keyword: 'do'; Token: kwDO), 
        (Keyword: 'domain'; Token: kwDOMAIN),
        (Keyword: 'double'; Token: kwDOUBLE), 
        (Keyword: 'drop'; Token: kwDROP), 
        (Keyword: 'dump'; Token: kwDUMP), 
        (Keyword: 'else'; Token: kwELSE),
        (Keyword: 'end'; Token: kwEND), 
        (Keyword: 'entry_point'; Token: kwENTRY_POINT),
        (Keyword: 'escape'; Token: kwESCAPE), 
        (Keyword: 'exception'; Token: kwEXCEPTION),
        (Keyword: 'execute'; Token: kwEXECUTE), 
        (Keyword: 'exists'; Token: kwEXISTS), 
        (Keyword: 'exit'; Token: kwEXIT),
        (Keyword: 'external'; Token: kwEXTERNAL), 
        (Keyword: 'extract'; Token: kwEXTRACT),
        (Keyword: 'file'; Token: kwFILE), 
        (Keyword: 'filter'; Token: kwFILTER), 
        (Keyword: 'float'; Token: kwFLOAT), 
        (Keyword: 'for'; Token: kwFOR), 
        (Keyword: 'foreign'; Token: kwFOREIGN), 
        (Keyword: 'fragment'; Token: kwFRAGMENT), 
        (Keyword: 'from'; Token: kwFROM), 
        (Keyword: 'full'; Token: kwFULL), 
        (Keyword: 'function'; Token: kwFUNCTION), 
        (Keyword: 'gdscode'; Token: kwGDSCODE), 
        (Keyword: 'generator'; Token: kwGENERATOR), 
        (Keyword: 'gen_id'; Token: kwGEN_ID), 
        (Keyword: 'grant'; Token: kwGRANT), 
        (Keyword: 'group'; Token: kwGROUP), 
        (Keyword: 'group_commit_wait_time'; Token: kwGROUP_COMMIT_WAIT_TIME), 
        (Keyword: 'having'; Token: kwHAVING),
        (Keyword: 'hour'; Token: kwHOUR), 
        (Keyword: 'if'; Token: kwIF),
        (Keyword: 'in'; Token: kwIN), 
        (Keyword: 'inactive'; Token: kwINACTIVE), 
        (Keyword: 'index'; Token: kwINDEX), 
        (Keyword: 'info'; Token: kwINFO), 
        (Keyword: 'inner'; Token: kwINNER), 
        (Keyword: 'input_type'; Token: kwINPUT_TYPE),
        (Keyword: 'insert'; Token: kwINSERT), 
        (Keyword: 'int'; Token: kwINT), 
        (Keyword: 'integer'; Token: kwINTEGER), 
        (Keyword: 'intersect'; Token: kwINTERSECT),
        (Keyword: 'into'; Token: kwINTO), 
        (Keyword: 'is'; Token: kwIS),
        (Keyword: 'isolation'; Token: kwISOLATION), 
        (Keyword: 'join'; Token: kwJOIN),
        (Keyword: 'key'; Token: kwKEY), 
        (Keyword: 'leading'; Token: kwLEADING), 
        (Keyword: 'left'; Token: kwLEFT),
        (Keyword: 'length'; Token: kwLENGTH), 
        (Keyword: 'level'; Token: kwLEVEL), 
        (Keyword: 'like'; Token: kwLIKE), 
        (Keyword: 'lock'; Token: kwLOCK), 
        (Keyword: 'logfile'; Token: kwLOGFILE), 
        (Keyword: 'log_buffer_size'; Token: kwLOG_BUFFER_SIZE), 
        (Keyword: 'long'; Token: kwLONG), 
        (Keyword: 'lower'; Token: kwLOWER), 
        (Keyword: 'manual'; Token: kwMANUAL), 
        (Keyword: 'max'; Token: kwMAX), 
        (Keyword: 'maximum_segment'; Token: kwMAXIMUM_SEGMENT), 
        (Keyword: 'merge'; Token: kwMERGE), 
        (Keyword: 'message'; Token: kwMESSAGE), 
        (Keyword: 'min'; Token: kwMIN), 
        (Keyword: 'minute'; Token: kwMINUTE), 
        (Keyword: 'module_name'; Token: kwMODULE_NAME), 
        (Keyword: 'money'; Token: kwMONEY), 
        (Keyword: 'month'; Token: kwMONTH),
        (Keyword: 'names'; Token: kwNAMES), 
        (Keyword: 'national'; Token: kwNATIONAL),
        (Keyword: 'natural'; Token: kwNATURAL), 
        (Keyword: 'nchar'; Token: kwNCHAR), 
        (Keyword: 'no'; Token: kwNO), 
        (Keyword: 'not'; Token: kwNOT), 
        (Keyword: 'null'; Token: kwNULL), 
        (Keyword: 'numeric'; Token: kwNUMERIC),
        (Keyword: 'num_log_buffers'; Token: kwNUM_LOG_BUFFERS), 
        (Keyword: 'of'; Token: kwOF), 
        (Keyword: 'on'; Token: kwON),
        (Keyword: 'only'; Token: kwONLY),
        (Keyword: 'open'; Token: kwOPEN), 
        (Keyword: 'option'; Token: kwOPTION),
        (Keyword: 'or'; Token: kwOR), 
        (Keyword: 'order'; Token: kwORDER),
        (Keyword: 'outer'; Token: kwOUTER), 
        (Keyword: 'output_type'; Token: kwOUTPUT_TYPE), 
        (Keyword: 'overflowpage_size'; Token: kwOVERFLOWPAGE_SIZE),
        (Keyword: 'page'; Token: kwPAGE), 
        (Keyword: 'pages'; Token: kwPAGES), 
        (Keyword: 'parameter'; Token: kwPARAMETER), 
        (Keyword: 'password'; Token: kwPASSWORD), 
        (Keyword: 'plan'; Token: kwPLAN), 
        (Keyword: 'position'; Token: kwPOSITION), 
        (Keyword: 'post_event'; Token: kwPOST_EVENT), 
        (Keyword: 'precision'; Token: kwPRECISION), 
        (Keyword: 'primary'; Token: kwPRIMARY), 
        (Keyword: 'privileges'; Token: kwPRIVILEGES), 
        (Keyword: 'procedure'; Token: kwPROCEDURE), 
        (Keyword: 'program'; Token: kwPROGRAM), 
        (Keyword: 'protected'; Token: kwPROTECTED), 
        (Keyword: 'raw_partitions'; Token: kwRAW_PARTITIONS), 
        (Keyword: 'read'; Token: kwREAD), 
        (Keyword: 'real'; Token: kwREAL), 
        (Keyword: 'record_version'; Token: kwRECORD_VERSION), 
        (Keyword: 'references'; Token: kwREFERENCES),
        (Keyword: 'reserv'; Token: kwRESERV), 
        (Keyword: 'reserving'; Token: kwRESERVING),
        (Keyword: 'retain'; Token: kwRETAIN), 
        (Keyword: 'returning_values'; Token: kwRETURNING_VALUES), 
        (Keyword: 'returns'; Token: kwRETURNS), 
        (Keyword: 'revoke'; Token: kwREVOKE),
        (Keyword: 'right'; Token: kwRIGHT), 
        (Keyword: 'rollback'; Token: kwROLLBACK),
        (Keyword: 'rollforward'; Token: kwROLLFORWARD), 
        (Keyword: 'schema'; Token: kwSCHEMA), 
        (Keyword: 'second'; Token: kwSECOND), 
        (Keyword: 'segment'; Token: kwSEGMENT),
        (Keyword: 'select'; Token: kwSELECT), 
        (Keyword: 'set'; Token: kwSET),
        (Keyword: 'shadow'; Token: kwSHADOW), 
        (Keyword: 'shared'; Token: kwSHARED),
        (Keyword: 'show'; Token: kwSHOW), 
        (Keyword: 'singular'; Token: kwSINGULAR), 
        (Keyword: 'size'; Token: kwSIZE),
        (Keyword: 'smallint'; Token: kwSMALLINT), 
        (Keyword: 'snapshot'; Token: kwSNAPSHOT), 
        (Keyword: 'some'; Token: kwSOME), 
        (Keyword: 'sort'; Token: kwSORT), 
        (Keyword: 'sqlcode'; Token: kwSQLCODE), 
        (Keyword: 'stability'; Token: kwSTABILITY), 
        (Keyword: 'start'; Token: kwSTART), 
        (Keyword: 'starting'; Token: kwSTARTING), 
        (Keyword: 'starts'; Token: kwSTARTS), 
        (Keyword: 'statistics'; Token: kwSTATISTICS), 
        (Keyword: 'store'; Token: kwSTORE), 
        (Keyword: 'substring'; Token: kwSUBSTRING), 
        (Keyword: 'sub_type'; Token: kwSUB_TYPE), 
        (Keyword: 'sum'; Token: kwSUM), 
        (Keyword: 'suspend'; Token: kwSUSPEND), 
        (Keyword: 'sync'; Token: kwSYNC), 
        (Keyword: 'synonym'; Token: kwSYNONYM), 
        (Keyword: 'table'; Token: kwTABLE),
        (Keyword: 'then'; Token: kwTHEN),
        (Keyword: 'time'; Token: kwTIME),
        (Keyword: 'timestamp'; Token: kwTIMESTAMP), 
        (Keyword: 'timezone_hour'; Token: kwTIMEZONE_HOUR), 
        (Keyword: 'timezone_minute'; Token: kwTIMEZONE_MINUTE), 
        (Keyword: 'to'; Token: kwTO), 
        (Keyword: 'trailing'; Token: kwTRAILING), 
        (Keyword: 'transaction'; Token: kwTRANSACTION),
        (Keyword: 'trigger'; Token: kwTRIGGER), 
        (Keyword: 'trim'; Token: kwTRIM), 
        (Keyword: 'type'; Token: kwTYPE), 
        (Keyword: 'uncommitted'; Token: kwUNCOMMITTED),
        (Keyword: 'union'; Token: kwUNION), 
        (Keyword: 'unique'; Token: kwUNIQUE),
        (Keyword: 'update'; Token: kwUPDATE), 
        (Keyword: 'upper'; Token: kwUPPER),
        (Keyword: 'use'; Token: kwUSE),
        (Keyword: 'user'; Token: kwUSER),
        (Keyword: 'value'; Token: kwVALUE), 
        (Keyword: 'values'; Token: kwVALUES),
        (Keyword: 'varchar'; Token: kwVARCHAR), 
        (Keyword: 'variable'; Token: kwVARIABLE), 
        (Keyword: 'varying'; Token: kwVARYING), 
        (Keyword: 'view'; Token: kwVIEW), 
        (Keyword: 'wait'; Token: kwWAIT), 
        (Keyword: 'when'; Token: kwWHEN), 
        (Keyword: 'where'; Token: kwWHERE), 
        (Keyword: 'while'; Token: kwWHILE), 
        (Keyword: 'with'; Token: kwWITH), 
        (Keyword: 'work'; Token: kwWORK), 
        (Keyword: 'write'; Token: kwWRITE), 
        (Keyword: 'year'; Token: kwYEAR)
);

function GetKeywordToken(const S: String): Integer;

// checks whether the current token is a keyword,
// if so then the current token type is set accordingly

var L, R, K : Integer;
    Res     : Integer;

begin
  Result := 0;

  // binary search
  L := 0;
  R := KeywordCount - 1;
  while L <= R do
  begin
    K := L + (R - L) div 2;
    Res := CompareText(S, Keywords[K].Keyword);
    if Res = 0 then
    begin
       Result := Keywords[K].Token;
      Break;
    end
    else
      if Res > 0 then L := K + 1
                 else R := K - 1;
  end;
end;
 
//--------------------------------------------------------------------------------
 
constructor TUCESQLHighlighter.Create(AOwner: TComponent);
 
begin
  inherited;
  // allocate positions array (length is highest possible mark position)
  SetLength(FPositions, MarksHigh[StateCount - 1] + 1);
 
  FCommentAttributes := THighlightAttributes.Create('comment');
  FCommentAttributes.Style:= [fsItalic];
  FIdentifierAttributes := THighlightAttributes.Create('identifier');
  FKeyAttributes := THighlightAttributes.Create('reserved word');
  FKeyAttributes.Style:= [fsBold];
  FNumberAttributes := THighlightAttributes.Create('number');
  FSpaceAttributes := THighlightAttributes.Create('space');
  FStringAttributes := THighlightAttributes.Create('String');
  FSymbolAttributes := THighlightAttributes.Create('symbol');
  FCommentAttributes.Onchange:= HighlightChange;
  FIdentifierAttributes.Onchange:= HighlightChange;
  FKeyAttributes.Onchange:= HighlightChange;
  FNumberAttributes.Onchange:= HighlightChange;
  FSpaceAttributes.Onchange:= HighlightChange;
  FStringAttributes.Onchange:= HighlightChange;
  FSymbolAttributes.Onchange:= HighlightChange;
  DefaultFilter := 'SQL script files (*.sql)|*.sql';
end;
 
//-----------------------------------------------------------------------------
 
destructor TUCESQLHighlighter.Destroy;
 
begin
  FPositions := nil;
  FMatchStack := nil;
  FCommentAttributes.Free;
  FIdentifierAttributes.Free;
  FKeyAttributes.Free;
  FNumberAttributes.Free;
  FSpaceAttributes.Free;
  FStringAttributes.Free;
  FSymbolAttributes.Free;
  inherited;
end;
 
//-----------------------------------------------------------------------------
 
function TUCESQLHighlighter.IsKeyword: Boolean;
 
// checks whether the current token is a keyword,
// if so then the current token type is set accordingly
 
var Word: String;
 
begin
  SetString(Word, FInput + FTokenPos, FCurrentPos - FTokenPos);
  FCurrentToken := GetKeywordToken(Word);
  if FCurrentToken > 0 then
  begin
    Result := True;
    FDoStop := True;
  end
  else Result := False;
end;
 
//-----------------------------------------------------------------------------
 
function TUCESQLHighlighter.EOL: Boolean;
 
begin
  Result := (FCurrentToken = -1) or FEOL;
end;
 
//-----------------------------------------------------------------------------
 
function TUCESQLHighlighter.GetToken: String;
 
var Len: Cardinal;
 
begin
  Len := FCurrentPos - FTokenPos;
  SetString(Result, FInput + FTokenPos, Len);
end;
 
//-----------------------------------------------------------------------------
 
procedure TUCESQLHighlighter.Reset;
 
begin
  FTokenPos := 0;
  FCurrentPos := 0;
  FCurrentToken := 0;
  FLineState := 1;
  FLastChar := #0;
  FMatchCount := 0;
  FEOL := False;
end;
 
//-----------------------------------------------------------------------------
 
procedure TUCESQLHighlighter.SetToken(Token: Integer);
 
begin
  FCurrentToken := Token;
  FDoStop := True;
end;
 
//-----------------------------------------------------------------------------
 
procedure TUCESQLHighlighter.RuleToToken(Rule: Integer);
 
var S : String;
 
begin
  SetString(S, FInput + FTokenPos, FCurrentPos - FTokenPos);
  // actions
  case Rule of
    // <Normal>integer
         0 : SetToken(INTEGERNUMBER);
    // <Normal>realinteger
         1 : SetToken(FLOATNUMBER);
    // <Normal>white+
         2 : SetToken(WHITESPACE);
    // <Normal>identifier
         3 : if not IsKeyword then SetToken(IDENTIFIER);
    // <Normal>'
         4 : repeat
                        case CurrentChar of
                          ''''   : begin
                                     NextChar;
                                     if CurrentChar <> '''' then
                                     begin
                                       SetToken(STRINGCONSTANT);
                                       Break;
                                     end;
                                   end;
                          CR, #0 : begin
                                     SetToken(STRINGCONSTANT);
                                     Break;
                                   end;
                        end;
                        NextChar;
                      until False;
    // <Normal>\"
         5 : repeat
                        case CurrentChar of
                          '"', '`',
                          CR, #0 : begin
                                     if (CurrentChar = '"') or (CurrentChar = '`') then NextChar;
                                     SetToken(STRINGCONSTANT);
                                     Break;
                                   end;
                        end;
                        NextChar;
                      until False;
    // <Normal>"{"
         6 : repeat
                        case CurrentChar of
                          '}',
                          #0 : begin
                                 if CurrentChar = '}' then NextChar
                                                      else State := Comment1;
                                 SetToken(MLCOMMENT);
                                 Break;
                               end;
                        end;
                        NextChar;
                      until False;
    // <Comment1>[^}]*
         7 : SetToken(MLCOMMENT);
    // <Comment1>"}"
         8 : begin
                        SetToken(MLCOMMENT);
                        State := Normal;
                      end;
    // <Normal>"//"
         9 : repeat
                        case CurrentChar of
                          CR, #0 : begin
                                     SetToken(SLCOMMENT);
                                     Break;
                                   end;
                        end;
                        NextChar;
                      until False;
    // <Normal>"/*"
        10 : repeat
                        case CurrentChar of
                          '*'    : if Lookahead = '/' then
                                   begin
                                     // skip lookahead and break loop
                                     NextChar;
                                     NextChar;
                                     SetToken(MLCOMMENT);
                                     Break;
                                   end;
                          #0 : begin
                                 State := Comment2;
                                 SetToken(MLCOMMENT);
                                 Break;
                               end;
                        end;
                        NextChar;
                      until False;
    // <Comment2>[^\*]*
        11 : begin
                        SetToken(MLCOMMENT);
                      end;
    // <Comment2>\*
        12 : begin
               SetToken(MLCOMMENT);
               if CurrentChar = '/' then
                      begin
                        NextChar;
                        State := Normal;
                      end;
             end;
    // <Normal>.
        13 : SetToken(SYMBOL);
    // eof
        14 : SetToken(-1); // special token to mark input end
 
  end;
end;
 
//-----------------------------------------------------------------------------
 
function TUCESQLHighlighter.MatchRule(var Rule : Integer ) : Boolean;
 
// finds the last match and the corresponding marked position and adjusts
// the matched string accordingly;
// returns:
// - True if a rule has been matched, False otherwise
// - Rule: the number of the matched rule
 
begin
  FRejected := False;
  while (FMatchCount > 0) and (FPositions[FMatchStack[FMatchCount - 1]] = 0) do Dec(FMatchCount);
 
  if FMatchCount > 0 then
  begin
    Rule := FMatchStack[FMatchCount - 1];
    FCurrentPos := FTokenPos + FPositions[Rule];
    FPositions[Rule] := 0;
    if (FCurrentPos - FTokenPos) > 0 then FLastChar := FInput[FCurrentPos]
                                     else FLastChar := #0;
    Result := True;
  end
  else
  begin
    // return position to starting point
    FCurrentPos := FTokenPos;
    FLastChar := #0;
    Result := False;
  end
end;
 
//-----------------------------------------------------------------------------
 
function TUCESQLHighlighter.DoDefault : Boolean;
 
begin
  FRejected := False;
  if NextChar <> #0 then Result := True
                    else
  begin
    FLineState := 1;
    Result := False;
  end;
  if not FEOL then FLastChar := FInput[FCurrentPos];
end;
 
//-----------------------------------------------------------------------------
 
procedure TUCESQLHighlighter.Next;
 
// scans for the next token and determines its kind (FCurrentToken and FTokenPos
// are set appropriately)
 
var I : Integer;
    Len : Cardinal;
    Matched : Boolean;
 
begin
  FDoStop := False;

  repeat
    // initialize run
    FTokenPos := FCurrentPos;
 
    if FLastChar <> #0 then
      if FLastChar = LF then FLineState := 1
                        else FLineState := 0;
    FCurrentState := FStartState + FLineState;
 
    repeat // character scan loop
      // mark positions and matches
      Len := FCurrentPos - FTokenPos;
      for I := MarksLow[FCurrentState] to MarksHigh[FCurrentState] do FPositions[MarkPositionTable[I]] := Len;
      for I := MatchesHigh[FCurrentState] downto MatchesLow[FCurrentState] do SetMatch(MatchTable[I]);
 
      if TransitionsLow[FCurrentState] > TransitionsHigh[FCurrentState] then Break;  // dead state, do action
 
      // determine action
      I := TransitionsLow[FCurrentState];
      while (I <= TransitionsHigh[FCurrentState]) and not (FInput[FCurrentPos] in TransitionTable[I].CharClass) do Inc(I);
 
      if I > TransitionsHigh[FCurrentState] then Break; // no transition on current char in this state
 
      // get next character
      NextChar;
 
      // switch to new state
      FCurrentState := TransitionTable[I].NextState;
 
    until False;
 
    repeat
      Matched := MatchRule(I);
      if Matched then RuleToToken(I)
                 else Break;
      // FRejected can be set in RuleToToken by calling Reject
    until Matched and not FRejected;
 
    if not (Matched or DoDefault) then
    begin
      FStartState := Normal;
      FLineState := 1;
      FLastChar := #0;
      SetToken(0);
    end;
  until FDoStop;
end;
 
//-----------------------------------------------------------------------------
 
procedure TUCESQLHighlighter.SetLine(const NewValue: String);
 
begin
  FInput := PChar(NewValue);
  Reset;
  Next;
end;
 
//-----------------------------------------------------------------------------
 
function TUCESQLHighlighter.GetNextChar: Char;
 
begin
  if not FEOL and (FInput[FCurrentPos] = #0) then FEOL := True;
  Inc(FCurrentPos);
  if not FEOL then
  begin
    FLookahead := FInput[FCurrentPos + 1];
    Result := FInput[FCurrentPos];
  end
  else Result := #0;
end;
 
//-----------------------------------------------------------------------------
 
procedure TUCESQLHighlighter.SetState(Value: Cardinal);
 
begin
  FStartState := Value;
end;
 
//-----------------------------------------------------------------------------
 
procedure TUCESQLHighlighter.SetMatch(I: Integer);
 
begin
  if FMatchCount >= Length(FMatchStack) then SetLength(FMatchStack, FMatchCount + 1);
  FMatchStack[FMatchCount] := I;
  Inc(FMatchCount);
end;
 
//-----------------------------------------------------------------------------
 
procedure TUCESQLHighlighter.Reject;
 
begin
  FRejected := True;
end;
 
//-----------------------------------------------------------------------------
 
function TUCESQLHighlighter.GetTokenInfo: TTokenData;
 
// returns token string, token position relative to entire input string, and token length in one call
 
begin
  with Result do
  begin
    if FEOL then
    begin
      Token := StrEnd(FInput);
      Length := Token - FInput - Integer(FTokenPos);
    end
    else
    begin
      Token := FInput + FTokenPos;
      Length := FCurrentPos - FTokenPos;
    end;
    TokenType := FCurrentToken;
    Position := FTokenPos;

    case FCurrentToken of
      SLCOMMENT,
      MLCOMMENT:
        begin
          Background := ColorToRGB(FCommentAttributes.Background);
          Foreground := ColorToRGB(FCommentAttributes.Foreground);
          Style := FCommentAttributes.Style;
        end;
      IDENTIFIER:
        begin
          Background := ColorToRGB(FIdentifierAttributes.Background);
          Foreground := ColorToRGB(FIdentifierAttributes.Foreground);
          Style := FIdentifierAttributes.Style;
        end;
      257..489, 1000:
        begin
          Background := ColorToRGB(FKeyAttributes.Background);
          Foreground := ColorToRGB(FKeyAttributes.Foreground);
          Style := FKeyAttributes.Style;
        end;
      FLOATNUMBER,
      INTEGERNUMBER:
        begin
          Background := ColorToRGB(FNumberAttributes.Background);
          Foreground := ColorToRGB(FNumberAttributes.Foreground);
          Style := FNumberAttributes.Style;
        end;
      WHITESPACE:
        begin
          Background := ColorToRGB(FSpaceAttributes.Background);
          Foreground := ColorToRGB(FSpaceAttributes.Foreground);
          Style := FSpaceAttributes.Style;
        end;
      STRINGCONSTANT:
        begin
          Background := ColorToRGB(FStringAttributes.Background);
          Foreground := ColorToRGB(FStringAttributes.Foreground);
          Style := FStringAttributes.Style;
        end;
      SYMBOL:
        begin
          Background := ColorToRGB(FSymbolAttributes.Background);
          Foreground := ColorToRGB(FSymbolAttributes.Foreground);
          Style := FSymbolAttributes.Style;
        end;
    end;
  end;
end;
 
//-----------------------------------------------------------------------------
 
function TUCESQLHighlighter.GetCurrentChar: Char;
 
begin
  Result := FInput[FCurrentPos];
end;
 
//-----------------------------------------------------------------------------
 
procedure TUCESQLHighlighter.HighlightChange(Sender:TObject);
 
begin
  WindowList.Invalidate;
end;
 
//--------------------------------------------------------------------------------
 
function TUCESQLHighlighter.GetIdentChars: TIdentChars;
 
begin
  Result := ['_', '0'..'9', 'a'..'z', 'A'..'Z'];
end;
 
//--------------------------------------------------------------------------------
 
function TUCESQLHighlighter.GetAttributeCount: Integer;
 
begin
  Result := 7;
end;
 
//--------------------------------------------------------------------------------
 
function TUCESQLHighlighter.GetAttribute(Index: Integer): THighlightAttributes;
 
begin
  case Index of
    0: Result := FCommentAttributes;
    1: Result := FIdentifierAttributes;
    2: Result := FNumberAttributes;
    3: Result := FKeyAttributes;
    4: Result := FSpaceAttributes;
    5: Result := FStringAttributes;
    6: Result := FSymbolAttributes;
    else Result := nil;
  end;
end;
 
//--------------------------------------------------------------------------------
 
function TUCESQLHighlighter.GetLanguageName: String;
 
begin
  Result := 'SQL';
end;
 
//--------------------------------------------------------------------------------
 
procedure TUCESQLHighlighter.SetAttribute(const Index: Integer; const Value: THighlightAttributes);
 
begin
  case Index of
    0 : FCommentAttributes.Assign(Value);
    1 : FIdentifierAttributes.Assign(Value);
    2 : FNumberAttributes.Assign(Value);
    3 : FKeyAttributes.Assign(Value);
    4 : FSpaceAttributes.Assign(Value);
    5 : FStringAttributes.Assign(Value);
    6 : FSymbolAttributes.Assign(Value);
  end;
end;
 
//--------------------------------------------------------------------------------
 
function TUCESQLHighlighter.GetRange: Pointer;
 
begin
  Result := Pointer(FStartState);
end;
 
//--------------------------------------------------------------------------------
 
procedure TUCESQLHighlighter.SetRange(Value: Pointer);
 
begin
  FStartState := Integer(Value);
end;
 
//--------------------------------------------------------------------------------
 
procedure TUCESQLHighlighter.ResetRange;
 
begin
  FStartState := Normal;
  // in case someone will ask for immediately afterwards
  FCurrentState := Normal;
end;
 
//--------------------------------------------------------------------------------
 
end.
 
