unit UniCodeEditorReg;

//----------------------------------------------------------------------------------------------------------------------
//
// UniCodeEditor, a Unicode Source Code Editor for Delphi.
// Copyright (c) 1999, 2000, 2004 by Dipl. Ing. Mike Lischke (public@lischke-online.de)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//----------------------------------------------------------------------------------------------------------------------
//
// Description
//   Registration unit for the UniCodeEditor control.
//
// Changes:
//   Version 2.0, 2003-08-17, Mike Zinner
//     Repackaged for D7
//   Version 1.0, 1999-03-10, Mike Lischke
//     Initial Version
//
//----------------------------------------------------------------------------------------------------------------------

interface

procedure Register;

//----------------------------------------------------------------------------------------------------------------------

implementation

uses
  DesignIntf, DesignWindows, DesignEditors, ComponentDesigner,
  VCLEditors,
  Dialogs, Forms, Graphics, Classes,
  UCEEditorKeyCommands,
  UCEKeyCommandsEditor,
  UCEDelphiHighlighter,
  UCECPPHighlighter,
  UCESQLHighlighter,
  UCEHTMLHighlighter,
  UCEDCGHighlighter,
  UniCodeEditor,
  UniCodeConsole;

type
  TSyntaxEditorFontProperty = class(TFontProperty)
  public
    procedure Edit; override;
  end;

//----------------------------------------------------------------------------------------------------------------------

procedure TSyntaxEditorFontProperty.Edit;

const
  // context ids for the Font editor 
  hcDFontEditor = 25000;

var FontDialog: TFontDialog;

begin
  FontDialog := TFontDialog.Create(Application);
  try
    FontDialog.Font := TFont(GetOrdValue);
    FontDialog.HelpContext := hcDFontEditor;
    FontDialog.Options := FontDialog.Options + [fdShowHelp, fdForceFontExist{, fdFixedPitchOnly}];
    if FontDialog.Execute then SetOrdValue(Longint(FontDialog.Font));
  finally
    FontDialog.Free;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

type
  TSyntaxEditorCommandProperty = class(TIntegerProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: String; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: String); override;
  end;

//----------------- TSyntaxEditorCommandProperty -----------------------------------------------------------------------

procedure TSyntaxEditorCommandProperty.Edit;

begin
  ShowMessage('I''m thinking that this will show a dialog that has a list'#13#10+
              'of all editor commands and a description of them to choose from.');
end;

//----------------------------------------------------------------------------------------------------------------------

function TSyntaxEditorCommandProperty.GetAttributes: TPropertyAttributes;

begin
  Result := [paMultiSelect, paDialog, paValueList, paRevertable];
end;

//----------------------------------------------------------------------------------------------------------------------

function TSyntaxEditorCommandProperty.GetValue: String;

begin
  Result := EditorCommandToCodeString(TEditorCommand(GetOrdValue));
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TSyntaxEditorCommandProperty.GetValues(Proc: TGetStrProc);

begin
  GetEditorCommandValues(Proc);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TSyntaxEditorCommandProperty.SetValue(const Value: String);

var NewValue: Longint;

begin
  if IdentToEditorCommand(Value, NewValue) then SetOrdValue(NewValue)
                                           else inherited SetValue(Value);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure Register;

begin
  RegisterComponents('UniCodeEditor', [TUniCodeEdit, TUniCodeConsole,
    TUCEDelphiHighlighter, TUCECPPHighlighter, TUCEHTMLHighlighter,
    TUCESQLHighlighter, TUCEDCGHighlighter]);
  RegisterPropertyEditor(TypeInfo(TFont), TUniCodeEdit, 'Font', TSyntaxEditorFontProperty);
  RegisterPropertyEditor(TypeInfo(TEditorCommand), nil, 'Command', TSyntaxEditorCommandProperty);
  RegisterPropertyEditor(TypeInfo(TKeystrokes), nil, '', TKeyStrokesProperty);
end;

//--------------------------------------------------------------------------------

end.
